/* $Id: BezierNode.hh,v 1.9 2004/07/21 06:31:20 pklaus Exp $
 *
 * BezierNode.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef _BEZIERNODE_HH
#  define _BEZIERNODE_HH

#include "gestalter.h"
#include <glib--.h>
#include "config.h"
#include <gdk--.h>
#include "DLList.hh"
#include "SelectableItem.hh"
#include "TransformationMatrix.hh"
#include "DocWindow.hh"

#define ROUND(x)	((gint32) ((x) + 0.5))
#define	BEZIER_RADIUS	2
#define BEZIER_WIDTH	5
#define BEZIER_SIZE		5
#define BEZIER_HALFSIZE	2
#define SUBDIVISIONS	100
#define POINT_TOLERANCE	3

class BezierPath;

typedef gdouble BezierMatrix[4][4];


class BezierNode : public SelectableItem, public DLList<BezierNode> {

protected:
	static void matrixMultiply(BezierMatrix a, BezierMatrix b, BezierMatrix ab);
	

public:
	position onCurve;
	position prevHandle;
	position nextHandle;
	
	gboolean straightHandles;
	
	gint selection;
	gint selectedItem;
	
	gdouble curveDivision;
	
	BezierPath *path;

	BezierNode(BezierPath *parent);
	
	gint32 calcSegment(BezierNode *b, int subdivisions, GdkPoint *gdkpts, 
					gdouble zoom, gdouble xoffset, gdouble yoffset);
	void calcSegment(BezierNode *b, int subdivisions, position *gdkpts);
	IntersectionVector *getIntersections(CurveSegment& a, CurveSegment& b);

	void drawCurve(DocWindow *docwin, Gdk_GC& context);
	void drawSegment(DocWindow *docwin, Gdk_GC& context);
	void drawSegments(DocWindow *docwin, Gdk_GC& context);
	
	void setOnCurvePoint(gdouble x, gdouble y);
	void setPreviousHandle(gdouble x, gdouble y);
	void setNextHandle(gdouble x, gdouble y);
	
	void drawSingleNode(DocWindow *docwin, Gdk_GC& context);
	void drawPoint(DocWindow *docwin, Gdk_GC& context);
	void drawPreviousHandle(DocWindow *docwin, Gdk_GC& context);
	void drawNextHandle(DocWindow *docwin, Gdk_GC& context);

	void bufferDrawPoint(OffscreenBuffer& osBuffer, guint32 color);
	void bufferDrawPreviousHandle(OffscreenBuffer& osBuffer, guint32 color);
	void bufferDrawNextHandle(OffscreenBuffer& osBuffer, guint32 color);
	
	gboolean testPreviousHandleVisible();
	gboolean testNextHandleVisible();
	
	void connectNodes(BezierNode *to);
	
	// SelectableItem methods
	void drawSelection(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea);
	void transformByMatrix(TransformationMatrix& m, gboolean incFillPattern = TRUE);
	void setFillMatrix(TransformationMatrix& m) {};
	SelectableItem *clone();
	void setCoords(SelectableItem* from);
	BezierNode *prevNode() { return prev; };
	BezierNode *nextNode() { return next; };
	GObject *getGObject() { return (GObject *) path; };
	void clear();
	gboolean selected();
	void select(gboolean shiftSelect);
	void deSelect();
	void selectPath(gboolean shiftSelect);
	void deSelectPath();
	void selectObject(gboolean shiftSelect);
	void deSelectObject();
	int getCoordinates(gdouble& x, gdouble& y);
	gchar *identify();
	BoundingBox getBoundingBox();
	
	gboolean checkRectangle(gdouble left, gdouble top, gdouble right, gdouble bottom);
	gint checkPoint(gdouble x, gdouble y, gdouble tolerance);
	gint checkHandles(gdouble x, gdouble y, gdouble tolerance);
	void boundingBox(gdouble& l, gdouble& t, gdouble& r, gdouble& b);
	void select();
	void selectCurve();
	
	void reverseOrder();
	
	BezierNode *subDivide(gdouble t);
	
	~BezierNode();
};

#endif
