/* $Id: ClipGroup.hh,v 1.13 2004/07/21 06:31:20 pklaus Exp $
 *
 * ClipGroup.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef _CLIPGROUP_HH
#  define _CLIPGROUP_HH

#include "ObjectGroup.hh"
#include <glib--.h>
#include <gdk--.h>
#include <gtk--.h>
#include <libart_lgpl/art_vpath.h>

class BezierNode;
class BoundingBox;
class SelectionItem;
class GDDocument;
class GDStyle;
class SelectableItem;

class ClipGroup : public ObjectGroup {


public:
	GObject *clipparts;


	ClipGroup(GDDocument *document);
	
	void addClipPart(GObject *newObject);
	void removeClipPart(GObject *object);

	// DrawObject methods
	void drawWithSelection(OffscreenBuffer& osBuffer, RectangleArea& drawArea);
	void drawWireFrameSelection(DocWindow *docwin, RectangleArea& drawArea);
	void gatherResource(GDResourceList& collection, gint type);
	
	// GObject methods
	void draw(OffscreenBuffer& osBuffer, RectangleArea& drawArea);
	void drawBlack(OffscreenBuffer& osBuffer, RectangleArea& drawArea);
	void fillBlack(OffscreenBuffer& osBuffer, RectangleArea& drawArea);
	void drawWireFrame(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea);
	void drawPoints(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea);
	BoundingBox getBoundingBox();
	gdouble attractPoint(gdouble x, gdouble y, gdouble& ax, gdouble& ay, gdouble tolerance, int attractType = 0);
	SelectableItem *checkPoints(gdouble x, gdouble y, gdouble tolerance, gboolean onlyCurvePoints);
	GObject *checkArea(gdouble x, gdouble y);
	SelectableItem *checkCurves(gdouble x, gdouble y, gdouble tolerance);
	void selectRectangle(gdouble left, gdouble top, gdouble right, gdouble bottom, gint what);
	void deSelectRectangle(gdouble left, gdouble top, gdouble right, gdouble bottom, gint what);
	void selectAll();
	void deSelectAll();
	GObject *findObject(GObject *object);
	GObject *removeObject(GObject *object);
	gint getSelection(SelectionItemList& selectionList, gint mode);
	gint writeXML(ostream& xmlout);
	void writePSPathDescription(ostream& psout);
	gint writePS(ostream& psout);
	void reverseOrder();

	// SelectableItem methods
	void drawSelection(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea);
	void transformByMatrix(TransformationMatrix& m, gboolean incFillPattern = TRUE);
	SelectableItem *clone();
	void setCoords(SelectableItem* from);
	void setDocument(GDDocument *document);
	gboolean selected();
	void clear();
	gchar *identify();

	~ClipGroup();
};

#endif
