/* $Id: DocWindow.hh,v 1.9 2004/07/21 06:31:20 pklaus Exp $
 *
 * DocWindow.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



// generated 2001/11/11 16:37:03 CET by pklaus@iron.
// using glademm V0.6.2
//
// newer (non customized) versions of this file go to DocWindow.hh_new

// you might replace
//    class foo : public foo_glade { ... };
// by
//    typedef foo_glade foo;
// if you didn't make any modifications to the widget

#ifndef _DOCWINDOW_HH
#  include "DocWindow_glade.hh"
#  define _DOCWINDOW_HH
#include "config.h"
#include "glademm_support.hh"
#include "TransformationMatrix.hh"
#include "RectangleArea.hh"
#include "gdk--.h"

class GDDocument;

#define DOCUMENT_SIZE_X		6000
#define DOCUMENT_SIZE_Y		4000

class OffscreenBuffer {
public:
	gdouble scaleFactor;
	position origin;  // position of the upper left corner of the window
	
	size_t pxbRowstride;
	size_t pxbSize;
	guchar *pixbuffer;
	guint pxbHeight;
	guint pxbWidth;

	OffscreenBuffer() { pxbRowstride = 0; pxbSize = 0; pxbHeight = 0; pxbWidth = 0; pixbuffer = NULL; };
	void allocBuffer(uint rowstride, uint width, uint height);
	void whiteBackground();
	~OffscreenBuffer() { if (pixbuffer) free(pixbuffer); };
};

class DocWindow : public DocWindow_glade {
	SigC::Connection rulerUnitSelectConnection;
	SigC::Connection rulerUnitCancelConnection;
	
public:
	GDDocument *doc;

	OffscreenBuffer osBuffer;
	
	position motionPosition;

	gdouble rulerUnitFactor;
	
	gboolean doScrollRedraw;
	
	gboolean displayWireFrame;
	gboolean needRedraw;
	
	
	DocWindow(GDDocument *myDoc);
    gint onDocWindowMenuPopup(GdkEventButton *ev);
	gint onVRulerButtonPress(GdkEventButton *ev);
	gint onHRulerButtonPress(GdkEventButton *ev);
	void rulerUnitsChanged();
	void rulerUnitsCancel();
	void updateScaling();
	void allocPixBuffer();
	void drawPixmap();
	void drawPixmap(GdkRectangle *area);
	void whiteBackground();

	guint getWidth() { return DocWinDrawingArea->width(); };
	guint getHeight() { return DocWinDrawingArea->height(); };
	
	RectangleArea getContentRectangle();
	
	void setWindowTitle(string title);
	
private: 
		gboolean releaseTool;
		
        friend class DocWindow_glade;
        gint buttonDown(GdkEventButton *ev);
        gint motionEvent(GdkEventMotion *ev);
        gint button_up(GdkEventButton *ev);
        gint resizedEvent(GdkEventConfigure *ev);
        gint deleteEvent(GdkEventAny *ev);
		gint drawingConfigureEvent(GdkEventConfigure *ev);
        gint keyPressEvent(GdkEventKey *ev);
        gint keyReleaseEvent(GdkEventKey *ev);
        gint onExposeEvent(GdkEventExpose *ev);
        gint onDocWindowFocus(GdkEventFocus *ev);
        gint onDocWindowFocusOut(GdkEventFocus *ev);
		void updateScrollbarRanges();
		void updateRulerRanges();
		void scrollUpdate();
		
		static const int sizeX = DOCUMENT_SIZE_X;
		static const int sizeY = DOCUMENT_SIZE_Y;
};
#endif
