/* $Id: FillTransTool.cc,v 1.2 2004/03/20 00:01:37 pklaus Exp $
 *
 * FillTransTool.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <math.h>
#include "../pixmaps/filltrans.xpm"
#include "../cursors/crosshair.xbm"
#include "../cursors/crosshair_mask.xbm"
#include "../cursors/move.xbm"
#include "../cursors/move_mask.xbm"
#include "../cursors/move_magnet.xbm"
#include "../cursors/move_magnet_mask.xbm"
#include "FillTransTool.hh"
#include "GDDocument.hh"
#include "DocWindow.hh"
#include "CursorFactory.hh"
#include "PatternTransAction.hh"
#include "DimensionDialog.hh"
#include "SelectionItemList.hh"
#include "TransformationMatrix.hh"

#define doubleMAX(a,b)	(fabs(a) > fabs(b)) ? fabs(a) : fabs(b)

FillTransTool::FillTransTool() {
	pixmapData = filltrans_xpm;
	toolName = "FillTransTool";
	moving = false;
	nodeCaught = false;
	CursorFactory::createCursorFromData(crosshairCursor,
					crosshair_bits, crosshair_mask_bits,
					crosshair_width, crosshair_height,
					crosshair_x_hot, crosshair_y_hot);
	CursorFactory::createCursorFromData(moveCursor,
					move_bits, move_mask_bits,
					move_width, move_height,
					move_x_hot, move_y_hot);
	CursorFactory::createCursorFromData(moveMagnetCursor,
					move_magnet_bits, move_magnet_mask_bits,
					move_magnet_width, move_magnet_height,
					move_magnet_x_hot, move_magnet_y_hot);
}

void FillTransTool::activate() {
	Tool::activate();
}

void FillTransTool::deactivate() {
	Tool::deactivate();
}

gint FillTransTool::buttonDown(GdkEventButton *ev, DocWindow *docwin) {
	GDDocument	*doc;
	downButton = ev->button;

	if (downButton == 1) {
		doc = docwin->doc;
		sx = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
		sy = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
		gboolean caught = magnet(sx, sy, docwin);
		startx = (gint) ev->x;
		starty = (gint) ev->y;
		oldx = startx;
		oldy = starty;
		moving = true;
	} else {
		Tool::buttonDown(ev, docwin);
	}
	return 0;
}

gint FillTransTool::motion(GdkEventMotion *ev, DocWindow *docwin) {
	if (ev->state & GDK_BUTTON1_MASK) {
		GDDocument *doc = docwin->doc;
		Gdk_Window drawable = docwin->DocWinDrawingArea->get_window();
		doc->context.set_function(GDK_INVERT);
		gdouble dx = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
		gdouble dy = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
		nodeCaught = magnet(dx, dy, docwin);
		setCursor(ev->state, docwin);
		if (ev->state & GDK_SHIFT_MASK) {
			// square
			gdouble delta = doubleMAX(dx-sx, dy-sy);
			dx = sx + copysign(delta, dx-sx);
			dy = sy + copysign(delta, dy-sy);
		}
		gint x = (gint) ((dx - docwin->osBuffer.origin.x) * docwin->osBuffer.scaleFactor);
		gint y = (gint) ((dy - docwin->osBuffer.origin.y) * docwin->osBuffer.scaleFactor);
		drawable.draw_line(doc->context, startx, starty, oldx, oldy);
		drawable.draw_line(doc->context, startx, starty, x, y);
		oldx = x;
		oldy = y;
	} else if (ev->state & (GDK_BUTTON2_MASK | GDK_BUTTON3_MASK)) {
		Tool::motion(ev, docwin);
	} else {
		nodeCaught = false;
		setCursor(ev->state, docwin);
	}
	
  return 0;
}

gint FillTransTool::buttonUp(GdkEventButton *ev, DocWindow *docwin) {
	if (downButton == 1) {
		GDDocument *doc = docwin->doc;
		Gdk_Window drawable = docwin->DocWinDrawingArea->get_window();
		gdouble dx = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
		gdouble dy = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
		gboolean caught = magnet(dx, dy, docwin);
		if (ev->state & GDK_SHIFT_MASK) {
			// square
			gdouble delta = doubleMAX(dx-sx, dy-sy);
			dx = sx + copysign(delta, dx-sx);
			dy = sy + copysign(delta, dy-sy);
		}
		gdouble w = dx - sx;
		gdouble h = dy - sy;
		if (!(w == 0 && h == 0)) {
			TransformationMatrix tmp, fac;
			gdouble gamma = tmp.getAngle(w,h);
			gdouble scale = sqrt(w * w + h * h);
			tmp.setScale(scale, scale);
			fac.setRotation(gamma);
			tmp *= fac;
			fac.reset();
			fac.setTranslation(sx, sy);
			tmp *= fac;
			PatternTransAction *newAction = new PatternTransAction(tmp);
			doc->setAction(newAction);
			SelectionItemList selection;
			doc->getSelection(selection, SELECTIONITEM_MODE_ORIG_ONLY);
			selection.setFillMatrix(tmp);
			// redraw the document
			doc->context.set_function(GDK_SET);
			doc->redraw();
		}
		moving = false;
	} else {
		Tool::buttonUp(ev, docwin);
	}
	return 0;
}

void FillTransTool::setCursor(guint state, DocWindow *docwin) {
	if (moving) {
		if (nodeCaught) {
			// set the cursor
			docwin->DocWinDrawingArea->get_window().set_cursor(moveMagnetCursor);
		} else {
			// set the cursor
			docwin->DocWinDrawingArea->get_window().set_cursor(moveCursor);
		}
	} else {
		if (state & GDK_MOD1_MASK) {
			docwin->DocWinDrawingArea->get_window().set_cursor(crosshairCursor);
		} else {
			docwin->DocWinDrawingArea->get_window().set_cursor(crosshairCursor);
		}
	}
}
