/* $Id: FlipTool.cc,v 1.3 2003/09/18 20:38:33 pklaus Exp $
 *
 * FlipTool.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "FlipTool.hh"
#include <math.h>

FlipTool::FlipTool() {
	pixmapData = flip_xpm;
	toolName = "FlipTool";
	valueDialog = NULL;
}

TransformationMatrix FlipTool::calcMatrix(gdouble sx, gdouble sy, 
						gdouble cx, gdouble cy,
						gdouble x, gdouble y, gint state) {

	/* flipping by a free axis is the combination of a translation
	   followed by a rotation to get the axis on one of the base axis.
	   Then flip at that base axis (simply by changing the sign of that
	   axis) and do everything back in reverse order (first reverse
	   rotation, than reverse translation).
	*/
	
	int i = 0;
	double gamma = 0;
	gdouble dx = x - cx;
	gdouble dy = y - cy;
	if (fabs(dx) > fabs(dy)) {
		gamma = atan(dy/dx);
		i = 1;
	} else {
		gamma = -atan(dx/dy);
		i = 0;
	}
	//g_printerr("FlipTool::calcMatrix dx:%f dy:%f gamma:%f axis:%c\n", dx, dy, gamma, (i>0) ? 'y' : 'x');
	
	TransformationMatrix tm;
	tm.setTranslation(-cx, -cy);
	TransformationMatrix temp;
	temp.setRotation(-gamma);
	tm *= temp;
	temp.reset();
	temp.m[i][i] = -1;
	tm *= temp;
	temp.reset();
	temp.setRotation(gamma);
	tm *= temp;
	temp.reset();
	temp.setTranslation(cx, cy);
	tm *= temp;
	return tm;
}

TransformationMatrix FlipTool::setMatrix(gdouble angle) {
	TransformationMatrix tm;
	tm.setTranslation(-cx, -cy);
	TransformationMatrix temp;
	temp.setRotation(-angle);
	tm *= temp;
	temp.reset();
	int i = 1;
	temp.m[i][i] = -1;
	tm *= temp;
	temp.reset();
	temp.setRotation(angle);
	tm *= temp;
	temp.reset();
	temp.setTranslation(cx, cy);
	tm *= temp;
	return tm;
}

gint FlipTool::deleteValueDialog(GdkEventAny *ev) {
	cancelValueDialog();
	return 1;
}

void FlipTool::cancelValueDialog() {
	valueDialog->hide();
	mode = 0;
}

void FlipTool::transformObjects() {
	gdouble angle = valueDialog->getAngle();
	TransformationMatrix tm = setMatrix(angle);
	doTransform(tm, false);
	valueDialog->hide();
}

void FlipTool::copyObjects() {
	gdouble angle = valueDialog->getAngle();
	TransformationMatrix tm = setMatrix(angle);
	doTransform(tm, true);
	valueDialog->hide();
}

void FlipTool::showValueDialog() {
	if (! valueDialog) {
		valueDialog = new FlipDialog();
		valueDialog->getOKButton()->clicked.connect(SigC::slot(this, &FlipTool::transformObjects));
		valueDialog->getCopyButton()->clicked.connect(SigC::slot(this, &FlipTool::copyObjects));
		valueDialog->getCancelButton()->clicked.connect(SigC::slot(this, &FlipTool::cancelValueDialog));
		valueDialog->delete_event.connect(SigC::slot(this, &FlipTool::deleteValueDialog));
		valueDialog->set_modal(true);
	}
	valueDialog->show();
}

