/* $Id: FontPool.cc,v 1.17 2005/05/07 17:39:32 pklaus Exp $
 *
 * FontPool.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "FontPool.hh"
#include "FontCache.hh"
#include "GDApplication.hh"
#include "Preferences.hh"
#include "GDFont.hh"
#include <vector>
#include <stdio.h>

FontPool::FontPool(GDApplication *gda) {
	application = gda;
	gda->prefsChanged.connect(SigC::slot(this, &FontPool::onPrefsChanged));
	readFontLists();
}

void FontPool::onPrefsChanged(Preferences *newPrefs) {
	readFontLists();
}

void FontPool::readFontLists() {
	FontCache fCache;
	fCache.load(&(application->preferences));
	vector<string>::iterator i = application->preferences.fontDir.begin();
	for (; i != application->preferences.fontDir.end(); i++) {
		string dirPath = *i;
		dirPath.append("/");
		//g_printerr("Reading dir %s\n", dirPath.c_str());
		string dirFile = dirPath;
		dirFile.append("fonts.scale");
		gchar line[512];
		FILE *fDirFile = fopen(dirFile.c_str(), "r");
		if (fDirFile) {
			fgets(line, 512-1, fDirFile);
			while (fgets(line, 512-1, fDirFile)) {
				string lineString = line;
				// fill the info fields
				string delims = "-";
				vector<string> fields = splitLine(lineString, delims);
				// construct file path
				string filePath = dirPath;
				filePath.append(fields[0]);
				string foundry = fields[1];
				string family = fields[2];
				string weight = fields[3];
				string slant = fields[4];
				string spacing = fields[11];
				string charset = fields[13];
				// construct the font's name
				string name = family;
				name.append("-");
				name.append(weight);
				name.append("-");
				name.append(slant);
				//g_printerr("loading font %s\n", filePath.c_str());
				GDFont *font = new GDFont(filePath, name, foundry, family, weight,
											slant, spacing, charset, &fCache);
				// insert into filelist
				if (fontList.count(font->getNameString()) < 1) {
					fontList[font->getNameString()] = font;
					familyList[font->getFamilyString()][font->getFaceName()] = font;
				} else {
					delete font;
				}
				//g_printerr("FontPool::readFontList: fa: %s#%s, w: %s, s: %s, sp: %s\n", foundry.c_str(), family.c_str(), weight.c_str(), slant.c_str(), spacing.c_str());
				//g_printerr("FontPool::readFontLists fontList[%s] = %lX font:%lX\n", font->getNameString().c_str(), fontList[font->getNameString()], font);
				//g_printerr("FontPool::readFontLists familyList[%s][%s] = %lX\n", font->getFamilyString().c_str(), divider.c_str(), familyList[family][divider]);
			}
			fclose(fDirFile);
		}
	}
	FontCache newCache;
	FontMap::iterator it = fontList.begin();
	for(; it != fontList.end(); it++) {
		FontCacheEntry *entry = new FontCacheEntry();
		entry->familyName = (*it).second->getFamilyString();
		entry->fullName = (*it).second->getNameString();
		entry->printName = (*it).second->getPSNameString();
		entry->weight = (*it).second->getWeightString();
		entry->slant = (*it).second->getSlantString();
		newCache.fontCacheMap[(*it).second->getFilePathString()] = entry;
		//g_printerr("saving font cache for %s (%s)\n", (*it).second->getFilePath(), (*it).second->getName());
	}	
	newCache.save(&(application->preferences));
}

GDFont *FontPool::getFont(gchar *name) {
	//g_printerr("FontPool::getFont name:%s\n", name);
	gchar *fontName = NULL;
	FontIterator fit;
	if ((fit = fontList.find(name)) != fontList.end()) {
		fontName = name;
	} else if ((fit = fontList.find("Helvetica")) != fontList.end()) {
		fontName = "Helvetica";
	} else if ((fit = fontList.find("Nimbus Sans L Regular")) != fontList.end()) {
		fontName = "Nimbus Sans L Regular";
	} else if ((fit = fontList.find("Courier")) != fontList.end()) {
		fontName = "Courier";
	} else if ((fit = fontList.find("Courier 10 Pitch")) != fontList.end()) {
		fontName = "Courier 10 Pitch";
	}
	if (fit != fontList.end()) {
		GDFont *font = fit->second;
		//g_printerr("FontPool::getFont return name:%s font:%lX\n", fontName, (long) font);
		font->initialize();
		return font;
	} else {
		g_printerr("FontPool::getFont: ATTENTION font %s NOT FOUND and no alternatives available\n", name);
		return NULL;
	}
}

vector<string> FontPool::splitLine(string& input, string& delims) {
	//g_printerr("FontPool::splitLine: called %s\n", input.c_str());
	string whitespace = " \t\n\r";
	vector<string> result;
	guint l = 0;
	gint p = 0;
	while ((p = input.find_first_of(delims, l)) >= 0) {
		string token = input.substr(l, p - l);
		if (p - l) {
			gint wsspos = token.find_first_not_of(whitespace);
			gint wsepos = token.find_last_not_of(whitespace);
			//g_printerr("FontPool::splitLine: wss:%d wse:%d str:|%s|\n", wsspos, wsepos, token.c_str());
			if (wsspos || (guint) wsepos != token.size() - 1) {
				token = token.substr(wsspos, wsepos - wsspos + 1);
			}
		}
		//g_printerr("FontPool::splitline: pos: %d l: %d token: |%s|\n", p, l, token.c_str());
		result.insert(result.end(), token);
		l = p + 1;
	}
	if (l < input.size() - 1) {
		string token = input.substr(l, input.size() - l);
		result.insert(result.end(), token);
	}
	return result;
}

FontPool::~FontPool() {
	FontMap::iterator it = fontList.begin();
	for(; it != fontList.end(); it++) {
		delete (*it).second;
	}
	FamilyMap::iterator fit = familyList.begin();
}
