/* $Id: GDFont.cc,v 1.14 2005/05/07 17:39:33 pklaus Exp $
 *
 * GDFont.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "GDFont.hh"
#include "FontCache.hh"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <fstream>
#include <vector>
#ifdef GEST_CXX_STRSTREAM_H
#include <strstream.h>
#else
#include <sstream>
#endif
extern FT_Library freetype_library;

#if defined(__hpux)
	/* #define UT_BIG_ENDIAN */
	#include <machine/param.h>
	#if defined(_BIG_ENDIAN)
		#define UT_BIG_ENDIAN
	#endif
	#if defined(_LITTLE_ENDIAN)
		#define UT_LITTLE_ENDIAN
	#endif
#elif defined(__QNXNTO__)
	#include <sys/platform.h>
	#if defined(__LITTLEENDIAN__)
		#define UT_LITTLE_ENDIAN
	#else
		#define UT_BIG_ENDIAN
	#endif
#elif defined(__FreeBSD__)
	#include <machine/endian.h>
	#if __BYTE_ORDER == __LITTLE_ENDIAN		
		#define UT_LITTLE_ENDIAN
	#else
		#define UT_BIG_ENDIAN
	#endif
#elif defined(__sgi)
	#include <sys/endian.h>
	#if BYTE_ORDER == LITTLE_ENDIAN
		#define UT_LITTLE_ENDIAN
	#else
		#define UT_BIG_ENDIAN
	#endif
#elif defined(__MACH__) && defined(__APPLE__)
	#if defined(__BIG_ENDIAN__)
		#define UT_BIG_ENDIAN
	#else
		#define UT_LITTLE_ENDIAN
	#endif
#elif defined(SunOS)	/* this is defined by src/config/platform/sunos.mk */
        #include <sys/isa_defs.h>
        #if defined(_BIG_ENDIAN)
                #define UT_BIG_ENDIAN
        #elif defined(_LITTLE_ENDIAN)
                #define UT_LITTLE_ENDIAN
        #endif
#elif defined(__NetBSD__)
	#include <machine/endian.h>
	#if BYTE_ORDER == LITTLE_ENDIAN         
		#define UT_LITTLE_ENDIAN
	#elif BYTE_ORDER == BIG_ENDIAN
		#define UT_BIG_ENDIAN
	#endif
#elif defined(_AIX)
        #include <sys/machine.h>
        #if BYTE_ORDER == LITTLE_ENDIAN
                #define UT_LITTLE_ENDIAN
        #elif BYTE_ORDER == BIG_ENDIAN
                #define UT_BIG_ENDIAN
        #endif
#elif defined(__osf__)
        #include <machine/endian.h>
        #if BYTE_ORDER == LITTLE_ENDIAN
                #define UT_LITTLE_ENDIAN
        #elif BYTE_ORDER == BIG_ENDIAN
                #define UT_BIG_ENDIAN
        #endif
#elif defined(_WIN32)
	/* We should probably do some check as the WinAPI could be on other computers */
	#if !defined(UT_LITTLE_ENDIAN) && !defined(UT_BIG_ENDIAN)
		#define UT_LITTLE_ENDIAN
	#endif		
#else /* this is for Linux */
	#include <endian.h>
	#if __BYTE_ORDER == __LITTLE_ENDIAN		
		#define UT_LITTLE_ENDIAN
	#else
		#define UT_BIG_ENDIAN
	#endif
#endif


/*  Make sure we got a definition for our platform:  */
#if defined(UT_BIG_ENDIAN)
#elif defined(UT_LITTLE_ENDIAN)
#else
	#error Must define UT_BIG_ENDIAN or UT_LITTLE_ENDIAN in ut_endian.h
#endif

/*XP macros

 convert single UCS character
 x,y are pointers to UT_UCSChar
 we will use a temporary variable, so that x and y
 can be the same
*/

#ifdef UT_LITTLE_ENDIAN		
#define LE2BE16(x,y)                                  \
char * lb1;                                           \
UT_UCSChar tucs;                                      \
tucs = * ((UT_UCSChar *)(x)); lb1 = (char*) (&tucs);  \
*((char*)(y)) = *(lb1+1); *(((char*)(y)+1)) = *lb1;
#else
#define LE2BE16(x,y)
#endif

/* Endianness conversions */
#ifdef UT_LITTLE_ENDIAN
#define H(x) ((TTF_SHORT)((((TTF_SHORT)(x))&0xFF)<<8)+(((TTF_USHORT)(x))>>8))
#define UH(x) ((TTF_USHORT)((((TTF_USHORT)(x))&0xFF)<<8)+(((TTF_USHORT)(x))>>8))
#define L(x) ((TTF_LONG)(((TTF_LONG)UH((x)&0xFFFF))<<16)+UH(((TTF_ULONG)(x))>>16))
#define UL(x) ((TTF_ULONG)(((TTF_ULONG)UH((x)&0xFFFF))<<16)+UH(((TTF_ULONG)(x))>>16))
#else
#define H(x) ((TTF_SHORT)x)
#define UH(x) ((TTF_USHORT)x)
#define L(x) ((TTF_LONG)x)
#define UL(x) ((TTF_ULONG)x)
#endif

#define FIX_H(x) x=H(x)
#define FIX_UH(x) x=UH(x)
#define FIX_L(x) x=L(x)
#define FIX_UL(x) x=UL(x)


typedef unsigned char	TTF_BYTE;
typedef signed char		TTF_CHAR;
typedef unsigned short	TTF_USHORT;
typedef signed short    TTF_SHORT;
typedef unsigned long   TTF_ULONG;
typedef signed long     TTF_LONG;
typedef unsigned long   TTF_FIXED_L;
typedef unsigned short  TTF_FUNIT;
typedef signed short    TTF_FWORD;
typedef unsigned short  TTF_UFWORD;
typedef unsigned short  TTF_F2DOT14;
struct TTF_FIXED {
	TTF_SHORT mantissa;
	TTF_USHORT fraction;
};
struct longDateTime {
	TTF_BYTE data[8];
};
struct HeadTable {
	TTF_FIXED version;
	TTF_FIXED fontRevision;
	TTF_ULONG checkSumAdjustment;
	TTF_ULONG magicNumber;
	TTF_USHORT flags;
	TTF_USHORT unitsPerEm;
	longDateTime created;
	longDateTime modified;
	TTF_FWORD xMin;
	TTF_FWORD yMin;
	TTF_FWORD xMax;
	TTF_FWORD yMax;
	TTF_USHORT macStyle;
	TTF_USHORT lowestRecPPEM;
	TTF_SHORT fontDirectionHint;
	TTF_SHORT indexToLocFormat;
	TTF_SHORT glyphDataFormat;
};
struct OffsetTable {
	TTF_FIXED version;
	TTF_USHORT numTables;
	TTF_USHORT searchRange;
	TTF_USHORT entrySelector;
	TTF_USHORT rangeShift;
};

struct TableDirectoryEntry {
	TTF_ULONG tag;
	TTF_ULONG checkSum;
	TTF_ULONG offset;
	TTF_ULONG length;
};
struct NameRecord {
	TTF_USHORT platformID;
	TTF_USHORT encodingID;
	TTF_USHORT languageID;
	TTF_USHORT nameID;
	TTF_USHORT length;
	TTF_USHORT offset;
};

#define FIX_Fixed(x) {(x).mantissa=H((x).mantissa); (x).fraction=UH((x).fraction);}

#define FIX_OffsetTable(x) \
  {FIX_Fixed((x).version);\
   FIX_UH((x).numTables);\
   FIX_UH((x).searchRange);\
   FIX_UH((x).entrySelector);}                            
#define FIX_TableDirectoryEntry(x) \
  {FIX_UL((x).tag); FIX_UL((x).checkSum);\
   FIX_UL((x).offset); FIX_UL((x).length);}
#define MAKE_ULONG(a,b,c,d) ((TTF_ULONG)(((TTF_ULONG)(a)<<24)|((b)<<16)|((c)<<8)|(d)))
#define FIX_HeadTable(x) \
  {FIX_Fixed((x).version); FIX_Fixed((x).fontRevision);\
   FIX_UL((x).checkSumAdjustment); FIX_UL((x).magicNumber);\
   FIX_UH((x).flags); FIX_UH((x).unitsPerEm);\
   FIX_UH((x).xMin); FIX_UH((x).yMin); FIX_UH((x).xMax); FIX_UH((x).yMax);\
   FIX_UH((x).macStyle); FIX_UH((x).lowestRecPPEM);\
   FIX_H((x).fontDirectionHint); FIX_H((x).indexToLocFormat);\
   FIX_H((x).glyphDataFormat);}
#define FIX_NameRecord(x) \
  {FIX_UH((x).platformID); FIX_UH((x).encodingID); FIX_UH((x).languageID);\
   FIX_UH((x).nameID); FIX_UH((x).length); FIX_UH((x).offset);}

#define read_FIXED(fd, valuebuffer, size, fix_call) \
	{ \
	int ret; \
	if ((ret = read(fd, &valuebuffer, size)) < (int) (size)) { \
		g_printerr("Read error in ttf file %s (%d!=%d)\n", file.c_str(), size, ret); \
	} \
	} \
	fix_call(valuebuffer)

   
static char *mac_glyph_names[] = {
/* 0x00 */
    ".notdef", ".null", "CR","space","exclam","quotedbl","numbersign",
    "dollar","percent","ampersand","quotesingle","parenleft","parenright",
    "asterisk","plus","comma",
/* 0x10 */
    "hyphen","period","slash","zero","one","two","three","four","five",
    "six","seven","eight","nine","colon","semicolon","less",
/* 0x20 */
    "equal","greater","question","at","A","B","C","D","E","F","G","H",
    "I","J","K","L",
/* 0x30 */
    "M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z",
    "bracketleft","backslash",
/* 0x40 */
    "bracketright","asciicircum","underscore","grave","a","b","c","d",
    "e","f","g","h","i","j","k","l",
/* 0x50 */
    "m","n","o","p","q","r","s","t","u","v","w","x","y","z",
    "braceleft","bar",
/* 0x60 */
    "braceright","asciitilde","Adieresis","Aring","Ccedilla","Eacute",
    "Ntilde","Odieresis","Udieresis","aacute","agrave","acircumflex",
    "adieresis","atilde","aring","ccedilla",
/* 0x70 */
    "eacute","egrave","ecircumflex","edieresis","iacute","igrave",
    "icircumflex","idieresis","ntilde","oacute","ograve","ocircumflex",
    "odieresis","otilde","uacute","ugrave",
/* 0x80 */
    "ucircumflex","udieresis","dagger","degree","cent","sterling","section",
    "bullet","paragraph","germandbls","registered","copyright","trademark",
    "acute","dieresis","notequal",
/* 0x90 */
    "AE","Oslash","infinity","plusminus","lessequal","greaterequal","yen",
    "mu","partialdiff","Sigma","Pi","pi","integral","ordfeminine",
	"ordmasculine","Omega",
/* 0xa0 */
    "ae","oslash","questiondown","exclamdown","logicalnot","radical",
    "florin","approxequal","Delta","guillemotleft","guillemotright",
    "ellipsis","nbspace","Agrave","Atilde","Otilde",
/* 0xb0 */
    "OE","oe","endash","emdash","quotedblleft","quotedblright","quoteleft",
    "quoteright","divide","lozenge","ydieresis","Ydieresis","fraction",
    "currency","guilsinglleft","guilsinglright",
/* 0xc0 */
    "fi","fl","daggerdbl","periodcentered","quotesinglbase","quotedblbase",
    "perthousand","Acircumflex","Ecircumflex","Aacute","Edieresis",
    "Egrave","Iacute","Icircumflex","Idieresis","Igrave",
/* 0xd0 */
	"Oacute","Ocircumflex","applelogo","Ograve","Uacute","Ucircumflex",
	"Ugrave","dotlessi","circumflex","tilde","macron","breve","dotaccent",
	"ring","cedilla","hungarumlaut",
/* 0xe0 */
    "ogonek","caron","Lslash","lslash","Scaron","scaron","Zcaron",
    "zcaron","brokenbar","Eth","eth","Yacute","yacute","Thorn","thorn",
    "minus",
/* 0xf0 */
    "multiply","onesuperior","twosuperior","threesuperior","onehalf",
    "onequarter","threequarters","franc","Gbreve","gbreve","Idot",
    "Scedilla","scedilla","Cacute","cacute","Ccaron",
/* 0x100 */
    "ccaron","dmacron"
};

#define NMACGLYPHS      258

/* gcc on Solaris has a funny trick of using a struct to
 * represent off_t, which screws up constant values and
 * arithmetic, so I'm replacing off_t in/from surely_lseek
 * with long - fjf
 * verbatim from ttftool by the abiword team
 */
long surely_lseek(int fildes, long long_offset, int whence) {
	long long_result;
	off_t result;
	off_t offset;
#if defined(SunOS) && _FILE_OFFSET_BITS == 64
#if __STDC__ - 0 == 0 && !defined(_NO_LONGLONG)
	offset = long_offset;
#else
	offset._l[0] = 0;
	offset._l[1] = long_offset;
#endif
#else
	offset = long_offset;
#endif
	
	result = lseek (fildes, offset, whence);
	
#if defined(SunOS) && _FILE_OFFSET_BITS == 64
#if __STDC__ - 0 == 0 && !defined(_NO_LONGLONG)
	long_result = result;
#else
	long_result = result._l[1];
#endif
#else
	long_result = result;
#endif
		
	return long_result;
}

char * unistrncpy(char *dst, char *str, size_t length) {
	int i,j;
	
	for (i = j = 0; i < (int) length; i += 2) {
		if (str[i] == 0) {
			dst[j++] = str[i + 1];
		}
	}
	dst[j] = '\0';
	return dst;
}

GDFont::GDFont(string& filepath, string& name, string& foundry, string& familyName,
					string& weight, string& slant, string& spacing, string& charset,
					FontCache *fcache) : GDResource(GDRESOURCE_TYPE_FONT) {
	this->name = name;
	family = familyName;
	this->foundry = foundry;
	this->weight = weight;
	this->slant = slant;
	this->spacing = spacing;
	this->charset = charset;
	isFixedPitch = false;
	psName = name;
	file = filepath;
	gint len = file.size();
	const char *ext = file.c_str() + len - 3;
	if (! strncasecmp(ext, "pfa", 3)) {
		type = GDFONT_TYPE_PFA;
	} else if (! strncasecmp(ext, "pfb", 3)) {
		type = GDFONT_TYPE_PFB;
	} else if (! strncasecmp(ext, "ttf", 3)) {
		type = GDFONT_TYPE_TTF;
	} else {
		type = -1;
		g_printerr("GDFont::GDFont type of file %s unknown.\n", file.c_str());
	}
	if (type == GDFONT_TYPE_PFA || type == GDFONT_TYPE_PFB || type == GDFONT_TYPE_TTF) {
		if (fcache->fontCacheMap.find(filepath) != fcache->fontCacheMap.end()) {
			// font already exist in cache
			FontCacheEntry *fcentry = fcache->fontCacheMap[filepath];
			if (fcentry->fullName.size()) {
				this->name = fcentry->fullName;
			}
			if (fcentry->printName.size()) {
				psName = fcentry->printName;
			}
			if (fcentry->familyName.size()) {
				family = fcentry->familyName;
			}
			if (fcentry->weight.size()) {
				this->weight = fcentry->weight;
			}
			if (fcentry->slant.size()) {
				this->slant = fcentry->slant;
			}
			this->isFixedPitch = fcentry->isFixedPitch;
			//g_printerr("Loading font data for %s from Cache (%s)\n", name.c_str(), family.c_str());
		} else {
			// no entry in cache found
			if (type == GDFONT_TYPE_PFA) {
				//g_printerr("GDFont::GDFont reading file %s\n", file.c_str());
				ifstream input(file.c_str());
				type1GetNames(input);
			} else if (type == GDFONT_TYPE_PFB) {
	#ifdef GEST_CXX_STRSTREAM_H
				ostrstream output;
	#else
				ostringstream output;
	#endif
				pfb2pfaStream(file, output);
	#ifdef GEST_CXX_STRSTREAM_H
				istrstream input(output.str());
	#else
				istringstream input(output.str());
	#endif
				type1GetNames(input);
			} else if (type == GDFONT_TYPE_TTF) {
				long headInd=-1, maxpInd=-1, postInd=-1, nameInd=-1, locaInd=-1, glyfInd=-1, hheaInd=-1, hmtxInd=-1, kernInd=-1;
				int fd;
				if ((fd = open(file.c_str(), O_RDONLY)) < 0) {
					g_printerr("Cannot open ttf font file %s for reading\n", file.c_str());
				}
				TableDirectoryEntry *td;
				OffsetTable ot;
				if (read(fd, &ot, sizeof(OffsetTable)) < (int) sizeof(OffsetTable)) g_printerr("Error in ttf file %s\n", file.c_str());
				FIX_OffsetTable(ot);
				int n = sizeof(TableDirectoryEntry) * ot.numTables;
				td = (TableDirectoryEntry *) g_malloc(n);
				if (read(fd, td, n) < n) g_printerr("Error in ttf file %s\n", file.c_str());
				for (int i = 0; i < ot.numTables; i++) {
					FIX_TableDirectoryEntry(td[i]);
				}
				for (int i = 0; i < ot.numTables; i++) {
					//g_printerr("Found `%c%c%c%c' table\n", (char)(td[i].tag>>24), (char)(td[i].tag>>16)&255, (char)(td[i].tag>>8)&255, (char)td[i].tag&255);
					
					switch(td[i].tag) {
						case MAKE_ULONG('m','a','x','p'):
							maxpInd=i;
							break;
						case MAKE_ULONG('h','e','a','d'):
							headInd=i;
							break;
						case MAKE_ULONG('p','o','s','t'):
							postInd=i;
							break;
						case MAKE_ULONG('n','a','m','e'):
							nameInd=i;
							break;
						case MAKE_ULONG('l','o','c','a'):
							locaInd=i;
							break;
						case MAKE_ULONG('g','l','y','f'):
							glyfInd=i;
							break;
						case MAKE_ULONG('h','h','e','a'):
							hheaInd=i;
							break;
						case MAKE_ULONG('h','m','t','x'):
							hmtxInd=i;
							break;
						case MAKE_ULONG('k','e','r','n'):
							kernInd=i;
							break;
					}
				}
				//g_printerr("Index: maxp=%lX head=%lX post=%lX name=%lX loca=%lX glyf=%lX hhea=%lX hmtx=%lX kern=%lX\n", maxpInd, headInd, postInd, nameInd, locaInd, glyfInd, hheaInd,hmtxInd, kernInd);
				
				// read font name stuff
				surely_lseek(fd, td[nameInd].offset, SEEK_SET);
				TTF_USHORT format;
				TTF_USHORT nrecords;
				long position;
				TTF_USHORT offset;
				int i,index,maxIndex;
				NameRecord *records;
				char *data;
				
				position = surely_lseek(fd, 0, SEEK_CUR);
				//g_printerr("position=%lX should be %lX\n", position, td[nameInd].offset);
				
				read_FIXED(fd, format, sizeof(TTF_USHORT), FIX_UH);
				if (format != 0) g_printerr("Bad ttf file %s\n", file.c_str());
				read_FIXED(fd, nrecords, sizeof(TTF_USHORT), FIX_UH);
				read_FIXED(fd, offset, sizeof(TTF_USHORT), FIX_UH);
				records = (NameRecord *) g_malloc(nrecords * sizeof(NameRecord));
				if (read(fd, records, nrecords * sizeof(NameRecord)) < nrecords * (int) sizeof(NameRecord)) g_printerr("Error in ttf file %s\n", file.c_str());
				for (i = 0, maxIndex = -1; i < nrecords; i++) {
					FIX_NameRecord(records[i]);
					index = records[i].offset + records[i].length;
					maxIndex = maxIndex > index ? maxIndex : index;
				}
				data = (char *) g_malloc(maxIndex);
				surely_lseek(fd, position + offset, SEEK_SET);
				if (read(fd, data, maxIndex) < maxIndex) g_printerr("Error in ttf file %s\n", file.c_str());
				for (i = 0; i < nrecords; i++) {
					if (records[i].platformID == 3 && /* Microsoft */
						records[i].encodingID == 1 && /* UGL */
						records[i].languageID == 0x0409 && /* US English */
						records[i].nameID <= 7) {
						char isostr[512];
						unistrncpy(isostr, data + records[i].offset, records[i].length < 512 ? records[i].length : 511);
						string str(isostr);
						switch (records[i].nameID) {
							case 1:
								this->family = str;
								break;
							case 2:
								this->weight = str;
								break;
							case 4:
								this->name = str;
								break;
							case 6:
								this->psName = str;
								break;
						}
						//g_printerr("nameID=%d string=%s i=%d l=%d o=%d\n", records[i].nameID, str.c_str(), i, records[i].length, records[i].offset);
					}
				}
				g_free(records);
				g_free(data);
				// get isFixedPitch value
				TTF_ULONG IsFixedPitch;
				surely_lseek(fd, td[postInd].offset + 2 *(sizeof(TTF_FIXED) + sizeof(TTF_FWORD)), SEEK_SET);
				read_FIXED(fd, IsFixedPitch, sizeof(TTF_ULONG), FIX_UL);
				//g_printerr("isFixedPitch=%lX\n", IsFixedPitch);
				this->isFixedPitch = (IsFixedPitch != 0);
				g_free(td);
				close(fd);
			}
			//g_printerr("Reading font data for %s from file (%s)\n", name.c_str(), family.c_str());
		}
	}
	string faceName = this->weight;
	faceName.append(" ");
	faceName.append(this->slant);
	this->faceName = faceName;

	//g_printerr("GDFont::GDFont file: %s name: %s psName: %s\n", file.c_str(), this->name.c_str(), psName.c_str());
	initialized = false;
	usageCount = 0;
	face = NULL;
}

void GDFont::initialize() {
	if (initialized) return;
	//g_printerr("GDFont::initialize name: %s file: %s\n", name.c_str(), file.c_str());
	int error = FT_New_Face(freetype_library, file.c_str(), 0, &face);
	if (error) {
		g_printerr("Cannot load font %s\n", file.c_str());
		return;
	}
	int slen = file.size();
	if (type == GDFONT_TYPE_PFA || type == GDFONT_TYPE_PFB) {
		string afmpath = file;
		afmpath.replace(slen-3, 3, "afm");
		error = FT_Attach_File(face, afmpath.c_str());
		if (error) {
			g_printerr("Cannot attach metrics file %s\n", afmpath.c_str());
			return;
		}
	}
	initialized = true;
}

void GDFont::incUsageCount() {
	usageCount++;
	if (! initialized) initialize();
	//g_printerr("GDFont::incUsageCount name: %s, count:%d\n", getName(), usageCount);
}

void GDFont::decUsageCount() {
	usageCount--;
	//g_printerr("GDFont::decUsageCount name: %s, count:%d\n", getName(), usageCount);
	if (usageCount <= 0 && initialized) {
		// release the freetype memory
		//g_printerr("FaceDone: %s\n", name.c_str());
		FT_Done_Face(face);
		face = NULL;
		initialized = false;
	}
}

const char *GDFont::getName() {
	return name.c_str();
}

const char *GDFont::getFamily() {
	return family.c_str();
}

const char *GDFont::getFilePath() {
	return file.c_str();
}

const char *GDFont::getFoundry() {
	return foundry.c_str();
}

const char *GDFont::getWeight() {
	return weight.c_str();
}

const char *GDFont::getSlant() {
	return slant.c_str();
}

const char *GDFont::getSpacing() {
	return spacing.c_str();
}

const char *GDFont::getCharset() {
	return charset.c_str();
}

const char *GDFont::getFaceName() {
	return faceName.c_str();
}

const char *GDFont::getPSName() {
	return psName.c_str();
}

string GDFont::getNameString() {
	return name;
}

string GDFont::getFamilyString() {
	return family;
}

string GDFont::getFilePathString() {
	return file;
}

string GDFont::getFoundryString() {
	return foundry;
}

string GDFont::getWeightString() {
	return weight;
}

string GDFont::getSlantString() {
	return slant;
}

string GDFont::getSpacingString() {
	return spacing;
}

string GDFont::getCharsetString() {
	return charset;
}

string GDFont::getFaceNameString() {
	return faceName;
}

string GDFont::getPSNameString() {
	return psName;
}

FT_Face GDFont::getFace() {
	return face;
}

static void ordinary(char, ostream&);
static void cr(char, ostream&);
static void lf(char, ostream&);

void (*outputchar)(char, ostream&);

static void ordinary(char c, ostream& psout) {
	switch(c) {
		case '\r': psout << endl; outputchar = cr; break;
		case '\n': psout << endl; outputchar = lf; break;
		default: psout << c; break;
	}
}

static void cr(char c, ostream& psout) {
	switch(c) {
		case '\r': psout << endl; break;
		default: psout << c;
		case '\n': outputchar = ordinary; break;
	}
}

static void lf(char c, ostream& psout) {
	switch (c) {
	case '\n': psout << endl; break;
	default: psout << c;
	case '\r': outputchar = ordinary; break;
	}
}

gint GDFont::pfb2pfaStream(string filepath, ostream& out) {
	ifstream input(filepath.c_str(), ios::binary);
	// PFB version
	static const char *hexchar = "0123456789abcdef";
	outputchar = ordinary;
	for (;;) {
		int i;
		if ((i = input.get()) == EOF) {
			return -1;
		} else if (i != 0x80) {
			return -2;
		}

		guint count;

		switch (input.get()) {
			case EOF:
				return -1;
				break;
			case 1: // PFB_ASCII
				count  = (guint)input.get();
				count |= (guint)input.get() << 8;
				count |= (guint)input.get() << 16;
				count |= (guint)input.get() << 24;
				//g_printerr("PFB_ASCII: %d bytes\n", count);
				for (; count; count--) {
					char c;
					if ((c = input.get()) == EOF) return -1;
					outputchar(c, out);
				}
				break;
			case 2: // PFB_BINARY
				count  = (guint)input.get();
				count |= (guint)input.get() << 8;
				count |= (guint)input.get() << 16;
				count |= (guint)input.get() << 24;
				//g_printerr("PFB_BINARY: %d bytes\n", count);
				for (i = 0; count; count--) {
					if (! input.good()) {
						//g_printerr("Unexpected end of file in binary part\n");
						return -1;
					}
					char c = input.get();
					out << hexchar[(c >> 4) & 0xf];
					out << hexchar[c & 0xf];
					if (++i == 32) {
						out << endl;
						i = 0;
					}
				}
				if (i) out << endl;
				break;
			case 3: // PFB_DONE
				return 0;
				break;
			default:
				//g_printerr("PFB: unexpected input\n");
				return -2;
		}
	}
}

#define TTF_CHUNKSIZE	65534

gint GDFont::writePS(ostream& psout) {
	//g_printerr("GDFont::writePS: type: %d\n", type);
	psout << "%%BeginResource: font " << psName << endl;
	if (type == GDFONT_TYPE_PFB) {
		gint retval = pfb2pfaStream(file, psout);
		if (retval) return retval;
	} else if (type == GDFONT_TYPE_PFA) {
		ifstream input(file.c_str());
		string line;
		while (getline(input, line)) {
			psout << line << endl;
		}
	} else if (type == GDFONT_TYPE_TTF) {
		HeadTable ht;
		int nglyphs = 0;
		vector<string> mtx_tab;
		int headInd=-1, maxpInd=-1, postInd=-1, nameInd=-1, locaInd=-1, glyfInd=-1, hheaInd=-1, hmtxInd=-1, kernInd=-1;
		
		int fd;
		if ((fd = open(file.c_str(), O_RDONLY)) < 0) {
			g_printerr("Cannot open ttf font file %s for reading\n", file.c_str());
		}
		TableDirectoryEntry *td;
		OffsetTable ot;
		if (read(fd, &ot, sizeof(OffsetTable)) < (int) sizeof(OffsetTable)) g_printerr("Error in ttf file %s\n", file.c_str());
		FIX_OffsetTable(ot);
		int n = sizeof(TableDirectoryEntry) * ot.numTables;
		td = (TableDirectoryEntry *) g_malloc(n);
		if (read(fd, td, n) < n) g_printerr("Error in ttf file %s\n", file.c_str());
		for (int i = 0; i < ot.numTables; i++) {
			FIX_TableDirectoryEntry(td[i]);
		}
		for (int i = 0; i < ot.numTables; i++) {
			//g_printerr("Found `%c%c%c%c' table", (char)(td[i].tag>>24), (char)(td[i].tag>>16)&255, (char)(td[i].tag>>8)&255, (char)td[i].tag&255);
			
			switch(td[i].tag) {
				case MAKE_ULONG('m','a','x','p'):
					maxpInd=i;
					break;
				case MAKE_ULONG('h','e','a','d'):
					headInd=i;
					break;
				case MAKE_ULONG('p','o','s','t'):
					postInd=i;
					break;
				case MAKE_ULONG('n','a','m','e'):
					nameInd=i;
					break;
				case MAKE_ULONG('l','o','c','a'):
					locaInd=i;
					break;
				case MAKE_ULONG('g','l','y','f'):
					glyfInd=i;
					break;
				case MAKE_ULONG('h','h','e','a'):
					hheaInd=i;
					break;
				case MAKE_ULONG('h','m','t','x'):
					hmtxInd=i;
					break;
				case MAKE_ULONG('k','e','r','n'):
					kernInd=i;
					break;
			}
		}
		surely_lseek(fd, td[headInd].offset, SEEK_SET);
		if (read(fd, &ht, sizeof(HeadTable)) < (int) sizeof(HeadTable)) g_printerr("Error in ttf file %s\n", file.c_str());
		FIX_HeadTable(ht);
		if(ht.magicNumber!=0x5F0F3CF5) g_printerr("Bad magic number in ttf file %s\n", file.c_str());
		surely_lseek(fd, td[maxpInd].offset + sizeof(TTF_FIXED_L), SEEK_SET);
		read_FIXED(fd, nglyphs, sizeof(TTF_USHORT), FIX_UH);

		TTF_FIXED post_format;
		TTF_FIXED ItalicAngle;
		TTF_FWORD UnderlinePosition;
		TTF_FWORD UnderlineThickness;
		TTF_ULONG IsFixedPitch;
		TTF_ULONG minMemType42;
		TTF_ULONG maxMemType42;
		TTF_ULONG minMemType1;
		TTF_ULONG maxMemType1;
		surely_lseek(fd, td[postInd].offset, SEEK_SET);
		read_FIXED(fd, post_format, sizeof(TTF_FIXED), FIX_Fixed);
		read_FIXED(fd, ItalicAngle, sizeof(TTF_FIXED), FIX_Fixed);
		read_FIXED(fd, UnderlinePosition, sizeof(TTF_FWORD), FIX_H);
		read_FIXED(fd, UnderlineThickness, sizeof(TTF_FWORD), FIX_H);
		read_FIXED(fd, IsFixedPitch, sizeof(TTF_ULONG), FIX_UL);
		read_FIXED(fd, minMemType42, sizeof(TTF_ULONG), FIX_UL);
		read_FIXED(fd, maxMemType42, sizeof(TTF_ULONG), FIX_UL);
		read_FIXED(fd, minMemType1, sizeof(TTF_ULONG), FIX_UL);
		read_FIXED(fd, maxMemType1, sizeof(TTF_ULONG), FIX_UL);
		switch (post_format.mantissa) {
			case 0x0001:
				for (int i = 0; i < NMACGLYPHS; i++) {
					mtx_tab.push_back(string(mac_glyph_names[i]));
				}
				break;
			case 0x0002:
				TTF_USHORT l;
				read_FIXED(fd, l, sizeof(TTF_USHORT), FIX_H);/* some fonts have this value different from nglyphs */
				//g_printerr("post table contains %d entries", l);
				
				vector<int> pm_index;
				for (int i = 0; i < l; i++) {
					TTF_USHORT index;
					read_FIXED(fd, index, sizeof(TTF_USHORT), FIX_H);
					int ind = index;
					pm_index.push_back(ind);
				}
				if (postInd < 0) {
					g_printerr("Error in ttf file %s\n", file.c_str());
					g_free(td);
					return -1;
				}
				n = td[postInd].length - (surely_lseek(fd, 0, SEEK_CUR) - td[postInd].offset);
				char *ps_glyphs_buf = (char *) g_malloc(n + 1);
				char *p;
				int m;
				for (m = 0, p = ps_glyphs_buf; p - ps_glyphs_buf < n;) {
					unsigned char i;
					if (read(fd, &i, 1) < 1) g_printerr("Error in ttf file %s\n", file.c_str());
					for (; i > 0; i--){
						char ch;
						if (read(fd, &ch, 1) < 1) g_printerr("Error in ttf file %s\n", file.c_str());
						*p++ = ch;
					}
					*p++ = 0;
					m++;
				}
				
				/* now we retrieve the names that are in the font table*/
				for (int i = 0; i < l; i++) {
					if (pm_index[i] < NMACGLYPHS) {
						mtx_tab.push_back(string(mac_glyph_names[pm_index[i]]));
					} else {
						int k = pm_index[i] - NMACGLYPHS;
						if (k <= m) {
							for (p = ps_glyphs_buf; k > 0; k--) {
								p = (char *)strchr(p, 0) + 1;
							}
							mtx_tab.push_back(string(p));
						} else {
							g_printerr("index 0x%04x out of range (glyphs: 0x%04x, m=0x%04x)", k, l, m);
							mtx_tab.push_back(string(""));
						}
					}
				}
				g_free(ps_glyphs_buf);
		}

		
		g_free(td);
		close(fd);
		// print ps font header
		psout << "%!PS-TrueTypeFont" << endl;
		psout << "11 dict begin" << endl;
		psout << "/FontName /" << psName << " def" << endl;
		psout << "/Encoding StandardEncoding def" << endl;
		psout << "/PaintType 0 def" << endl;
		psout << "/FontMatrix [1 0 0 1 0 0] def" << endl;
		psout << "/FontBBox [";
		psout << ht.xMin*1000L/ht.unitsPerEm << " ";
		psout << ht.yMin*1000L/ht.unitsPerEm << " ";
		psout << ht.xMax*1000L/ht.unitsPerEm << " ";
		psout << ht.yMax*1000L/ht.unitsPerEm;
		psout << "] def" << endl;
		psout << "/FontType 42 def" << endl;
		psout << "/FontInfo 8 dict dup begin" << endl;
		psout << "/version (" << ht.fontRevision.mantissa << "." << ht.fontRevision.fraction << ") def" << endl;
		if (name.size()) {
			psout << "/FullName (" << name << ") def" << endl;
		}
		if (family.size()) {
			psout << "/FamilyName (" << family << ") def" << endl;
		}
		psout << "/isFixedPitch " << (isFixedPitch ? "true" : "false") << " def" << endl;
		psout << "/UnderlinePosition " << UnderlinePosition << " def" << endl;
		psout << "/UnderlineThickness " << UnderlineThickness << " def" << endl;
		psout << "end readonly def" << endl;
		// print binary tt font description
		psout << "/sfnts [";
		int ttf_fd = open(file.c_str(), O_RDONLY);
		if (ttf_fd > 0) {
			static char xdigits[]="0123456789ABCDEF";
			unsigned char *buffer = (unsigned char *) g_malloc(TTF_CHUNKSIZE);
			if (buffer) {
				for (;;) {
					int i = read(ttf_fd, buffer, TTF_CHUNKSIZE);
					if (i==0) {
						break;
					}
					psout << endl << "<";
					for (int j = 0; j < i; j++) {
						if (j != 0 && j % 36 == 0) {
							psout << endl;
						}
						psout << xdigits[(buffer[j] & 0xF0) >> 4];
						psout << xdigits[buffer[j] & 0x0F];
					}
					psout << "00>";  // Adobe bug?
					if (i < TTF_CHUNKSIZE) {
						break;
					}
				}
				g_free(buffer);
			}
			close(fd);
		}
		psout << "] def" << endl;
		g_printerr("post_format = %d %d\n", post_format.mantissa, post_format.fraction);
		psout << "/CharStrings " << nglyphs << " dict dup begin" << endl;
		switch (post_format.mantissa) {
			int i, n;
			case 0x0002:
				for (n = i = 0; i < nglyphs; i++) {
					if (n != 0 && n % 4 == 0) {
						psout << endl;
					}
					if (mtx_tab[i].size()) {
						psout << "/" << mtx_tab[i] << " " << i << " def ";
						n++;
					}
				}
				break;
			default:
				for (i = 0; i < 258 && i < nglyphs; i++) {
					psout << "/" << mac_glyph_names[i] << " " << i << " def ";
					if (i != 0 && i % 4 == 0) {
						psout << endl;
					}
				}
				break;
		}
		psout << "end readonly def" << endl;
		psout << "FontName currentdict end definefont pop" << endl;
	}
	psout << "/" << psName << " /" << psName << "-isoenc isovec ReEncode" << endl;
	psout << "%%EndResource" << endl;
	return 0;
}

void GDFont::type1GetNames(istream& input) {
	string line;
	while (getline(input, line)) {
		//g_printerr("GDFont::type1GetNames: line=%s\n", line.c_str());
		guint pos = line.npos;
		if ((pos = line.find("/FullName")) != line.npos) {
			gint b = line.find("(", pos);
			gint e = line.find(")", b);
			if ((guint) b != line.npos && (guint) e != line.npos) {
				b++;
				name = line.substr(b, e-b);
				//g_printerr("FOUND: fullname: %s\n", name.c_str());
			}
		}
		if ((pos = line.find("/FontName")) != line.npos) {
			pos++;
			gint b = line.find("/", pos);
			gint e = line.find(" ", b);
			if ((guint) b != line.npos && (guint) e != line.npos) {
				b++;
				psName = line.substr(b, e-b);
				//g_printerr("FOUND: PSname: %s\n", psName.c_str());
			}
		}
		if ((pos = line.find("/FamilyName")) != line.npos) {
			gint b = line.find("(", pos);
			gint e = line.find(")", b);
			if ((guint) b != line.npos && (guint) e != line.npos) {
				b++;
				family = line.substr(b, e-b);
				//g_printerr("FOUND: family: %s\n", family.c_str());
			}
		}
		if ((pos = line.find("/Weight")) != line.npos) {
			gint b = line.find("(", pos);
			gint e = line.find(")", b);
			if ((guint) b != line.npos && (guint) e != line.npos) {
				b++;
				weight = line.substr(b, e-b);
				//g_printerr("FOUND: weight: %s\n", weight.c_str());
			}
		}
		if ((pos = line.find("/isFixedPitch")) != line.npos) {
			gint b = line.find("true", pos);
			if ((guint) b != line.npos) {
				isFixedPitch = true;
			}
			//g_printerr("FOUND: fixedPitch: %d\n", isFixedPitch);
		}
		if (line.find("currentdict end") != line.npos) {
			return;
		}
	}
}

GDFont::~GDFont() {
	FT_Done_Face(face);
}
