/* $Id: GDStyle.hh,v 1.19 2005/05/07 17:39:33 pklaus Exp $
 *
 * GDStyle.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef _GDSTYLE_HH
#  define _GDSTYLE_HH

#include <gtk--.h>
#include <stdio.h>
#include "gestalter.h"
#include "SVGInput.hh"
#include "GDResource.hh"
#include "TransformationMatrix.hh"
#include "GDColor.hh"
#include <libart_lgpl/art_svp.h>
#include <libart_lgpl/art_rgb_svp.h>
#include <libart_lgpl/art_svp_render_aa.h>
#include <libart_lgpl/art_svp_vpath.h>

#define GDSTYLE_DRAWTYPE_TRANSPARENT 	1
#define GDSTYLE_DRAWTYPE_OPAQUE			2
#define GDSTYLE_DRAWTYPE_GRADIENT		3
#define GDSTYLE_DRAWTYPE_PATTERN		4

#define GDSTYLE_CAPS_BUTT			1
#define GDSTYLE_CAPS_SQUARE			2
#define GDSTYLE_CAPS_ROUND			3

#define GDSTYLE_JOINS_BEVEL			1
#define GDSTYLE_JOINS_MITER			2
#define GDSTYLE_JOINS_ROUND			3

#define GDSTYLE_DRAWTYPE_MASK		(1 << 0)	/*  1 */
#define GDSTYLE_COLORTYPE_MASK		(1 << 1)	/*  2 */
#define GDSTYLE_COLOR_MASK			(1 << 2)	/*  4 */
#define GDSTYLE_STROKEWIDTH_MASK	(1 << 3)	/*  8 */
#define GDSTYLE_LINETYPE_MASK		(1 << 4)	/* 16 */
#define GDSTYLE_DASHES_MASK			(1 << 5)	/* 32 */
#define GDSTYLE_CAPS_MASK			(1 << 6)	/* 64 */
#define GDSTYLE_JOINS_MASK			(1 << 7)	/* 128 */
#define GDSTYLE_GRADIENT_MASK		(1 << 8)	/* 256 */
#define GDSTYLE_PATTERN_MASK		(1 << 9)	/* 512 */
#define GDSTYLE_MATRIX_MASK			(1 << 10)	/* 1024 */
#define GDSTYLE_GUIDE_MASK			(1 << 11)	/* 2048 */

#define GDSTYLE_LINETYPE_PASS			1
#define GDSTYLE_LINETYPE_DASHED			2

class GDDocument;
class Gradient;
class Pattern;

class GDStyle {

public:
	gint drawType;
	GDColor color;
	gint valueMask;
	gdouble opacity;

	// line style values
	gdouble strokeWidth;
	gint lineType;
	gdouble dashes[6];
	gint lineCaps;
	gint lineJoins;
	gdouble lineMiterLimit;
	gboolean isGuide;

	// gradient style values
	Gradient *gradient;

	// pattern style values
	Pattern *pattern;

	// filling (pattern/gradient) transformation
	TransformationMatrix tMatrix;
	
	GDStyle();
	
	GDStyle(const GDStyle& style);
	
	GDStyle operator=(const GDStyle& style);
	void operator&=(const GDStyle& fromStyle);
	GDStyle operator&(const GDStyle& style);
	void operator|=(const GDStyle& gs);
	int operator==(const GDStyle& from);
	
	virtual void getRGBColor(guint& r, guint& g, guint& b) const;
	
	virtual void setColor(const GDStyle *fromStyle);
	
	virtual void setStrokeWidth(gdouble width);
	
	virtual void setColorFromString(gchar *str);

	void fillSVP(ArtSVP *svp, guchar *buffer, guint rowstride, guint width, guint height, gdouble ox, gdouble oy, gdouble scaleFactor);

	void strokeAVP(ArtVpath *avp, guchar *buffer, guint rowstride, guint width, guint height, gdouble ox, gdouble oy, gdouble scaleFactor);

	void transformByMatrix(TransformationMatrix& m);

	void writeColorXML(ostream& xmlout, gchar *name);
	
	gint writeColorPS(ostream& psout, gboolean include_cmd = true);

	gint writePSStrokeStyle(ostream& psout, GDDocument *doc, gboolean include_cmd);
	
	void writeStrokePS(ostream& psout, GDDocument *doc);
	
	void writeGradientPS(ostream& psout, GDDocument *doc);
	
	void writePatternPS(ostream& psout, GDDocument *doc);
	
	void writeFillPS(ostream& psout, GDDocument *doc);
	
	void writeStrokeXML(ostream& xmlout, GDDocument *doc);
	
	void writeFillXML(ostream& xmlout, GDDocument *doc);
	
	void writeFillPreparationXML(ostream& xmlout, GDDocument *doc);
	
	virtual void setStrokeOptionsFromXML(XMLOBJECT *xmlObj, const char **attrs);
	
	virtual void setStrokeStyleFromXML(XMLOBJECT *xmlObj, const char **attrs, GDDocument *doc);
	
	virtual void setFillStyleFromXML(XMLOBJECT *xmlObj, const char **attrs, GDDocument *doc);

	gint writeXMLStrokeStyle(ostream& xmlout);
	
	virtual void gatherResource(GDResourceList& collection, gint type);
	
	virtual void changeDocument(GDDocument *document);
	
	void calcMorphValues(GDStyle *source, GDStyle *target, double n);

	virtual ~GDStyle();
};

#endif
