/* $Id: GObject.hh,v 1.13 2004/07/21 06:31:20 pklaus Exp $
 *
 * GObject.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef _GOBJECT_HH
#  define _GOBJECT_HH

#include "gestalter.h"
#include <glib--.h>
#include <gdk--.h>
#include <gtk--.h>
#include "SelectionItem.hh"
#include "DLList.hh"
#include "DrawObject.hh"
#include "DocWindow.hh"
#include "RectangleArea.hh"
#include <iostream>
#include <vector>

class BezierNode;
class BezierPath;
class GDDocument;

typedef vector<BezierPath *> BezierPathVector;

class GObject : public DrawObject, public DLList<GObject>, public virtual SigC::Object {

public:
	GObject *parent;
	GDDocument *doc;
	
	GObject(GDDocument *document) { parent = NULL; doc = document; }
	
	virtual void addPart(GObject *object) {};

	virtual void draw(OffscreenBuffer& osBuffer, RectangleArea& drawArea) = 0;
	
	virtual void drawBlack(OffscreenBuffer& osBuffer, RectangleArea& drawArea) = 0;
	
	virtual void fillBlack(OffscreenBuffer& osBuffer, RectangleArea& drawArea) = 0;
	
	virtual void drawWireFrame(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea) = 0;
	
	virtual void drawPoints(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea) = 0;
	
	//virtual void fill(DocWindow *docwin, Gdk_GC& context) = 0;
	virtual gdouble attractPoint(gdouble x, gdouble y, gdouble& ax, gdouble& ay, gdouble tolerance, int attractType = 0) = 0;
	
	virtual SelectableItem *checkPoints(gdouble x, gdouble y, gdouble tolerance, gboolean onlyCurvePoints) = 0;
	
	virtual GObject *checkArea(gdouble x, gdouble y) = 0;
	
	virtual SelectableItem *checkCurves(gdouble x, gdouble y, gdouble tolerance) = 0;
		
	virtual BezierPathVector *getBezierPaths() { return NULL; };
	
	virtual void selectRectangle(gdouble left, gdouble top, gdouble right, gdouble bottom, gint what) = 0;
	
	virtual void deSelectRectangle(gdouble left, gdouble top, gdouble right, gdouble bottom, gint what) = 0;
	
	virtual GObject *removeObject(GObject *object) = 0;

	virtual GObject *findObject(GObject *object) = 0;

	virtual void selectAll() = 0;	

	virtual void deSelectAll() = 0;
	
	virtual void select(gboolean shiftSelect) {
		if (shiftSelect && selected()) { deSelectAll(); } else { selectAll(); }
	}
	
	virtual void deSelect() { deSelectAll(); }
	
	virtual void selectObject(gboolean shiftSelect) {
		GObject *p = this; while (p->parent) p = p->parent; p->select(shiftSelect);
	}

	virtual void deSelectObject() {
		GObject *p = this; while (p->parent) p = p->parent; p->deSelect();
	}

	virtual gint getSelection(SelectionItemList& selectionList, gint mode) = 0;

	virtual gint writeXML(ostream& xmlout) = 0;
	
	virtual void writePSPathDescription(ostream& psout) { writePS(psout); };

	virtual gint writePS(ostream& psout) = 0;
	
	virtual void setDocument(GDDocument *document) { doc = document; };

	virtual void reverseOrder() = 0;
};
#endif
