/* $Id: Gradient.cc,v 1.4 2004/03/23 11:42:41 pklaus Exp $
 *
 * Gradient.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002-2004 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "Gradient.hh"

Gradient::Gradient() : GDResource(GDRESOURCE_TYPE_GRADIENT) {
	type = GRADIENT_TYPE_LINEAR;
}

Gradient::Gradient(Gradient& from) : GDResource(GDRESOURCE_TYPE_GRADIENT) {
	type = from.type;
	name = from.name;
	for (vector<ColorPoint>::iterator it = from.colorPoints.begin(); it != from.colorPoints.end(); it++) {
		colorPoints.push_back(*it);
	}
}

Gradient Gradient::operator=(Gradient& from) {
	type = from.type;
	name = from.name;
	colorPoints.clear();
	for (vector<ColorPoint>::iterator it = from.colorPoints.begin(); it != from.colorPoints.end(); it++) {
		colorPoints.push_back(*it);
	}
	return *this;
}

int Gradient::operator==(Gradient& b) {
	if (type != b.type) {
		return 0;
	}
	if (name != b.name) {
		return 0;
	}
	if (colorPoints.size() != b.colorPoints.size()) {
		return 0;
	}
	vector<ColorPoint>::iterator it = colorPoints.begin();
	vector<ColorPoint>::iterator it_b = b.colorPoints.begin();
	for (;it != colorPoints.end(); it++, it_b++) {
		if ((*it).pos != (*it_b).pos) {
			return 0;
		}
		if (!((*it).color == (*it_b).color)) {
			return 0;
		}
	}
	return 1;
}

gint Gradient::writePS(ostream& psout) {
	psout << "/gradient_" << ((long) this) << "\n";
	psout << "<< /PatternType 2\n";
	psout << "\t/Shading\n";
	psout << "\t<<\t/ShadingType " << (type == GRADIENT_TYPE_LINEAR ? "2" : "3") << "\n";
	psout << "\t\t/ColorSpace /DeviceRGB\n";
	psout << "\t\t/Coords [ 0 0 " << (type == GRADIENT_TYPE_LINEAR ? "1 0" : "0 0 0 1") << " ]\n";
	psout << "\t\t/Extend [ true true ]\n";
	psout << "\t\t/Function\n";
	psout << "\t\t<<\t/FunctionType 3\n";
	psout << "\t\t\t/Domain [0 1]\n";
	psout << "\t\t\t/Range  [0 1 0 1 0 1]\n";
	psout << "\t\t\t/Bounds [ ";
	for (uint i = 1; i < colorPoints.size() - 1; i++) {
		psout << colorPoints[i].pos << " ";
	}
	psout << "]\n";
	psout << "\t\t\t/Encode [ ";
	for (uint i = 1; i < colorPoints.size(); i++) {
		psout << "0 1 ";
	}
	psout << "]\n";
	psout << "\t\t\t/Functions [\n";
	vector<ColorPoint>::iterator it = colorPoints.begin();
	guint lr,lg,lb,hr,hg,hb;
	(*it).color.getRGBColor(lr, lg, lb);
	for (it++; it != colorPoints.end(); it++) {
		(*it).color.getRGBColor(hr, hg, hb);
		psout << "\t\t\t<<\t/FunctionType 2\n";
		psout << "\t\t\t\t/Domain [ 0 1 ]\n";
		psout << "\t\t\t\t/N 1\n";
		psout << "\t\t\t\t/C0 [ " << (double) lr / 255.0 << " " << (double) lg / 255.0 << " " << (double) lb / 255.0 << " ]\n";
		psout << "\t\t\t\t/C1 [ " << (double) hr / 255.0 << " " << (double) hg / 255.0 << " " << (double) hb / 255.0 << " ]\n";
		psout << "\t\t\t>>\n";
		lr = hr;
		lg = hg;
		lb = hb;
	}
	psout << "\t\t\t]\n";
	psout << "\t\t>>\n";
	psout << "\t>>\n";
	psout << ">>\n";
	psout << "def\n";
	
	return 0;
}

gint Gradient::writeXML(ostream& xmlout) {
	xmlout << "<" << (type == GRADIENT_TYPE_LINEAR ? "linearGradient" : "radialGradient");
	xmlout << " id=\"gradient_" << (long) this << "\" gradientUnits=\"userSpaceOnUse\"";
	xmlout << " gestalter:name=\"" << name << "\"";
	if (type == GRADIENT_TYPE_LINEAR) {
		xmlout << " x1=\"0\" y1=\"0\" x2=\"1\" y2=\"0\"";
	} else {
		xmlout << " cx=\"0\" cy=\"0\" r=\"1\"";
	}
	xmlout << ">\n";
	for (vector<ColorPoint>::iterator it = colorPoints.begin(); it != colorPoints.end(); it++) {
		xmlout << "<stop offset=\"" << (*it).pos << "\" stop-color=\"";
		gchar colorval[8];
		guint r,g,b;
		(*it).color.getRGBColor(r,g,b);
		snprintf(colorval, 8, "#%02hhX%02hhX%02hhX", r, g, b);
		xmlout << colorval << "\" stop-opacity=\"1\"/>\n";
	}		
	xmlout << "</" << (type == GRADIENT_TYPE_LINEAR ? "linearGradient" : "radialGradient") << ">\n";
	return 0;
}

Gradient::~Gradient() {
}
