/* $Id: GradientListOrgDialog.cc,v 1.3 2004/03/23 11:42:41 pklaus Exp $
 *
 * GradientListOrgDialog.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002-2004 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "GradientListOrgDialog.hh"
#include "GDApplication.hh"
#include "GDDocument.hh"
#include "Gradient.hh"
#include "EditGradientDialog.hh"
#include <string>
#include <gtk--.h>
#include "../pixmaps/gradient_preview.xpm"

#include <libgnome/libgnome.h>  /* for _() macro */

GradientListOrgDialog::GradientListOrgDialog(GDApplication *app) : ListOrgDialog(app) {
	inEdit = NULL;
	currentGradient = NULL;
	untitledNumber = 1;
	editGradientDlg = NULL;
}

void GradientListOrgDialog::onClickColumn(gint column) {
	// do nothing, shouldn't be possible
}

void GradientListOrgDialog::onSelectRow(gint row, gint column, GdkEvent *event) {
	//g_printerr("GradientListOrgDialog::onSelectRow: row=%d, col=%d\n", row, column);
	if (!event) return;
	if (event->type == GDK_BUTTON_PRESS) {
		gint count = row;
		GradientMap::iterator it = doc->gradients.begin();
		for (; it != doc->gradients.end() && count; it++, count--);
		currentGradient = (*it).second;
	} else if (event->type == GDK_2BUTTON_PRESS) {
		if (currentGradient) {
			inEdit = new Gradient(*currentGradient);
			showEditDialog(inEdit);
		}
	}
}

void GradientListOrgDialog::onResize(GtkAllocation *all) {
}

gint GradientListOrgDialog::hideWindow(GdkEventAny *ev) {
	//g_printerr("GradientListOrgDialog::hideWindow\n");
	application->hideGradients();
	return 1;
}

void GradientListOrgDialog::newClicked() {
	currentGradient = NULL;
	inEdit = new Gradient();
	gchar number[20];
	snprintf(number, 18, "%d", untitledNumber++);
	string tmpname = string(_("untitled-"));
	tmpname.append(number);
	inEdit->setName(tmpname);
	GDColor white, black;
	white.setRGB(0xff, 0xff, 0xff);
	black.setRGB(0x00, 0x00, 0x00);
	ColorPoint cpwhite = { 1.0, white };
	ColorPoint cpblack = { 0.0, black };
	inEdit->colorPoints.push_back(cpblack);
	inEdit->colorPoints.push_back(cpwhite);
	showEditDialog(inEdit);
}

void GradientListOrgDialog::deleteClicked() {
	if (currentGradient && doc) {
		// check if this gradient is used in the document
		GDResourceList resList;
		doc->gatherResource(resList, GDRESOURCE_TYPE_GRADIENT);
		if (resList.count(currentGradient) < 1) {
			doc->gradients.erase(currentGradient->name);
			delete currentGradient;
			updateList(doc);
		}
	}
}

void GradientListOrgDialog::editClicked() {
	if (currentGradient) {
		inEdit = new Gradient(*currentGradient);
		showEditDialog(inEdit);
	}
}

void GradientListOrgDialog::closeDialog() {
	//g_printerr("GradientListOrgDialog::closeDialog\n");
	application->hideGradients();
}

void GradientListOrgDialog::showEditDialog(Gradient *toEdit) {
	editGradientDlg = new EditGradientDialog(toEdit);
	editGradientDlg->getOKButton()->clicked.connect(SigC::slot(this, &GradientListOrgDialog::saveChangedGradient));
	editGradientDlg->getCancelButton()->clicked.connect(SigC::slot(this, &GradientListOrgDialog::cancelEditGradientDialog));
	editGradientDlg->delete_event.connect(SigC::slot(this, &GradientListOrgDialog::deleteEditGradientDialog));
	editGradientDlg->set_modal(true);
}

gint GradientListOrgDialog::deleteEditGradientDialog(GdkEventAny *ev) {
	cancelEditGradientDialog();
}

void GradientListOrgDialog::cancelEditGradientDialog() {
	editGradientDlg->hide();
	
}

void GradientListOrgDialog::saveChangedGradient() {
	if (doc) {
		editGradientDlg->hide();
		if (currentGradient) {
			doc->gradients.erase(currentGradient->name);
			*currentGradient = *inEdit;
			doc->gradients[currentGradient->name] = currentGradient;
			delete inEdit;
		} else {
			doc->gradients[inEdit->name] = inEdit;
			currentGradient = inEdit;
			inEdit = NULL;
		}
		updateList(doc);
	}
}



void GradientListOrgDialog::setDocument(GDDocument *document) {
	ListOrgDialog::setDocument(document);
	updateList(document);
}

void GradientListOrgDialog::updateList(GDDocument *document) {
	if (! document) document = doc;
	if (! document) return;
	objectList->clear();
	gint count = 0;
	gint selected = -1;
	if (! document) return;
	for (GradientMap::iterator it = document->gradients.begin(); it != document->gradients.end(); it++) {
		char *text[] = { "", "" };
		if ((*it).second == currentGradient) {
			selected = count;
		}
		count++;
		int row = objectList->append(text);
		Gdk_Bitmap previewMask;
		Gdk_Pixmap previewIcon;
		createGradientPreview((*it).second, previewIcon, previewMask, document);
		objectList->cell(row, 0).set_pixmap(previewIcon, previewMask);
		objectList->cell(row, 1).set_text((*it).first);
	}
	if (selected >= 0) {
		objectList->row(selected).select();
	}
}

Gdk_Pixmap GradientListOrgDialog::createGradientPreview(Gradient *gradient, Gdk_Pixmap& previewPixmap, Gdk_Bitmap& previewMask, GDDocument *document) {
	gchar *colorchars = "+@#$%&*=-;>,')!~{]^/(_:<[}|1234567890abcdefghijklm";
	gchar colors[50][15];
	ColorPoint lowCP = gradient->colorPoints[0];
	ColorPoint hiCP = gradient->colorPoints[0];
	gdouble cindex = 0.0;
	guint cpindex = 0;
	guint rl, rh, gl, gh, bl, bh;
	lowCP.color.getRGBColor(rl, gl, bl);
	hiCP.color.getRGBColor(rh, gh, bh);
	//g_printerr("GradientListOrgDialog::createGradientPreview lc=%X %X %X hc=%X %X %X\n", rl, gl, bl, rh, gh, bh);
	for (int i = 0; i < 50; i++) {
		if (hiCP.pos < cindex) {
			lowCP = hiCP;
			cpindex++;
			hiCP = gradient->colorPoints[cpindex];
			rl = rh; gl = gh; bl = bh;
			hiCP.color.getRGBColor(rh, gh, bh);
			//g_printerr("GradientListOrgDialog::createGradientPreview lc=%X %X %X hc=%X %X %X\n", rl, gl, bl, rh, gh, bh);
		}
		gdouble cf = (hiCP.pos == lowCP.pos) ? cindex : (cindex - lowCP.pos) / (hiCP.pos - lowCP.pos);
		gint r = (gint) rl + (gint) ( ((gdouble) rh - (gdouble) rl) * cf );
		gint g = (gint) gl + (gint) ( ((gdouble) gh - (gdouble) gl) * cf );
		gint b = (gint) bl + (gint) ( ((gdouble) bh - (gdouble) bl) * cf );
		snprintf(colors[i], sizeof(colors[i]), "%c\tc #%02X%02X%02X", colorchars[i], r, g, b);
		//g_printerr("color: %s, cf=%f, cind=%f, i=%d\n", colors[i], cf, cindex, i);
		gradient_preview_xpm[3 + i] = colors[i];
		cindex += 1.0 / 49.0;
	}
	Gdk_Color transparentColor;
	previewPixmap.create_from_xpm_d(document->application->toolWindow->get_window(), previewMask, transparentColor, gradient_preview_xpm);
	return previewPixmap;
}
