/* $Id: Layer.hh,v 1.17 2005/05/07 17:39:33 pklaus Exp $
 *
 * Layer.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef _LAYER_HH
#  define _LAYER_HH

#include "gestalter.h"
#include <glib--.h>
#include <gdk--.h>
#include <gtk--.h>
#include "DLList.hh"
#include "SelectionItemList.hh"
#include "GDResource.hh"
#include "DocWindow.hh"
#include "BoundingBox.hh"
#include <iostream>
#include <sigc++/object.h>

class SelectionItem;
class GObject;
class BezierPath;
class SelectableItem;
class GDDocument;

class Layer : public DLList<Layer>, public SigC::Object {

	string name;
	guint32 selectionColor;
	GDDocument *doc;
	
public:
	GObject *paths;
	gboolean viewable;
	gboolean editable;
	gboolean containsSelectedItems;
	
	Layer(GDDocument *document);
	Layer(Layer &fromLayer);
	
	guint32 getSelectionColor();
	void setSelectionColor(guint32 color);
	
	void setName(string name);
	
	string getName();
	
	void addObject(GObject *newPath);
	
	GObject *removeObject(GObject *object);
	
	gboolean findObjectLayer(GObject *object);

	gboolean getObjectNumber(GObject *object, guint& number);

	void draw(OffscreenBuffer& osBuffer, RectangleArea& drawArea);
	
	void drawWireFrame(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea);
	
	BoundingBox getBoundingBox();
	
	gdouble attractPoint(gdouble x, gdouble y, gdouble& ax, gdouble& ay, gdouble tolerance, int attractType = 0);

	SelectableItem *checkPoints(gdouble x, gdouble y, gdouble tolerance, gboolean onlyCurvePoints);

	GObject *checkArea(gdouble x, gdouble y);
	
	SelectableItem *checkCurves(gdouble x, gdouble y, gdouble tolerance);
	
	void selectRectangle(gdouble left, gdouble top, gdouble right, gdouble bottom, gint what);
	
	void deSelectRectangle(gdouble left, gdouble top, gdouble right, gdouble bottom, gint what);
	
	void selectAll();
	
	void deSelectAll();
	
	gint getSelection(SelectionItemList& selectionList, gint mode);
	
	void gatherResource(GDResourceList& collection, gint type);
	
	void setDocument(GDDocument *document);
	
	Layer *clone(gboolean copy = false);
	
	gint writeXML(ostream& xmlout);
	
	gint writePS(ostream& psout);
	
	~Layer();
};

#endif
