/* $Id: LibXML.hh,v 1.1 2003/11/01 15:25:02 pklaus Exp $
 *
 * LibXML.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef _LIBXML_HH
#  define _LIBXML_HH

#include <string>
#include <sigc++/object.h>
#include <sigc++/basic_signal.h>
#include "gestalter.h"
#include <libxml/parser.h>

#define MAX_TAG_SIZE	512

void wrapbegintag(void *mydata, const xmlChar *name, const xmlChar **attrs);
void wrapendtag(void *mydata, const xmlChar *name);
void wraptextdata(void *mydata, const xmlChar *text, int len);

class LibXMLObject;

typedef struct _parser_data {
	LibXMLObject *xmlObject;
} parser_data;


class GObject;

class LibXMLObject : public SigC::Object {
	
protected:
	string tagname;
	parser_data *parser;

public:
	LibXMLObject *parent;
	SigC::Signal1<void, GObject *> addObject;
	
	
	LibXMLObject(LibXMLObject *p, const char *tag, parser_data *parser, const char **attrs=NULL);
	LibXMLObject();
	
	void init();
	void setParser(parser_data *p) { parser = p; };
	
	// working methods
	virtual void handleBeginTag(const char *name, const char **attrs);
	virtual void handleEndTag(const char *name);
	virtual void handleText(const char *text, int len);
	
	char *getAttributeValue(const char **attrs, const char *key);
	int getAttributeValue(const char **attrs, const char *key, int& value);
	int getAttributeValue(const char **attrs, const char *key, float& value);
	int getAttributeValue(const char **attrs, const char *key, double& value);
	int getAttributeValue(const char **attrs, const char *key, char *value, int len);
	
};

class LibXML {
	LibXMLObject *rp;
	const char *errstring;
	
public:
	LibXML();
	
	int parse(LibXMLObject *rootParser, string filePath);
	
	~LibXML();
};

#endif
