/* $Id: MoveDialog.cc,v 1.4 2004/03/24 19:31:43 pklaus Exp $
 *
 * MoveDialog.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "MoveDialog.hh"
#include "GDApplication.hh"
#include <math.h>
#include <stdio.h>

MoveDialog::MoveDialog(GDApplication *application) {
	app = application;
	axisDisabled = false;
	polarDisabled = false;
}

string MoveDialog::doubleToString(gdouble val) {
	char buffer[40];
	snprintf(buffer, 39, "%f", val);
	return string(buffer);
}

void MoveDialog::recalcPolar() {
	if (axisDisabled) return;
	polarDisabled = true;
	gdouble hm = getHorizontalTranslation();
	gdouble vm = getVerticalTranslation();
	gdouble distance = sqrt(hm * hm + vm * vm); // pythagoras
	gdouble angle;
	gdouble ahm = fabs(hm);
	gdouble avm = fabs(vm);
	if (ahm < 0.000001 && avm < 0.000001) {
		angle = 0;
	} else if (ahm > avm) {
		angle = atan(avm / ahm);
	} else {
		angle = M_PI_2 - atan(ahm / avm);
	}
	angle = angle * 180 / M_PI;
	if (hm < 0 && vm > 0) {
		angle = 180 - angle;
	} else if (hm < 0 && vm < 0) {
		angle += 180;
	} else if (hm > 0 && vm < 0) {
		angle = 360 - angle;
	}
	distanceEntry->set_text(doubleToString(distance));
	angleEntry->set_text(doubleToString(angle));
	polarDisabled = false;
}

void MoveDialog::recalcAxis() {
	if (polarDisabled) return;
	axisDisabled = true;
	gdouble dist = app->preferences.unitStrToDouble(distanceEntry->get_text());
	gdouble angle = strtod(angleEntry->get_text().c_str(), NULL);
	angle = angle * M_PI / 180;
	gdouble h = cos(angle) * dist;
	gdouble v = sin(angle) * dist;
	horizontalEntry->set_text(doubleToString(h));
	verticalEntry->set_text(doubleToString(v));
	axisDisabled = false;
}

gdouble MoveDialog::getHorizontalTranslation() {
	return app->preferences.unitStrToDouble(horizontalEntry->get_text());
}

gdouble MoveDialog::getVerticalTranslation() {
	return app->preferences.unitStrToDouble(verticalEntry->get_text());
}
