/* $Id: ObjectGroup.hh,v 1.15 2004/07/21 06:31:20 pklaus Exp $
 *
 * ObjectGroup.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef _OBJECTGROUP_HH
#  define _OBJECTGROUP_HH

#include "GObject.hh"
#include "DrawObject.hh"
#include <glib--.h>
#include <gdk--.h>
#include <gtk--.h>
#include <libart_lgpl/art_vpath.h>

class BezierNode;
class BoundingBox;
class SelectionItem;
class GDDocument;
class GDStyle;
class SelectableItem;

class ObjectGroup : public GObject {


public:
	GObject *parts;
	gboolean showCenter;
	GDStyle *drawStyle;
	GDStyle *fillStyle;


	ObjectGroup(GDDocument *document);
	
	void addPart(GObject *newObject);
	void removePart(GObject *object);
	
	// DrawObject methods
	virtual void drawWithSelection(OffscreenBuffer& osBuffer, RectangleArea& drawArea);
	virtual void drawWireFrameSelection(DocWindow *docwin, RectangleArea& drawArea);
	virtual GDStyle *getStrokeStyle();
	virtual GDStyle *getFillStyle();
	virtual void setStrokeStyle(const GDStyle &from);
	virtual void setFillStyle(const GDStyle &from);
	virtual void gatherResource(GDResourceList& collection, gint type);
	
	// GObject methods
	virtual void draw(OffscreenBuffer& osBuffer, RectangleArea& drawArea);
	virtual void drawBlack(OffscreenBuffer& osBuffer, RectangleArea& drawArea);
	virtual void fillBlack(OffscreenBuffer& osBuffer, RectangleArea& drawArea);
	virtual void drawWireFrame(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea);
	virtual void drawPoints(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea);
	virtual BezierPathVector *getBezierPaths();
	virtual BoundingBox getBoundingBox();
	virtual gdouble attractPoint(gdouble x, gdouble y, gdouble& ax, gdouble& ay, gdouble tolerance, int attractType = 0);
	virtual SelectableItem *checkPoints(gdouble x, gdouble y, gdouble tolerance, gboolean onlyCurvePoints);
	virtual GObject *checkArea(gdouble x, gdouble y);
	virtual SelectableItem *checkCurves(gdouble x, gdouble y, gdouble tolerance);
	virtual void selectRectangle(gdouble left, gdouble top, gdouble right, gdouble bottom, gint what);
	virtual void deSelectRectangle(gdouble left, gdouble top, gdouble right, gdouble bottom, gint what);
	virtual void selectAll();
	virtual void deSelectAll();
	virtual GObject *findObject(GObject *object);
	virtual GObject *removeObject(GObject *object);
	virtual gint getSelection(SelectionItemList& selectionList, gint mode);
	virtual gint writeXML(ostream& xmlout);
	virtual void writePSPathDescription(ostream& psout);
	virtual gint writePS(ostream& psout);
	virtual void reverseOrder();

	// SelectableItem methods
	virtual void drawSelection(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea);
	virtual void transformByMatrix(TransformationMatrix& m, gboolean incFillPattern = TRUE);
	virtual void setFillMatrix(TransformationMatrix& m);
	virtual SelectableItem *clone();
	virtual void setCoords(SelectableItem* from);
	virtual void setDocument(GDDocument *document);
	virtual gboolean selected();
	virtual void clear();
	virtual gchar *identify();

	~ObjectGroup();
};

#endif
