/* $Id: Pattern.cc,v 1.4 2004/03/23 11:42:41 pklaus Exp $
 *
 * Pattern.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002-2004 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "Pattern.hh"
#include "ClipGroup.hh"
#include "GDDocument.hh"

Pattern::Pattern(ClipGroup *cont, gdouble w, gdouble h, gdouble f) : GDResource(GDRESOURCE_TYPE_PATTERN) {
	content = cont;
	width = w;
	height = h;
	origFactor = f;
}

Pattern::Pattern(Pattern& from) : GDResource(GDRESOURCE_TYPE_PATTERN) {
	content = (ClipGroup *) from.content->clone();
	width = from.width;
	height = from.height;
	name = from.name;
	origFactor = from.origFactor;
}

Pattern Pattern::operator=(Pattern& from) {
	if (content) {
		delete content;
	}
	content = (ClipGroup *) from.content->clone();
	width = from.width;
	height = from.height;
	name = from.name;
	origFactor = from.origFactor;
	return *this;
}

int Pattern::operator==(Pattern& b) {
	if (content != b.content) return 0;
	if (name != b.name) return 0;
	if (width != b.width || height != b.height) return 0;
	if (origFactor != b.origFactor) return 0;
	return 1;
}

gint Pattern::writePS(ostream& psout) {
	psout << "/pattern_" << ((long) this) << "\n";
	psout << "<< /PatternType 1\n";
	psout << "\t/BBox [ 0 0 " << width << " " << height << " ]\n";
	psout << "\t/XStep " << width << "\n";
	psout << "\t/YStep " << height << "\n";
	psout << "\t/PaintType 1\n";
	psout << "\t/TilingType 1\n";
	psout << "\t/PaintProc\n";
	psout << "\t\t{\n";
	psout << "\t\t\tbegin\n";
	GObject *part = content->parts;
	GDDocument *doc = NULL;
	position docOrigin;
	position docPF;
	gdouble docPH = 0;
	if (part) {
		doc = part->doc;
		docOrigin = doc->nullPoint;
		doc->nullPoint.x = 0;
		doc->nullPoint.y = 0;
		docPF = doc->printFactor;
		doc->printFactor.x = 1.0;
		doc->printFactor.y = 1.0;
		docPH = doc->pageHeight;
		doc->pageHeight = height;
	}
	while (part) {
		part->writePS(psout);
		part = part->next;
	}
	if (doc) {
		doc->nullPoint = docOrigin;
		doc->printFactor = docPF;
		doc->pageHeight = docPH;
	}	
	psout << "\t\t\tend\n";
	psout << "\t\t} bind\n";
	psout << ">>\n";
	psout << "def\n";
	
	return 0;
}

gint Pattern::writeXML(ostream& xmlout) {
	xmlout << "<pattern id=\"pattern_" << (long) this << "\" patternUnits=\"userSpaceOnUse\"";
	xmlout << " patternContentUnits=\"userSpaceOnUse\"";
	xmlout << " gestalter:name=\"" << name << "\"";
	xmlout << " gestalter:origFactor=\"" << origFactor << "\"";
	xmlout << " width=\"" << width << "\" height=\"" << height << "\"";
	xmlout << ">\n";
	GObject *part = content->parts;
	GDDocument *doc = NULL;
	position docOrigin;
	if (part) {
		doc = part->doc;
		docOrigin = doc->nullPoint;
		doc->nullPoint.x = 0;
		doc->nullPoint.y = 0;
	}
	while (part) {
		part->writeXML(xmlout);
		part = part->next;
	}
	if (doc) {
		doc->nullPoint = docOrigin;
	}
	xmlout << "</pattern>\n";
	return 0;
}

void Pattern::setDocument(GDDocument *document) {
	if (content) {
		content->setDocument(document);
	}
}

Pattern::~Pattern() {
	if (content) {
		delete content;
		content = NULL;
	}
}
