/* $Id: PixelImage.cc,v 1.17 2004/07/21 06:31:20 pklaus Exp $
 *
 * PixelImage.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "PixelImage.hh"
#include "BezierNode.hh"
#include "GDDocument.hh"
#include "DocWindow.hh"
#include "Layer.hh"
#include "BoundingBox.hh"
#include "PixelImageData.hh"
#include "GDApplication.hh"
#include <libart_lgpl/art_rgb_rgba_affine.h>
#include <libart_lgpl/art_svp.h>
#include <libart_lgpl/art_svp_vpath.h>
#include <libart_lgpl/art_rgb_svp.h>
#include <libart_lgpl/art_svp_vpath_stroke.h>
#include <math.h>

PixelImage::PixelImage(GDDocument *document) : GObject(document) {
	imageData = NULL;
	this->init();
}

PixelImage::PixelImage(GDDocument *document, PixelImageData *image) : GObject(document) {
	imageData = image;
	this->init();
}

PixelImage::PixelImage(string filename, GDDocument *document) : GObject(document) {
	imageData = new PixelImageData(filename);
	this->init();
}

void PixelImage::init() {
	// select(false); // commented out for code cleanup
	level = 2;
	lastTarget = 0;
	calcCorners();
}

void PixelImage::draw(OffscreenBuffer& osBuffer, RectangleArea& drawArea) {
	//g_printerr("PixelImage::draw\n");
	BoundingBox bbox = getBoundingBox();
	if (drawArea.intersect(bbox) && imageData->data) {
		TransformationMatrix tmp = tMatrix;
		TransformationMatrix t;
		t.setTranslation(- osBuffer.origin.x, - osBuffer.origin.y);
		tmp *= t;
		t.reset();
		t.setScale(osBuffer.scaleFactor, osBuffer.scaleFactor);
		tmp *= t;
		double affine[6];
		affine[0] = tmp.m[0][0];
		affine[1] = tmp.m[1][0];
		affine[2] = tmp.m[0][1];
		affine[3] = tmp.m[1][1];
		affine[4] = tmp.m[0][2];
		affine[5] = tmp.m[1][2];
		art_rgb_rgba_affine(osBuffer.pixbuffer, 0, 0, 
							osBuffer.pxbWidth,
							osBuffer.pxbHeight,
							osBuffer.pxbRowstride,
							(art_u8 *) imageData->data,
							imageData->width, imageData->height,
							imageData->rowstride,
							affine, ART_FILTER_NEAREST, NULL);
	}
	//g_printerr("PixelImage::draw finished\n");
}

void PixelImage::drawBlack(OffscreenBuffer& osBuffer, RectangleArea& drawArea) {
	//g_printerr("PixelImage::drawBlack %lX\n", this);
	BoundingBox bbox = getBoundingBox();
	if (drawArea.intersect(bbox) && imageData->data) {
		TransformationMatrix tmp = tMatrix;
		TransformationMatrix t;
		t.setTranslation(- osBuffer.origin.x, - osBuffer.origin.y);
		tmp *= t;
		t.reset();
		t.setScale(osBuffer.scaleFactor, osBuffer.scaleFactor);
		tmp *= t;
		double affine[6];
		affine[0] = tmp.m[0][0];
		affine[1] = tmp.m[1][0];
		affine[2] = tmp.m[0][1];
		affine[3] = tmp.m[1][1];
		affine[4] = tmp.m[0][2];
		affine[5] = tmp.m[1][2];
		guint bufsize = imageData->rowstride * imageData->height;
		guchar *blackData = (guchar *) g_malloc0(bufsize);
		if (! blackData) return;
		for (guint i = 3; i < bufsize; i+=4) {
			blackData[i] = imageData->data[i];
		}
		art_rgb_rgba_affine(osBuffer.pixbuffer, 0, 0, 
							osBuffer.pxbWidth,
							osBuffer.pxbHeight,
							osBuffer.pxbRowstride,
							(art_u8 *) blackData,
							imageData->width, imageData->height,
							imageData->rowstride,
							affine, ART_FILTER_NEAREST, NULL);
	}
}

void PixelImage::fillBlack(OffscreenBuffer& osBuffer, RectangleArea& drawArea) {
	//g_printerr("PixelImage::fillBlack %lX\n", this);
	// with pixel the same as drawBlack
	drawBlack(osBuffer, drawArea);
}

void PixelImage::drawWireFrame(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea) {
	int i;
	for (i = 0; i < 4; i++) {
		//drawPoint(docwin, context, corner[i]);
		docwin->DocWinDrawingArea->get_window().draw_line(context,
			(int) ((corner[i].x - docwin->osBuffer.origin.x) * docwin->osBuffer.scaleFactor), 
			(int) ((corner[i].y - docwin->osBuffer.origin.y) * docwin->osBuffer.scaleFactor), 
			(int) ((corner[(i+1)%4].x - docwin->osBuffer.origin.x) * docwin->osBuffer.scaleFactor), 
			(int) ((corner[(i+1)%4].y - docwin->osBuffer.origin.y) * docwin->osBuffer.scaleFactor));
	}
	docwin->DocWinDrawingArea->get_window().draw_line(context,
			(int) ((corner[0].x - docwin->osBuffer.origin.x) * docwin->osBuffer.scaleFactor), 
			(int) ((corner[0].y - docwin->osBuffer.origin.y) * docwin->osBuffer.scaleFactor), 
			(int) ((corner[2].x - docwin->osBuffer.origin.x) * docwin->osBuffer.scaleFactor), 
			(int) ((corner[2].y - docwin->osBuffer.origin.y) * docwin->osBuffer.scaleFactor));
	docwin->DocWinDrawingArea->get_window().draw_line(context,
			(int) ((corner[1].x - docwin->osBuffer.origin.x) * docwin->osBuffer.scaleFactor), 
			(int) ((corner[1].y - docwin->osBuffer.origin.y) * docwin->osBuffer.scaleFactor), 
			(int) ((corner[3].x - docwin->osBuffer.origin.x) * docwin->osBuffer.scaleFactor), 
			(int) ((corner[3].y - docwin->osBuffer.origin.y) * docwin->osBuffer.scaleFactor));
	
}

void PixelImage::drawWireFrameSelection(DocWindow *docwin, RectangleArea& drawArea) {
	Layer *myLayer = docwin->doc->findObjectLayer(this);
	guint color = 0xFFFFFFFF;
	if (myLayer) {
		color = myLayer->getSelectionColor();
	} else {
		g_printerr("PixelImage::drawWireFrameSelection: no layer found for path %lX\n", (long) this);
	}
	//g_printerr("PixelImage::drawWireFrameSelection got color\n");
	Gdk_Colormap cmap = Gdk_Colormap::get_system();
	Gdk_Color gcolor;
	gcolor.set_rgb(((color >> 24) & 0xFF) << 8, 
					((color >> 16) & 0xFF) << 8,
					((color >> 8) & 0xFF) << 8);
	cmap.alloc(gcolor);
	Gdk_Window drawable = docwin->DocWinDrawingArea->get_window();
	Gdk_GC context = Gdk_GC(drawable);
	context.set_foreground(gcolor);
	context.set_line_width(1);
	context.set_function(GDK_COPY);
	drawWireFrame(docwin, context, drawArea);
	int i;
	for (i = 0; i < 4; i++) {
		drawPoint(docwin, context, corner[i]);
	}
}

void PixelImage::drawPoint(DocWindow *docwin, Gdk_GC& context, position point) {
	if (selected()) {
		docwin->DocWinDrawingArea->get_window().draw_rectangle(context, 1, 
		ROUND((point.x - docwin->osBuffer.origin.x) * docwin->osBuffer.scaleFactor) - BEZIER_HALFSIZE,
		ROUND((point.y - docwin->osBuffer.origin.y) * docwin->osBuffer.scaleFactor) - BEZIER_HALFSIZE,
						BEZIER_SIZE, BEZIER_SIZE);
	}
}	

void PixelImage::drawPoints(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea) {
	//g_printerr("PixelImage::drawPoints\n");
	int i;
	for (i = 0; i < 4; i++) {
		drawPoint(docwin, context, corner[i]);
		docwin->DocWinDrawingArea->get_window().draw_line(context,
			(int) ((corner[i].x - docwin->osBuffer.origin.x) * docwin->osBuffer.scaleFactor), 
			(int) ((corner[i].y - docwin->osBuffer.origin.y) * docwin->osBuffer.scaleFactor), 
			(int) ((corner[(i+1)%4].x - docwin->osBuffer.origin.x) * docwin->osBuffer.scaleFactor), 
			(int) ((corner[(i+1)%4].y - docwin->osBuffer.origin.y) * docwin->osBuffer.scaleFactor));
	}
}

BoundingBox PixelImage::getBoundingBox() {
	gdouble l,t,r,b;
	BoundingBox boundingBox;
	
	/* initialize the values */
	l = G_MAXDOUBLE;
	t = G_MAXDOUBLE;
	r = G_MINDOUBLE;
	b = G_MINDOUBLE;
	
	for (int i = 0; i < 4; i++) {
		if (corner[i].x < l) l = corner[i].x;
		if (corner[i].x > r) r = corner[i].x;
		if (corner[i].y < t) t = corner[i].y;
		if (corner[i].y > b) b = corner[i].y;
	}
	boundingBox.left = l;
	boundingBox.top = t;
	boundingBox.right = r;
	boundingBox.bottom = b;
	return boundingBox;
}

gdouble PixelImage::attractPoint(gdouble x, gdouble y, gdouble& ax, gdouble& ay, gdouble tolerance, int attractType) {
	gdouble nearest = G_MAXDOUBLE;
	for (int i = 0; i < 4; i++) {
		gdouble dx = fabs(corner[i].x - x);
		gdouble dy = fabs(corner[i].y - y);
		if (dx < tolerance && dy < tolerance) {
			if (dx * dx + dy * dy < nearest) {
				nearest = dx * dx + dy * dy;
				ax = corner[i].x;
				ay = corner[i].y;
			}
		}
	}
	return nearest;
}

SelectableItem *PixelImage::checkPoints(gdouble x, gdouble y, 
								gdouble tolerance, gboolean onlyCurvePoints) {
	for (int i = 0; i < 4; i++) {
		//g_printerr("PixelImage::checkPoints check corner %d: %f,%f\n", i, corner[i].x, corner[i].y);
		if (fabs(corner[i].x - x) < tolerance && fabs(corner[i].y - y) < tolerance) {
			lastTarget = i;
			return this;
		}
	}
	return NULL;
}

GObject *PixelImage::checkArea(gdouble x, gdouble y) {
	//g_printerr("PixelImage::checkArea\n");
	position check;
	check.x = x; check.y = y;
	invTMatrix.transform(check);
	//g_printerr("PixelImage::checkArea backtransformed: %f, %f\n", check.x, check.y);
	if (check.x > 0 && check.x < imageData->width && check.y > 0 && check.y < imageData->height) {
		return this;
	}
	return NULL;
}


SelectableItem *PixelImage::checkCurves(gdouble x, gdouble y, gdouble tolerance) {
	//g_printerr("PixelImage::checkCurves\n");
	position check;
	check.x = x; check.y = y;
	invTMatrix.transform(check);
	if (((fabs(check.x) < tolerance || fabs(imageData->width - check.x) < tolerance) && 
		check.y > -tolerance && check.y < imageData->height + tolerance) ||
		((fabs(check.y) < tolerance || fabs(imageData->height - check.y) < tolerance) &&
		check.x > -tolerance && check.x < imageData->width + tolerance)) {
		return this;
	}
	return NULL;
}


void PixelImage::selectRectangle(gdouble left, gdouble top, gdouble right, gdouble bottom, gint what) {
	//g_printerr("PixelImage::selectRectangle\n");
	for (int i = 0; i < 4; i++) {
		if (corner[i].x >= left && corner[i].x <= right &&
				corner[i].y >= top && corner[i].y <= bottom) {
			if ((what & 3) == 2) {
				selectObject((what & 128) != 0);
			} else {
				select((what & 128) != 0);
			}
			return;
		}
	}
}


void PixelImage::deSelectRectangle(gdouble left, gdouble top, gdouble right, gdouble bottom, gint what) {
	for (int i = 0; i < 4; i++) {
		if (corner[i].x >= left && corner[i].x <= right &&
				corner[i].y >= top && corner[i].y <= bottom) {
			if ((what & 3) == 2) {
				deSelectObject();
			} else {
				deSelect();
			}
			return;
		}
	}
}


GObject *PixelImage::removeObject(GObject *object) {
	if (object == this) {
		if (next) next->prev = prev;
		if (prev) prev->next = next;
		return this;
	}
	return NULL;
}


GObject *PixelImage::findObject(GObject *object) {
	if (object == this) {
		return this;
	}
	return NULL;
}


void PixelImage::selectAll() {
	//g_printerr("PixelImage::selectAll\n");
	selection = true;
	doc->selection->insertObject(this);
}


void PixelImage::deSelectAll() {
	//g_printerr("PixelImage::deSelectAll\n");
	selection = false;
	doc->selection->removeObject(this);
}

gboolean PixelImage::selected() {
	return selection;
}

gint PixelImage::getSelection(SelectionItemList& selectionList, gint mode) {
	//g_printerr("PixelImage::getSelection\n");
	if (selected()) {
		selectionList.insert(selectionList.end(), new SelectionItem(this, mode));
		return 1;
	}
	return 0;
}


gint PixelImage::writeXML(ostream& xmlout) {
	TransformationMatrix tmp = tMatrix;
	TransformationMatrix t;
	t.setTranslation(-doc->nullPoint.x, -doc->nullPoint.y);
	tmp *= t;
	xmlout << "<image";
	xmlout << " width=\"" << imageData->width << "\"";
	xmlout << " height=\"" << imageData->height << "\"";
	xmlout << " transform=\"";
	tmp.writeXML(xmlout);
	xmlout << "\">\n";
	imageData->writeXML(xmlout);
	xmlout << "</image>\n";
	return 0;
}


gint PixelImage::writePS(ostream& psout) {
	// calc the matrix
	TransformationMatrix tmp = tMatrix;
	TransformationMatrix t;
	// translate to doc origin
	t.reset();
	t.setTranslation(-doc->nullPoint.x, -doc->nullPoint.y);
	tmp *= t;
	// change the coordinate system to postscript
	// flip on the x-axis
	t.reset();
	t.m[1][1] = -1;
	tmp *= t;
	// move by page height
	t.reset();
	t.setTranslation(0, doc->pageHeight);
	tmp *= t;
	psout << "gs\n";
	if (level >= 2) {
		psout << "/DeviceRGB setcolorspace\n";
		tmp.writePS(psout);
		psout << " concat\n";
		psout << "<<\n";
		psout << "\t/ImageType 3\n";
		psout << "\t/DataDict\n";
		psout << "\t<<\n";
		psout << "\t\t/ImageType 1\n";
		psout << "\t\t/Width " << imageData->width << "\n";
		psout << "\t\t/Height " << imageData->height << "\n";
		psout << "\t\t/BitsPerComponent 8\n";
		psout << "\t\t/Decode [0 1 0 1 0 1]\n";
		psout << "\t\t/ImageMatrix [1 0 0 1 0 0]\n";
		psout << "\t\t/DataSource currentfile /ASCIIHexDecode filter\n";
		psout << "\t>>\n";
		psout << "\t/MaskDict\n";
		psout << "\t<<\n";
		psout << "\t\t/ImageType 1\n";
		psout << "\t\t/Width " << imageData->width << "\n";
		psout << "\t\t/Height " << imageData->height << "\n";
		psout << "\t\t/BitsPerComponent 8\n";
		psout << "\t\t/Decode [1 0]\n";
		psout << "\t\t/ImageMatrix [1 0 0 1 0 0]\n";
		psout << "\t>>\n";
		psout << "\t/InterleaveType 1\n";
		psout << ">>\n";
		psout << "image\n";
		imageData->writePS(psout);
	} else if (level < 1) {
		psout << "/DeviceRGB setcolorspace\n";
		tmp.writePS(psout);
		psout << "		concat\n";
		psout << "<<\n";
		psout << "\t/ImageType 1\n";
		psout << "\t/Width " << imageData->width << "\n";
		psout << "\t/Height " << imageData->height << "\n";
		psout << "\t/BitsPerComponent 8\n";
		psout << "\t/Decode [0 1 0 1 0 1]\n";
		psout << "\t/ImageMatrix [1 0 0 1 0 0]\n";
		psout << "\t/DataSource currentfile /ASCIIHexDecode filter\n";
		psout << ">>\n";
		psout << "image\n";
		imageData->writePSColor(psout);
		psout << ">\n";		psout << ">\n";
	} else {
		psout << "/picstr " << (imageData->width * imageData->height * 3) << " string def\n";
		psout << "[" << tmp.m[0][0] << " " << tmp.m[1][0];
		psout << " " << tmp.m[0][1] << " " << tmp.m[1][1];
		psout << " " << tmp.m[0][2] << " " << tmp.m[1][2];
		psout << "] concat\n";
		psout << imageData->width << " " << imageData->height << " 8\n";
		psout << "[1 0 0 1 0 0]\n";
		psout << "{\tcurrentfile\n\tpicstr readhexstring pop\n}\n";
		psout << "false 3\n";
		psout << "colorimage\n";
		imageData->writePSColor(psout);
	}
	psout << "gr\n";
	return 0;
}

void PixelImage::bufferDrawPoint(OffscreenBuffer& osBuffer, position point, guint color) {
	//g_printerr("PixelImage::bufferDrawPoint\n");
	ArtVpath apath[6];
	ArtSVP *asvp;
	apath[0].code = ART_MOVETO;
	apath[0].x = ROUND((point.x - osBuffer.origin.x) * osBuffer.scaleFactor) - BEZIER_HALFSIZE;
	apath[0].y = ROUND((point.y - osBuffer.origin.y) * osBuffer.scaleFactor) - BEZIER_HALFSIZE;
	apath[1].code = ART_LINETO;
	apath[1].x = apath[0].x + BEZIER_SIZE;
	apath[1].y = apath[0].y;
	apath[2].code = ART_LINETO;
	apath[2].x = apath[1].x;
	apath[2].y = apath[1].y + BEZIER_SIZE;
	apath[3].code = ART_LINETO;
	apath[3].x = apath[0].x;
	apath[3].y = apath[2].y;
	apath[4].code = ART_LINETO;
	apath[4].x = apath[0].x;
	apath[4].y = apath[0].y;
	apath[5].code = ART_END;
	apath[5].x = 0;
	apath[5].y = 0;
	asvp = art_svp_from_vpath(apath);
	art_rgb_svp_alpha(asvp, 0, 0,
				osBuffer.pxbWidth,
				osBuffer.pxbHeight,
				color,
				osBuffer.pixbuffer,
				osBuffer.pxbRowstride,
				NULL);
	art_svp_free(asvp);	
}

void PixelImage::drawWithSelection(OffscreenBuffer& osBuffer, RectangleArea& drawArea) {
	//g_printerr("PixelImage::drawWithSelection\n");
	BoundingBox bbox = getBoundingBox();
	if (drawArea.intersect(bbox)) {
		Layer *myLayer = doc->findObjectLayer(this);
		guint color = 0xFFFFFFFF;
		if (myLayer) {
			color = myLayer->getSelectionColor();
		} else {
			g_printerr("PixelImage::drawWithSelection: no layer found for path %lX\n", (long) this);
		}
		ArtVpath apath[6];
		apath[0].code = ART_MOVETO;
		apath[0].x = (corner[0].x - osBuffer.origin.x) * osBuffer.scaleFactor;
		apath[0].y = (corner[0].y - osBuffer.origin.y) * osBuffer.scaleFactor;
		for (int i = 1; i < 5; i++) {
			apath[i].code = ART_LINETO;
			apath[i].x = (corner[i%4].x - osBuffer.origin.x) * osBuffer.scaleFactor;
			apath[i].y = (corner[i%4].y - osBuffer.origin.y) * osBuffer.scaleFactor;
			bufferDrawPoint(osBuffer, corner[i-1], color);
		}
		apath[5].code = ART_END;
		apath[5].x = 0;
		apath[5].y = 0;
		ArtSVP *asvp = art_svp_vpath_stroke(apath, 
									ART_PATH_STROKE_JOIN_MITER,
									ART_PATH_STROKE_CAP_BUTT,
									1, 4, 0.5);
		art_rgb_svp_alpha(asvp, 0, 0,
						osBuffer.pxbWidth,
						osBuffer.pxbHeight,
						color,
						osBuffer.pixbuffer,
						osBuffer.pxbRowstride,
						NULL);
		art_svp_free(asvp);
		//g_printerr("PixelImage::drawWithSelection finished\n");
	}
}


GDStyle *PixelImage::getStrokeStyle() {
	//g_printerr("PixelImage::getStrokeStyle\n");
	return NULL;
}


GDStyle *PixelImage::getFillStyle() {
	//g_printerr("PixelImage::getFillStyle\n");
	return NULL;
}


void PixelImage::drawSelection(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea) {
	//g_printerr("PixelImage::drawSelection\n");
	drawPoints(docwin, context, drawArea);
}

void PixelImage::calcCorners() {
	//g_printerr("PixelImage::calcCorners\n");
	gdouble w = width();
	gdouble h = height();
	corner[0].x = 0;
	corner[0].y = 0;
	corner[1].x = w;
	corner[1].y = 0;
	corner[2].x = w;
	corner[2].y = h;
	corner[3].x = 0;
	corner[3].y = h;
	for (int i = 0; i < 4; i++) {
		tMatrix.transform(corner[i]);
		//g_printerr("transformed corner %d (%f,%f)\n", i, corner[i].x, corner[i].y);
	}
	//g_printerr("PixelImage::calcCorners finished\n");
}

void PixelImage::transformByMatrix(TransformationMatrix& m, gboolean incFillPattern) {
	//g_printerr("PixelImage::transformByMatrix\n");
	tMatrix *= m;
	invTMatrix = tMatrix.inverse();
	calcCorners();
}


SelectableItem *PixelImage::clone() {
	//g_printerr("PixelImage::clone\n");
	PixelImage *newPImage = new PixelImage(doc);
	newPImage->imageData = imageData;
	imageData->incRefCount();
	newPImage->transformByMatrix(tMatrix);
	return newPImage;
}


void PixelImage::setCoords(SelectableItem* from) {
	//g_printerr("PixelImage::setCoords\n");
	tMatrix.reset();
	PixelImage *piFrom = (PixelImage *) from;
	transformByMatrix(piFrom->tMatrix);
}

int PixelImage::getCoordinates(gdouble& x, gdouble& y) {
	//g_printerr("PixelImage::getCoordinates lastTarget: %d\n", lastTarget);
	x = corner[lastTarget].x;
	y = corner[lastTarget].y;
	return 1;
}

void PixelImage::clear() {
	//g_printerr("PixelImage::clear\n");
	doc->removeObject(this);
	deSelectAll(); // remove from document's selection
	delete this;
}


gchar *PixelImage::identify() {
	return "PixelImage";
}

guint PixelImage::width() {
	if (imageData) {
		return imageData->width;
	}
	return 0;
}

guint PixelImage::height() {
	if (imageData) {
		return imageData->height;
	}
	return 0;
}

PixelImage::~PixelImage() {
	//deSelectAll(); // remove from document's selection (not necessary anymore)
	imageData->decRefCount();
}
