/* $Id: PixelImage.hh,v 1.11 2004/07/21 06:31:20 pklaus Exp $
 *
 * PixelImage.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef _PIXELIMAGE_HH
#  define _PIXELIMAGE_HH

#include <gdk--.h>
#include "GObject.hh"
#include "TransformationMatrix.hh"
#include "gestalter.h"

class TransformationMatrix;
class BoundingBox;
class SelectableItem;
class BezierPath;
class DocWindow;
class GDStyle;
class SelectionItemList;
class GDDocument;
class PixelImageData;

class PixelImage : public GObject {

	PixelImageData *imageData;
	position corner[4];
	TransformationMatrix tMatrix;
	TransformationMatrix invTMatrix;
	
	gboolean selection;
	
	guint lastTarget;
	guint level;
	
	void init();
	void drawPoint(DocWindow *docwin, Gdk_GC& context, position point);
	void bufferDrawPoint(OffscreenBuffer& osBuffer, position point, guint color);
	void calcCorners();

public:

	PixelImage(GDDocument *document);
	PixelImage(GDDocument *document, PixelImageData *image);
	PixelImage(string filename, GDDocument *document);
	
	// GObject methods
	void draw(OffscreenBuffer& osBuffer, RectangleArea& drawArea);
	void drawBlack(OffscreenBuffer& osBuffer, RectangleArea& drawArea);
	void fillBlack(OffscreenBuffer& osBuffer, RectangleArea& drawArea);
	void drawWireFrame(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea);
	void drawPoints(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea);
	BoundingBox getBoundingBox();
	gdouble attractPoint(gdouble x, gdouble y, gdouble& ax, gdouble& ay, gdouble tolerance, int attractType = 0);
	SelectableItem *checkPoints(gdouble x, gdouble y, gdouble tolerance, gboolean onlyCurvePoints);
	GObject *checkArea(gdouble x, gdouble y);
	SelectableItem *checkCurves(gdouble x, gdouble y, gdouble tolerance);
	void selectRectangle(gdouble left, gdouble top, gdouble right, gdouble bottom, gint what);
	void deSelectRectangle(gdouble left, gdouble top, gdouble right, gdouble bottom, gint what);
	GObject *removeObject(GObject *object);
	GObject *findObject(GObject *object);
	void selectAll();
	void deSelectAll();
	gint getSelection(SelectionItemList& selectionList, gint mode);
	gint writeXML(ostream& xmlout);
	gint writePS(ostream& psout);
	void reverseOrder() {};

	// DrawObject methods
	void drawWithSelection(OffscreenBuffer& osBuffer, RectangleArea& drawArea);
	void drawWireFrameSelection(DocWindow *docwin, RectangleArea& drawArea);
	GDStyle *getStrokeStyle();
	GDStyle *getFillStyle();
	
	// SelectableItem methods
	void drawSelection(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea);
	void transformByMatrix(TransformationMatrix& m, gboolean incFillPattern = TRUE);
	void setFillMatrix(TransformationMatrix& m) {};
	SelectableItem *clone();
	void setCoords(SelectableItem* from);
	gboolean selected();
	int getCoordinates(gdouble& x, gdouble& y);
	void clear();
	gchar *identify();
	
	guint width();
	guint height();
	
	~PixelImage();

};

#endif
