/* $Id: Properties.cc,v 1.18 2004/07/21 06:31:20 pklaus Exp $
 *
 * Properties.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#include "config.h"
#include "Properties.hh"
#include "GDApplication.hh"
#include "FillColorDisplay.hh"
#include "StrokeColorDisplay.hh"
#include "GradientListSelectionDialog.hh"
#include "PatternListSelectionDialog.hh"
#include "Pattern.hh"
#include "GDDocument.hh"
#include "GDStyle.hh"
#include "AttributeAction.hh"
#include "BoundingBox.hh"
#include <gtk--.h>
#include <math.h>
#include <libgnome/libgnome.h> // for _() macro

Properties::Properties(GDApplication *gda) {
	application = gda;
	OpacityScale->set_adjustment(OpacitySpinbutton->get_adjustment());
	fcd = manage(new class FillColorDisplay());
	FillButton->add(*fcd);
	scd = manage(new class StrokeColorDisplay());
	StrokeButton->add(*scd);
	fcd->show();
	scd->show();
	csd = NULL;
	doc = NULL;
	update_active = true;
	OpacityScale->set_state(GTK_STATE_INSENSITIVE);
	OpacitySpinbutton->set_state(GTK_STATE_INSENSITIVE);
	dashEntries[0] = dash1Entry;
	dashEntries[1] = dash2Entry;
	dashEntries[2] = dash3Entry;
	dashEntries[3] = dash4Entry;
	dashEntries[4] = dash5Entry;
	dashEntries[5] = dash6Entry;
	strokeWidthConnection = StrokeWeightSpinbutton->get_adjustment()->value_changed.connect(SigC::slot(this, &Properties::strokeWidthChanged));
}

void Properties::setDocument(GDDocument *document) {
	//g_printerr("Properties::setDocument  document = %lX\n", document);
	if (doc != document) {
		//g_printerr("Properties::setDocument  document = %lX (doc: %lX)\n", document, doc);
		doc = document;
		//g_printerr("Properties::setDocument  should be equal: document = %lX (doc: %lX)\n", document, doc);
		if (selConnection.connected()) {
			selConnection.disconnect();
		}
		if (document) {
			document->selection->fillProperties(this);
			selConnection = doc->selection->changed.connect(SigC::slot(this, &Properties::updateValues));
		}
	}
}

void Properties::updateValues(GDDocument *document) {
	//g_printerr("Properties::updateValues\n");
	if (document) {
		document->selection->fillProperties(this);
	}
}

void Properties::strokeWidthChanged() {
	gdouble newWidth = StrokeWeightSpinbutton->get_adjustment()->get_value();
	gdouble oldWidth = style[GDPROPERTIES_STROKE].strokeWidth;
	if (oldWidth != newWidth) {
		style[GDPROPERTIES_STROKE].strokeWidth = newWidth;
		if (oldWidth >= 0 && doc) {
			AttributeAction *newAction = new AttributeAction(ATTRIBUTEACTION_TYPE_STROKEWIDTH, newWidth);
			doc->setAction(newAction);
			GDStyle newStyle;
			newStyle.setStrokeWidth(newWidth);
			newStyle.valueMask = GDSTYLE_STROKEWIDTH_MASK;
			doc->selection->setStrokeStyle(&newStyle);
			doc->currentStrokeStyle |= newStyle;
			doc->redraw();
		}
	}
}

gint Properties::hideProperties(GdkEventAny *ev) {
	application->hideProperties();
	return 1;
}

void Properties::onFillButtonToggled() {
	//g_printerr("Properties::onFillButtonToggled\n");
	//g_printerr("active: %d\n", FillButton->get_active());
	setColorButtons();
}

void Properties::onStrokeButtonToggled() {
	//g_printerr("Properties::onStrokeButtonToggled\n");
	//g_printerr("active: %d\n", StrokeButton->get_active());
	setColorButtons();
}

void Properties::onTransparentButtonClicked() {
	//g_printerr("Properties::onTransparentButtonClicked\n");
	//g_printerr("active: %d\n", TransparentButton->get_active());
	if (TransparentButton->get_active()) {
		gint which;
		if (FillButton->get_active()) {
			which = GDPROPERTIES_FILL;
		} else {
			which = GDPROPERTIES_STROKE;
		}
		style[which].drawType = GDSTYLE_DRAWTYPE_TRANSPARENT;
		style[which].color.colorType = GDCOLOR_TYPE_RGB;
		style[which].valueMask = GDSTYLE_DRAWTYPE_MASK;
		guint r,g,b;
		style[which].getRGBColor(r,g,b);
		if (FillButton->get_active()) {
			fcd->setColor(r,g,b);
			if (doc) {
				AttributeAction *newAction = new AttributeAction(ATTRIBUTEACTION_TYPE_FILLCOLOR, style[which]);
				doc->setAction(newAction);
				doc->selection->setFillStyle(&style[which]);
				doc->currentFillStyle |= style[which];
			}
		} else {
			scd->setColor(r,g,b);
			if (doc) {
				AttributeAction *newAction = new AttributeAction(ATTRIBUTEACTION_TYPE_STROKECOLOR, style[which]);
				doc->setAction(newAction);
				doc->selection->setStrokeStyle(&style[which]);
				doc->currentStrokeStyle |= style[which];
			}
		}
		if (doc) {
			doc->redraw();
		}
	}
}

void Properties::onWhiteButtonClicked() {
	//g_printerr("Properties::onWhiteButtonClicked\n");
	//g_printerr("active: %d\n", WhiteButton->get_active());
	if (WhiteButton->get_active()) {
		gint which;
		if (FillButton->get_active()) {
			which = GDPROPERTIES_FILL;
		} else {
			which = GDPROPERTIES_STROKE;
		}
		style[which].drawType = GDSTYLE_DRAWTYPE_OPAQUE;
		style[which].color.colorType = GDCOLOR_TYPE_WHITE;
		style[which].valueMask = GDSTYLE_DRAWTYPE_MASK | GDSTYLE_COLORTYPE_MASK;
		guint r,g,b;
		style[which].getRGBColor(r,g,b);
		if (FillButton->get_active()) {
			fcd->setColor(r,g,b);
			if (doc) {
				AttributeAction *newAction = new AttributeAction(ATTRIBUTEACTION_TYPE_FILLCOLOR, style[which]);
				doc->setAction(newAction);
				doc->selection->setFillStyle(&style[which]);
				doc->currentFillStyle |= style[which];
			}
		} else {
			scd->setColor(r,g,b);
			if (doc) {
				AttributeAction *newAction = new AttributeAction(ATTRIBUTEACTION_TYPE_STROKECOLOR, style[which]);
				doc->setAction(newAction);
				doc->selection->setStrokeStyle(&style[which]);
				doc->currentStrokeStyle |= style[which];
			}
		}
		if (doc) {
			doc->redraw();
		}
	}
}

void Properties::onBlackButtonClicked() {
	//g_printerr("Properties::onBlackButtonClicked\n");
	//g_printerr("active: %d\n", BlackButton->get_active());
	if (BlackButton->get_active()) {
		gint which;
		if (FillButton->get_active()) {
			which = GDPROPERTIES_FILL;
		} else {
			which = GDPROPERTIES_STROKE;
		}
		style[which].drawType = GDSTYLE_DRAWTYPE_OPAQUE;
		style[which].color.colorType = GDCOLOR_TYPE_BLACK;
		style[which].valueMask = GDSTYLE_DRAWTYPE_MASK | GDSTYLE_COLORTYPE_MASK;
		guint r,g,b;
		style[which].getRGBColor(r,g,b);
		if (FillButton->get_active()) {
			fcd->setColor(r,g,b);
			if (doc) {
				AttributeAction *newAction = new AttributeAction(ATTRIBUTEACTION_TYPE_FILLCOLOR, style[which]);
				doc->setAction(newAction);
				doc->selection->setFillStyle(&style[which]);
				doc->currentFillStyle |= style[which];
			}
		} else {
			scd->setColor(r,g,b);
			if (doc) {
				AttributeAction *newAction = new AttributeAction(ATTRIBUTEACTION_TYPE_STROKECOLOR, style[which]);
				doc->setAction(newAction);
				doc->selection->setStrokeStyle(&style[which]);
				doc->currentStrokeStyle |= style[which];
			}
		}
		if (doc) {
			doc->redraw();
		}
	}
}

void Properties::onRGBButtonClicked() {
	//g_printerr("Properties::onRGBButtonClicked\n");
	//g_printerr("active: %d\n", RGBButton->get_active());
	if (RGBButton->get_active()) {
		csd = new class Gtk::ColorSelectionDialog(_("Select Color"));
		guint r,g,b;
		gdouble opacity;
		if (FillButton->get_active()) {
			style[GDPROPERTIES_FILL].getRGBColor(r,g,b);
			opacity = style[GDPROPERTIES_FILL].opacity;
		} else {
			style[GDPROPERTIES_STROKE].getRGBColor(r,g,b);
			opacity = style[GDPROPERTIES_STROKE].opacity;
		}
		gdouble col[4] = { r / 255.0, g / 255.0, b / 255.0, opacity };
		csd->get_colorsel()->set_color(col);
		csd->set_modal(1);
		csd->get_ok_button()->clicked.connect(SigC::slot(this, &Properties::RGBSelectionOK));
		csd->get_cancel_button()->clicked.connect(csd->destroy.slot());
		csd->show();
	}
}

void Properties::RGBSelectionOK() {
	if (csd) {
		gdouble col[4];
		csd->get_colorsel()->get_color(col);
		gint which;
		if (FillButton->get_active()) {
			which = GDPROPERTIES_FILL;
		} else {
			which = GDPROPERTIES_STROKE;
		}
		style[which].color.setRGB((guint) (col[0] * 255), (guint) (col[1] * 255), (guint) (col[2] * 255));
		guint r,g,b;
		style[which].color.getRGBColor(r, g, b);
		style[which].drawType = GDSTYLE_DRAWTYPE_OPAQUE;
		style[which].valueMask = GDSTYLE_DRAWTYPE_MASK | GDSTYLE_COLORTYPE_MASK | GDSTYLE_COLOR_MASK;
		if (FillButton->get_active()) {
			fcd->setColor(r,g,b);
			if (doc) {
				AttributeAction *newAction = new AttributeAction(ATTRIBUTEACTION_TYPE_FILLCOLOR, style[which]);
				doc->setAction(newAction);
				doc->selection->setFillStyle(&style[which]);
				doc->currentFillStyle |= style[which];
			}
		} else {
			scd->setColor(r,g,b);
			if (doc) {
				AttributeAction *newAction = new AttributeAction(ATTRIBUTEACTION_TYPE_STROKECOLOR, style[which]);
				doc->setAction(newAction);
				doc->selection->setStrokeStyle(&style[which]);
				doc->currentStrokeStyle |= style[which];
			}
		}
		csd->destroy();
		if (doc) {
			doc->redraw();
		}
	}
}

void Properties::onCMYKButtonClicked() {
	g_printerr("Properties::onCMYKButtonClicked\n");
	g_printerr("active: %d\n", CMYKButton->get_active());
}

void Properties::onCustomColorButtonClicked() {
	g_printerr("Properties::onCustomColorButtonClicked\n");
	g_printerr("active: %d\n", CustomColorButton->get_active());
}

void Properties::onGradientButtonClicked() {
	//g_printerr("Properties::onGradientButtonClicked\n");
	//g_printerr("active: %d\n", GradientButton->get_active());
	if (GradientButton->get_active()) {
		glsd = new class GradientListSelectionDialog(doc, style[GDPROPERTIES_FILL].gradient);
		glsd->set_modal(1);
		glsd->getOKButton()->clicked.connect(SigC::slot(this, &Properties::gradientSelectOK));
		glsd->getCancelButton()->clicked.connect(glsd->destroy.slot());
		glsd->show();
	}
}

void Properties::gradientSelectOK() {
	if (doc) {
		style[GDPROPERTIES_FILL].gradient = glsd->getSelectedGradient();
		if (style[GDPROPERTIES_FILL].gradient) {
			style[GDPROPERTIES_FILL].drawType = GDSTYLE_DRAWTYPE_GRADIENT;
			style[GDPROPERTIES_FILL].valueMask = GDSTYLE_GRADIENT_MASK | GDSTYLE_MATRIX_MASK | GDSTYLE_DRAWTYPE_MASK;
			SelectionItemList sel;
			doc->getSelection(sel, SELECTIONITEM_MODE_ORIG_ONLY);
			BoundingBox bbox = sel.getBoundingBox();
			// set the gradient transformation using the selection bounding box
			gdouble w = bbox.right - bbox.left;
			gdouble h = bbox.bottom - bbox.top;
			gdouble gamma = 0;
			if (h > w) {
				gamma = M_PI_2 - atan(w/h);
			} else {
				gamma = atan(h/w);
			}
			TransformationMatrix tmp, fac;
			gdouble scale = sqrt(w * w + h * h);
			tmp.setScale(scale, scale);
			fac.setRotation(gamma);
			tmp *= fac;
			fac.reset();
			fac.setTranslation(bbox.left, bbox.top);
			tmp *= fac;
			style[GDPROPERTIES_FILL].tMatrix = tmp;
			AttributeAction *newAction = new AttributeAction(ATTRIBUTEACTION_TYPE_FILLCOLOR, style[GDPROPERTIES_FILL]);
			doc->setAction(newAction);
			doc->selection->setFillStyle(&style[GDPROPERTIES_FILL]);
			doc->currentFillStyle |= style[GDPROPERTIES_FILL];
			doc->redraw();
			//style[GDPROPERTIES_FILL].tMatrix.printMatrix();
		}
		//g_printerr("Properties::GradientSelectOK fill_gradient: %lX drawType: %d\n", (long) style[GDPROPERTIES_FILL].gradient, style[GDPROPERTIES_FILL].drawType);
	}
	glsd->destroy();
}

void Properties::onPatternButtonClicked() {
	/*g_printerr("Properties::onPatternButtonClicked\n");
	g_printerr("active: %d\n", PatternButton->get_active());*/
	if (PatternButton->get_active()) {
		plsd = new class PatternListSelectionDialog(doc, style[GDPROPERTIES_FILL].pattern);
		plsd->set_modal(1);
		plsd->getOKButton()->clicked.connect(SigC::slot(this, &Properties::patternSelectOK));
		plsd->getCancelButton()->clicked.connect(plsd->destroy.slot());
		plsd->show();
	}

}

void Properties::patternSelectOK() {
	if (doc) {
		style[GDPROPERTIES_FILL].pattern = plsd->getSelectedPattern();
		if (style[GDPROPERTIES_FILL].pattern) {
			style[GDPROPERTIES_FILL].drawType = GDSTYLE_DRAWTYPE_PATTERN;
			style[GDPROPERTIES_FILL].valueMask = GDSTYLE_PATTERN_MASK | GDSTYLE_MATRIX_MASK | GDSTYLE_DRAWTYPE_MASK;
			SelectionItemList sel;
			doc->getSelection(sel, SELECTIONITEM_MODE_ORIG_ONLY);
			BoundingBox bbox = sel.getBoundingBox();
			// set the gradient transformation using the selection bounding box
			/*gdouble w = bbox->right - bbox->left;
			gdouble h = bbox->bottom - bbox->top;
			gdouble gamma = 0;
			if (h > w) {
				gamma = M_PI_2 - atan(w/h);
			} else {
				gamma = atan(h/w);
			}*/
			TransformationMatrix tmp, fac;
			//gdouble scale = sqrt(w * w + h * h);
			gdouble scale = style[GDPROPERTIES_FILL].pattern->origFactor;
			tmp.setScale(scale, scale);
			//fac.setRotation(gamma);
			//tmp *= fac;
			fac.reset();
			fac.setTranslation(bbox.left, bbox.top);
			tmp *= fac;
			style[GDPROPERTIES_FILL].tMatrix = tmp;
			AttributeAction *newAction = new AttributeAction(ATTRIBUTEACTION_TYPE_FILLCOLOR, style[GDPROPERTIES_FILL]);
			doc->setAction(newAction);
			doc->selection->setFillStyle(&style[GDPROPERTIES_FILL]);
			doc->currentFillStyle |= style[GDPROPERTIES_FILL];
			doc->redraw();
			//style[GDPROPERTIES_FILL].tMatrix.printMatrix();
		}
		//g_printerr("Properties::GradientSelectOK fill_gradient: %lX drawType: %d\n", (long) style[GDPROPERTIES_FILL].gradient, style[GDPROPERTIES_FILL].drawType);
	}
	plsd->destroy();
}

void Properties::onStrokeSolidToggled() {
	if (update_active && strokeSolidRadio->get_active()) {
		for (int i = 0; i < 6; i++) {
			dashEntries[i]->set_sensitive(false);
		}
		style[GDPROPERTIES_STROKE].lineType = GDSTYLE_LINETYPE_PASS;
		style[GDPROPERTIES_STROKE].valueMask = GDSTYLE_LINETYPE_MASK;
		doc->selection->setStrokeStyle(&style[GDPROPERTIES_STROKE]);
		doc->currentStrokeStyle |= style[GDPROPERTIES_STROKE];
		doc->redraw();
	}
}

void Properties::onStrokeDashedToggled() {
	if (update_active && strokeDashedRadio->get_active()) {
		for (int i = 0; i < 6; i++) {
			dashEntries[i]->set_sensitive(true);
		}
		style[GDPROPERTIES_STROKE].lineType = GDSTYLE_LINETYPE_DASHED;
		gboolean valid = true;
		for (int i = 0; i < 6; i++) {
			if (valid) {
				string dashText = dashEntries[i]->get_text();
				if (dashText.size()) {
					style[GDPROPERTIES_STROKE].dashes[i] = application->preferences.unitStrToDouble(dashText, false);
				} else {
					valid = false;
					style[GDPROPERTIES_STROKE].dashes[i] = -1;
				}
			} else {
				style[GDPROPERTIES_STROKE].dashes[i] = -1;
			}
		}	
		style[GDPROPERTIES_STROKE].valueMask = GDSTYLE_LINETYPE_MASK | GDSTYLE_DASHES_MASK;
		doc->selection->setStrokeStyle(&style[GDPROPERTIES_STROKE]);
		doc->currentStrokeStyle |= style[GDPROPERTIES_STROKE];
		doc->redraw();
	}
}

void Properties::onDashEntryChanged() {
	if (update_active) {
		gboolean valid = true;
		for (int i = 0; i < 6; i++) {
			if (valid) {
				string dashText = dashEntries[i]->get_text();
				if (dashText.size()) {
					style[GDPROPERTIES_STROKE].dashes[i] = application->preferences.unitStrToDouble(dashText, false);
				} else {
					valid = false;
					style[GDPROPERTIES_STROKE].dashes[i] = -1;
				}
			} else {
				style[GDPROPERTIES_STROKE].dashes[i] = -1;
			}
		}	
		style[GDPROPERTIES_STROKE].valueMask = GDSTYLE_DASHES_MASK;
		doc->selection->setStrokeStyle(&style[GDPROPERTIES_STROKE]);
		doc->currentStrokeStyle |= style[GDPROPERTIES_STROKE];
		doc->redraw();
	}
}

void Properties::onCapButtClicked() {
	if (update_active) {
		update_active = false;
		capButtToggle->set_active(true);
		capSquareToggle->set_active(false);
		capRoundToggle->set_active(false);
		style[GDPROPERTIES_STROKE].lineCaps = GDSTYLE_CAPS_BUTT;
		style[GDPROPERTIES_STROKE].valueMask = GDSTYLE_CAPS_MASK;
		doc->selection->setStrokeStyle(&style[GDPROPERTIES_STROKE]);
		doc->currentStrokeStyle |= style[GDPROPERTIES_STROKE];
		update_active = true;
		doc->redraw();
	}
}

void Properties::onCapSquareClicked() {
	if (update_active) {
		update_active = false;
		capButtToggle->set_active(false);
		capSquareToggle->set_active(true);
		capRoundToggle->set_active(false);
		style[GDPROPERTIES_STROKE].lineCaps = GDSTYLE_CAPS_SQUARE;
		style[GDPROPERTIES_STROKE].valueMask = GDSTYLE_CAPS_MASK;
		doc->selection->setStrokeStyle(&style[GDPROPERTIES_STROKE]);
		doc->currentStrokeStyle |= style[GDPROPERTIES_STROKE];
		update_active = true;
		doc->redraw();
	}
}

void Properties::onCapRoundClicked() {
	if (update_active) {
		update_active = false;
		capButtToggle->set_active(false);
		capSquareToggle->set_active(false);
		capRoundToggle->set_active(false);
		style[GDPROPERTIES_STROKE].lineCaps = GDSTYLE_CAPS_ROUND;
		style[GDPROPERTIES_STROKE].valueMask = GDSTYLE_CAPS_MASK;
		doc->selection->setStrokeStyle(&style[GDPROPERTIES_STROKE]);
		doc->currentStrokeStyle |= style[GDPROPERTIES_STROKE];
		update_active = true;
		doc->redraw();
	}
}

void Properties::onJoinBevelClicked() {
	if (update_active) {
		update_active = false;
		joinBevelToggle->set_active(true);
		joinMiterToggle->set_active(false);
		joinRoundToggle->set_active(false);
		miterLimitEntry->set_sensitive(false);
		style[GDPROPERTIES_STROKE].lineJoins = GDSTYLE_JOINS_BEVEL;
		style[GDPROPERTIES_STROKE].valueMask = GDSTYLE_JOINS_MASK;
		doc->selection->setStrokeStyle(&style[GDPROPERTIES_STROKE]);
		doc->currentStrokeStyle |= style[GDPROPERTIES_STROKE];
		update_active = true;
		doc->redraw();
	}
}

void Properties::onJoinMiterClicked() {
	if (update_active) {
		update_active = false;
		joinBevelToggle->set_active(false);
		joinMiterToggle->set_active(true);
		joinRoundToggle->set_active(false);
		miterLimitEntry->set_sensitive(true);
		style[GDPROPERTIES_STROKE].lineJoins = GDSTYLE_JOINS_MITER;
		style[GDPROPERTIES_STROKE].valueMask = GDSTYLE_JOINS_MASK;
		doc->selection->setStrokeStyle(&style[GDPROPERTIES_STROKE]);
		doc->currentStrokeStyle |= style[GDPROPERTIES_STROKE];
		update_active = true;
		doc->redraw();
	}
}

void Properties::onJoinRoundClicked() {
	if (update_active) {
		update_active = false;
		joinBevelToggle->set_active(false);
		joinMiterToggle->set_active(false);
		joinRoundToggle->set_active(true);
		miterLimitEntry->set_sensitive(false);
		style[GDPROPERTIES_STROKE].lineJoins = GDSTYLE_JOINS_ROUND;
		style[GDPROPERTIES_STROKE].valueMask = GDSTYLE_JOINS_MASK;
		doc->selection->setStrokeStyle(&style[GDPROPERTIES_STROKE]);
		doc->currentStrokeStyle |= style[GDPROPERTIES_STROKE];
		update_active = true;
		doc->redraw();
	}
}

void Properties::onMiterLimitChanged() {
	if (update_active) {
		style[GDPROPERTIES_STROKE].lineMiterLimit = strtod(miterLimitEntry->get_text().c_str(), NULL);
		style[GDPROPERTIES_STROKE].valueMask = GDSTYLE_JOINS_MASK;
		doc->selection->setStrokeStyle(&style[GDPROPERTIES_STROKE]);
		doc->currentStrokeStyle |= style[GDPROPERTIES_STROKE];
		doc->redraw();
	}
}


void Properties::setFillStyle(GDStyle *style) {
	if (style) {
		this->style[GDPROPERTIES_FILL] = *style;
		//g_printerr("Properties::setFillStyle: valueMask=%lX\n", style->valueMask);
		if (style->valueMask & GDSTYLE_COLOR_MASK && style->valueMask & GDSTYLE_COLORTYPE_MASK &&
					style->valueMask & GDSTYLE_DRAWTYPE_MASK) {
			//g_printerr("Properties::setFillStyle all identical\n");
			guint r,g,b;
			style->getRGBColor(r,g,b);
			if (style->drawType == GDSTYLE_DRAWTYPE_TRANSPARENT) {
				r = 65535; g = 0; b = 0;
			}
			fcd->setColor(r,g,b);
		} else {
			//g_printerr("Properties::setFillStyle some are different\n");
			fcd->setColor(65534, 0, 0);
		}
		
		if (FillButton->get_active()) {
			setColorButtons();
		}
	}
}

void Properties::setStrokeStyle(GDStyle *style) {
	if (style) {
		this->style[GDPROPERTIES_STROKE] = *style;
		if (style->valueMask & GDSTYLE_COLOR_MASK && style->valueMask & GDSTYLE_COLORTYPE_MASK &&
					style->valueMask & GDSTYLE_DRAWTYPE_MASK) {
			guint r,g,b;
			style->getRGBColor(r,g,b);
			if (style->drawType == GDSTYLE_DRAWTYPE_TRANSPARENT) {
				r = 65535; g = 0; b = 0;
			}
			scd->setColor(r,g,b);
		} else {
			scd->setColor(65534, 0, 0);
		}
		
		if (StrokeButton->get_active()) {
			setColorButtons();
		}
		strokeWidthConnection.disconnect();
		if (style->valueMask & GDSTYLE_STROKEWIDTH_MASK) {
			StrokeWeightSpinbutton->set_value(style->strokeWidth);
		} else {
			StrokeWeightSpinbutton->set_value(-1);
		}
		strokeWidthConnection = StrokeWeightSpinbutton->get_adjustment()->value_changed.connect(SigC::slot(this, &Properties::strokeWidthChanged));
		update_active = false;
		if (style->valueMask & GDSTYLE_DRAWTYPE_MASK &&
				style->drawType != GDSTYLE_DRAWTYPE_TRANSPARENT) {
			// object has stroke set
			if (style->lineType == GDSTYLE_LINETYPE_PASS) {
				// no dashes
				strokeSolidRadio->set_active(true);
				strokeDashedRadio->set_active(false);
				for (int i = 0; i < 6; i++) {
					dashEntries[i]->set_sensitive(false);
				}
			} else {
				gchar floatvalue[21];
				strokeSolidRadio->set_active(false);
				strokeDashedRadio->set_active(true);
				for (int i = 0; i < 6; i++) {
					dashEntries[i]->set_sensitive(true);
					if (style->dashes[i] < 0) {
						dashEntries[i]->set_text("");
					} else {
						snprintf(floatvalue, 20, "%f", style->dashes[i]);
						dashEntries[i]->set_text(floatvalue);
					}
				}
			}
			capButtToggle->set_active(false);
			capSquareToggle->set_active(false);
			capRoundToggle->set_active(false);
			if (style->valueMask & GDSTYLE_CAPS_MASK) {
				if (style->lineCaps == GDSTYLE_CAPS_BUTT) {
					capButtToggle->set_active(true);
				} else if (style->lineCaps == GDSTYLE_CAPS_SQUARE) {
					capSquareToggle->set_active(true);
				} else if (style->lineCaps == GDSTYLE_CAPS_ROUND) {
					capRoundToggle->set_active(true);
				}
			}
			joinBevelToggle->set_active(false);
			joinMiterToggle->set_active(false);
			joinRoundToggle->set_active(false);
			miterLimitEntry->set_sensitive(false);
			if (style->valueMask & GDSTYLE_JOINS_MASK) {
				if (style->lineJoins == GDSTYLE_JOINS_BEVEL) {
					joinBevelToggle->set_active(true);
				} else if (style->lineJoins == GDSTYLE_JOINS_MITER) {
					joinMiterToggle->set_active(true);
					miterLimitEntry->set_sensitive(true);
					if (style->lineMiterLimit < 0) {
						miterLimitEntry->set_text("");
					} else {
						gchar floatvalue[21];
						snprintf(floatvalue, 20, "%f", style->lineMiterLimit);
						miterLimitEntry->set_text(floatvalue);
					}
				} else if (style->lineJoins == GDSTYLE_JOINS_ROUND) {
					joinRoundToggle->set_active(true);
				}
			}
		}
		update_active = true;
	}
}


void Properties::setColorButtons() {
	gint which = GDPROPERTIES_FILL;
	if (StrokeButton->get_active()) {
		which = GDPROPERTIES_STROKE;
	}
	if (style[which].drawType == GDSTYLE_DRAWTYPE_TRANSPARENT) {
		TransparentButton->set_active(1);
	} else if (style[which].drawType == GDSTYLE_DRAWTYPE_GRADIENT) {
		GradientButton->set_active(1);
	} else if (style[which].drawType == GDSTYLE_DRAWTYPE_PATTERN) {
		PatternButton->set_active(1);
	} else if (style[which].drawType == GDSTYLE_DRAWTYPE_OPAQUE) {
		if (style[which].color.colorType == GDCOLOR_TYPE_RGB) {
			RGBButton->set_active(1);
		} else if (style[which].color.colorType == GDCOLOR_TYPE_CMYK) {
			CMYKButton->set_active(1);
		} else if (style[which].color.colorType == GDCOLOR_TYPE_BLACK) {
			BlackButton->set_active(1);
		} else if (style[which].color.colorType == GDCOLOR_TYPE_WHITE) {
			WhiteButton->set_active(1);
		} else if (style[which].color.colorType == GDCOLOR_TYPE_CUSTOM) {
			CustomColorButton->set_active(1);
		}
	}
}

Properties::~Properties() {
}
