/* $Id: Properties.hh,v 1.10 2004/03/26 13:12:01 pklaus Exp $
 *
 * Properties.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef _PROPERTIES_HH
#  include "Properties_glade.hh"
#  define _PROPERTIES_HH
#include "config.h"
#include "glademm_support.hh"
#include "GDStyle.hh"

class GDDocument;
class GDApplication;
class FillColorDisplay;
class StrokeColorDisplay;
class GradientListSelectionDialog;
class PatternListSelectionDialog;

#define GDPROPERTIES_FILL	0
#define GDPROPERTIES_STROKE	1

class Properties : public Properties_glade {   
        
        friend class Properties_glade;
	
	GDApplication *application;
	GDDocument *doc;
	SigC::Connection selConnection;
	SigC::Connection strokeWidthConnection;
		
	gboolean update_active;
	
	Gtk::Entry *dashEntries[6];
	
	FillColorDisplay *fcd;
	StrokeColorDisplay *scd;
	
	GDStyle style[2];
	
	Gtk::ColorSelectionDialog *csd;
	GradientListSelectionDialog *glsd;
	PatternListSelectionDialog *plsd;

public:
	Properties(GDApplication *gda);
	
	void setDocument(GDDocument *document);
	void updateValues(GDDocument *document);
	
	void setFillStyle(GDStyle *style);
	void setStrokeStyle(GDStyle *style);
	
	void setColorButtons();
	
	void RGBSelectionOK();
	void gradientSelectOK();
	void patternSelectOK();
	void strokeWidthChanged();
	
	void onFillButtonToggled();
	void onStrokeButtonToggled();
	void onTransparentButtonClicked();
	void onWhiteButtonClicked();
	void onBlackButtonClicked();
	void onRGBButtonClicked();
	void onCMYKButtonClicked();
	void onCustomColorButtonClicked();
	void onGradientButtonClicked();
	void onPatternButtonClicked();
	void onStrokeSolidToggled();
	void onStrokeDashedToggled();
	void onDashEntryChanged();
	void onCapButtClicked();
	void onCapSquareClicked();
	void onCapRoundClicked();
	void onJoinBevelClicked();
	void onJoinMiterClicked();
	void onJoinRoundClicked();
	void onMiterLimitChanged();


	gint hideProperties(GdkEventAny *ev);
	
	~Properties();
};

#endif
