/* $Id: RectangleArea.cc,v 1.1 2004/07/21 06:31:20 pklaus Exp $
 *
 * RectangleArea.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "RectangleArea.hh"

RectangleArea::RectangleArea() {
	left = 0;
	top = 0;
	right = 0;
	bottom = 0;
}

RectangleArea::RectangleArea(const RectangleArea& from) {
	left = from.left;
	top = from.top;
	right = from.right;
	bottom = from.bottom;
}

RectangleArea::RectangleArea(gdouble l, gdouble t, gdouble r, gdouble b) {
	left = l;
	top = t;
	right = r;
	bottom = b;
}

void RectangleArea::do_intersect(const RectangleArea& with) {
	if (contained(with)) {
		// completely within "with", do nothing
		return;
	}
	if (with.contained(*this)) {
		// "with" is part of this rectangle, intersection is "with"
		left = with.left;
		top = with.top;
		right = with.right;
		bottom= with.bottom;
		return;
	}
	if (intersect(with)) {
		if (left < with.left) {
			left = with.left;
		}
		if (right > with.right) {
			right = with.right;
		}
		if (top < with.top) {
			top = with.top;
		}
		if (bottom > with.bottom) {
			bottom = with.bottom;
		}
	}
	// no intersection at all, make rect non-existing
	left = G_MAXDOUBLE;
	right = G_MINDOUBLE;
	top = G_MAXDOUBLE;
	bottom = G_MINDOUBLE;
}

gboolean RectangleArea::intersect(const RectangleArea& with) const {
	if (with.contained(*this)) {
		return true;
	}
	if (left <= with.right && right >= with.left && 
			(top >= with.top && top <= with.bottom || bottom >= with.top && bottom <= with.bottom)) {
		return true;
	}
	if (top <= with.bottom && bottom >= with.top &&
			(left >= with.left && left <= with.right || right >= with.left && right <= with.right)) {
		return true;
	}
	return false;
}

gboolean RectangleArea::contained(const RectangleArea& in) const {
	if (left >= in.left && right <= in.right && top >= in.top && bottom <= in.bottom) {
		return true;
	}
	return false;
}

gboolean RectangleArea::contains(gdouble x, gdouble y) const {
	if (x >= left && x <= right && y >= top && y <= bottom) {
		return true;
	}
	return false;
}
