/* $Id: RectangleTool.cc,v 1.6 2004/03/24 19:31:43 pklaus Exp $
 *
 * RectangleTool.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <math.h>
#include "../pixmaps/rectangle.xpm"
#include "../cursors/crosshair.xbm"
#include "../cursors/crosshair_mask.xbm"
#include "../cursors/move.xbm"
#include "../cursors/move_mask.xbm"
#include "../cursors/move_magnet.xbm"
#include "../cursors/move_magnet_mask.xbm"
#include "RectangleTool.hh"
#include "GDDocument.hh"
#include "BezierPath.hh"
#include "BezierNode.hh"
#include "DocWindow.hh"
#include "CursorFactory.hh"
#include "AddAction.hh"
#include "DimensionDialog.hh"
#include "SelectionItemList.hh"

#define doubleMAX(a,b)	(fabs(a) > fabs(b)) ? fabs(a) : fabs(b)

RectangleTool::RectangleTool() {
	pixmapData = rectangle_xpm;
	toolName = "RectangleTool";
	moving = false;
	nodeCaught = false;
	valueDialog = NULL;
	dialogDoc = NULL;
	CursorFactory::createCursorFromData(crosshairCursor,
					crosshair_bits, crosshair_mask_bits,
					crosshair_width, crosshair_height,
					crosshair_x_hot, crosshair_y_hot);
	CursorFactory::createCursorFromData(moveCursor,
					move_bits, move_mask_bits,
					move_width, move_height,
					move_x_hot, move_y_hot);
	CursorFactory::createCursorFromData(moveMagnetCursor,
					move_magnet_bits, move_magnet_mask_bits,
					move_magnet_width, move_magnet_height,
					move_magnet_x_hot, move_magnet_y_hot);
}

void RectangleTool::activate() {
	Tool::activate();
}

void RectangleTool::deactivate() {
	Tool::deactivate();
}

gint RectangleTool::buttonDown(GdkEventButton *ev, DocWindow *docwin) {
	GDDocument	*doc;
	downButton = ev->button;

	if (downButton == 1) {
		doc = docwin->doc;
		doc->deSelectAll();
		if (ev->state & GDK_MOD1_MASK) {
			sx = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
			sy = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
			magnet(sx, sy, docwin);
			dialogDoc = doc;
			showValueDialog();
		} else {			
			sx = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
			sy = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
			gboolean caught = magnet(sx, sy, docwin);
			cx = sx;
			cy = sy;
			startx = (gint) ev->x;
			starty = (gint) ev->y;
			oldx = startx;
			oldy = starty;
			moving = true;
		}
	} else {
		Tool::buttonDown(ev, docwin);
	}
	return 0;
}

gint RectangleTool::motion(GdkEventMotion *ev, DocWindow *docwin) {
	if (ev->state & GDK_BUTTON1_MASK) {
		GDDocument *doc = docwin->doc;
		Gdk_Window drawable = docwin->DocWinDrawingArea->get_window();
		doc->context.set_function(GDK_INVERT);
		gdouble dx = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
		gdouble dy = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
		nodeCaught = magnet(dx, dy, docwin);
		setCursor(ev->state, docwin);
		if (ev->state & GDK_SHIFT_MASK) {
			// square
			gdouble delta = doubleMAX(dx-cx, dy-cy);
			dx = cx + copysign(delta, dx-cx);
			dy = cy + copysign(delta, dy-cy);
		}
		if (ev->state & GDK_MOD1_MASK) {
			// grow from center
			sx = cx - (dx - cx);
			sy = cy - (dy - cy);
		} else {
			sx = cx;
			sy = cy;
		}
		gint x = (gint) ((dx - docwin->osBuffer.origin.x) * docwin->osBuffer.scaleFactor);
		gint y = (gint) ((dy - docwin->osBuffer.origin.y) * docwin->osBuffer.scaleFactor);
		drawRectangle(drawable, doc->context, startx, starty, oldx, oldy);
		startx = (gint) ((sx - docwin->osBuffer.origin.x) * docwin->osBuffer.scaleFactor);
		starty = (gint) ((sy - docwin->osBuffer.origin.y) * docwin->osBuffer.scaleFactor);
		drawRectangle(drawable, doc->context, startx, starty, x, y);
		oldx = x;
		oldy = y;
		
	} else if (ev->state & (GDK_BUTTON2_MASK | GDK_BUTTON3_MASK)) {
		Tool::motion(ev, docwin);
	} else {
		nodeCaught = false;
		setCursor(ev->state, docwin);
	}
	
  return 0;
}

gint RectangleTool::buttonUp(GdkEventButton *ev, DocWindow *docwin) {
	if (downButton == 1) {
		GDDocument *doc = docwin->doc;
		Gdk_Window drawable = docwin->DocWinDrawingArea->get_window();
		gdouble dx = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
		gdouble dy = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
		gboolean caught = magnet(dx, dy, docwin);
		if (ev->state & GDK_SHIFT_MASK) {
			// square
			gdouble delta = doubleMAX(dx-cx, dy-cy);
			dx = cx + copysign(delta, dx-cx);
			dy = cy + copysign(delta, dy-cy);
		}
		if (ev->state & GDK_MOD1_MASK) {
			// grow from center
			sx = cx - (dx - cx);
			sy = cy - (dy - cy);
		} else {
			sx = cx;
			sy = cy;
		}
		addRectangle(doc, sx, sy, dx, dy);
		// redraw the document
		doc->context.set_function(GDK_SET);
		doc->redraw();
		moving = false;
	} else {
		Tool::buttonUp(ev, docwin);
	}
	return 0;
}

void RectangleTool::setCursor(guint state, DocWindow *docwin) {
	if (moving) {
		if (nodeCaught) {
			// set the cursor
			docwin->DocWinDrawingArea->get_window().set_cursor(moveMagnetCursor);
		} else {
			// set the cursor
			docwin->DocWinDrawingArea->get_window().set_cursor(moveCursor);
		}
	} else {
		if (state & GDK_MOD1_MASK) {
			docwin->DocWinDrawingArea->get_window().set_cursor(crosshairCursor);
		} else {
			docwin->DocWinDrawingArea->get_window().set_cursor(crosshairCursor);
		}
	}
}

void RectangleTool::addRectangle(GDDocument *doc, gdouble tlx, gdouble tly, gdouble brx, gdouble bry) {
	BezierPath *path = new BezierPath(doc);
	BezierNode *n1 = new BezierNode(path);
	n1->setOnCurvePoint(tlx, tly);
	n1->setPreviousHandle(tlx, tly);
	n1->setNextHandle(tlx, tly);
	path->addNode(n1);
	BezierNode *n2 = new BezierNode(path);
	n2->setOnCurvePoint(brx, tly);
	n2->setPreviousHandle(brx, tly);
	n2->setNextHandle(brx, tly);
	path->addNode(n2);
	BezierNode *n3 = new BezierNode(path);
	n3->setOnCurvePoint(brx, bry);
	n3->setPreviousHandle(brx, bry);
	n3->setNextHandle(brx, bry);
	path->addNode(n3);
	BezierNode *n4 = new BezierNode(path);
	n4->setOnCurvePoint(tlx, bry);
	n4->setPreviousHandle(tlx, bry);
	n4->setNextHandle(tlx, bry);
	path->addNode(n4);
	n4->next = n1;
	n1->prev = n4; // close the path
	// generate an action event
	AddAction *newAction = new AddAction(path, doc->activeLayer);
	doc->setAction(newAction);
	path->selectAll();
	doc->addPath(path);
	doc->currPath = NULL;
}

gint RectangleTool::deleteValueDialog(GdkEventAny *ev) {
	cancelValueDialog();
	return 1;
}

void RectangleTool::cancelValueDialog() {
	valueDialog->hide();
}

void RectangleTool::createObject() {
	gdouble w = fabs(valueDialog->getWidth());
	gdouble h = fabs(valueDialog->getHeight());
	valueDialog->hide();
	addRectangle(dialogDoc, sx, sy, sx + w, sy + h);
	dialogDoc->redraw();
}

void RectangleTool::showValueDialog() {
	if (! valueDialog) {
		valueDialog = new DimensionDialog(dialogDoc->application);
		valueDialog->getOKButton()->clicked.connect(SigC::slot(this, &RectangleTool::createObject));
		valueDialog->getCancelButton()->clicked.connect(SigC::slot(this, &RectangleTool::cancelValueDialog));
		valueDialog->delete_event.connect(SigC::slot(this, &RectangleTool::deleteValueDialog));
		valueDialog->set_modal(true);
	}
	valueDialog->show();
}
