/* $Id: RotateTool.cc,v 1.4 2003/12/06 21:42:11 pklaus Exp $
 *
 * RotateTool.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "RotateTool.hh"
#include <math.h>


RotateTool::RotateTool() {
	pixmapData = rotate_xpm;
	toolName = "RotateTool";
	valueDialog = NULL;
}

TransformationMatrix RotateTool::calcMatrix(gdouble sx, gdouble sy, 
						gdouble cx, gdouble cy,
						gdouble x, gdouble y, gint state) {

	TransformationMatrix tm;
	gdouble angle = tm.getAngle(x - cx, y - cy) - tm.getAngle(sx - cx, sy - cy);
	if (state & GDK_SHIFT_MASK) {
		angle = floor((angle + M_PI / 8) * 4 / M_PI) * M_PI / 4;
	}
	tm.setRotation(angle);
	tm.setCenter(cx, cy);
	return tm;
}

TransformationMatrix RotateTool::setMatrix(gdouble angle) {
	TransformationMatrix tm;
	tm.setRotation(-angle);
	tm.setCenter(cx, cy);
	return tm;
}

gint RotateTool::deleteValueDialog(GdkEventAny *ev) {
	cancelValueDialog();
	return 1;
}

void RotateTool::cancelValueDialog() {
	valueDialog->hide();
	mode = 0;
}

void RotateTool::transformObjects() {
	string angleString = valueDialog->getDegreeString();
	gdouble angle = strtod(angleString.c_str(), NULL);
	angle = angle / 180 * M_PI;
	TransformationMatrix tm = setMatrix(angle);
	doTransform(tm, false);
	valueDialog->hide();
}

void RotateTool::copyObjects() {
	string angleString = valueDialog->getDegreeString();
	gdouble angle = strtod(angleString.c_str(), NULL);
	angle = angle / 180 * M_PI;
	TransformationMatrix tm = setMatrix(angle);
	doTransform(tm, true);
	valueDialog->hide();
}

void RotateTool::showValueDialog() {
	if (! valueDialog) {
		valueDialog = new RotationDialog();
		valueDialog->getOKButton()->clicked.connect(SigC::slot(this, &RotateTool::transformObjects));
		valueDialog->getCopyButton()->clicked.connect(SigC::slot(this, &RotateTool::copyObjects));
		valueDialog->getCancelButton()->clicked.connect(SigC::slot(this, &RotateTool::cancelValueDialog));
		valueDialog->delete_event.connect(SigC::slot(this, &RotateTool::deleteValueDialog));
		valueDialog->set_modal(true);
	}
	valueDialog->show();
}
