/* $Id: ScaleTool.cc,v 1.9 2004/03/23 11:42:41 pklaus Exp $
 *
 * ScaleTool.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "ScaleTool.hh"
#include <math.h>

ScaleTool::ScaleTool() {
	pixmapData = scale_xpm;
	toolName = "ScaleTool";
	valueDialog = NULL;
}


TransformationMatrix ScaleTool::calcMatrix(gdouble sx, gdouble sy, 
						gdouble cx, gdouble cy,
						gdouble x, gdouble y, gint state) {

	gdouble sdx = sx - cx;
	if (fabs(sdx) < 5.0) {
		sdx = copysign(5.0, sdx);
	}
	gdouble scaleX = (x - cx) / sdx;
	gdouble sdy = sy - cy;
	if (fabs(sdy) < 5.0) {
		sdy = copysign(5.0, sdy);
	}
	gdouble scaleY = (y - cy) / sdy;
	//g_printerr("sdx: %f, sdy: %f, Sx: %f, Sy: %f\n", sdx, sdy, scaleX, scaleY);
	if (state & GDK_SHIFT_MASK) {
		if (fabs(scaleX) > fabs(scaleY)) {
			scaleY = copysign(scaleX, scaleY);
		} else {
			scaleX = copysign(scaleY, scaleX);
		}
	}
	TransformationMatrix tm;
	tm.setScale(scaleX, scaleY);
	tm.setCenter(cx, cy);
	return tm;
}
TransformationMatrix ScaleTool::setMatrix(gdouble xfactor, gdouble yfactor) {
	TransformationMatrix tm;
	tm.setScale(xfactor, yfactor);
	tm.setCenter(cx, cy);
	return tm;
}

gint ScaleTool::deleteValueDialog(GdkEventAny *ev) {
	cancelValueDialog();
	return 1;
}

void ScaleTool::cancelValueDialog() {
	valueDialog->hide();
	mode = 0;
}

void ScaleTool::transformObjects() {
	gdouble xfactor = valueDialog->getHorizontalScale();
	gdouble yfactor = valueDialog->getVerticalScale();
	TransformationMatrix tm = setMatrix(xfactor, yfactor);
	gboolean lwScale = valueDialog->getScaleLineWeight();
	doTransform(tm, false, !lwScale);
	valueDialog->hide();
}

void ScaleTool::copyObjects() {
	gdouble xfactor = valueDialog->getHorizontalScale();
	gdouble yfactor = valueDialog->getVerticalScale();
	TransformationMatrix tm = setMatrix(xfactor, yfactor);
	gboolean lwScale = valueDialog->getScaleLineWeight();
	doTransform(tm, true, !lwScale);
	valueDialog->hide();
}

void ScaleTool::showValueDialog() {
	if (! valueDialog) {
		valueDialog = new ScaleDialog();
		valueDialog->getOKButton()->clicked.connect(SigC::slot(this, &ScaleTool::transformObjects));
		valueDialog->getCopyButton()->clicked.connect(SigC::slot(this, &ScaleTool::copyObjects));
		valueDialog->getCancelButton()->clicked.connect(SigC::slot(this, &ScaleTool::cancelValueDialog));
		valueDialog->delete_event.connect(SigC::slot(this, &ScaleTool::deleteValueDialog));
		valueDialog->set_modal(true);
	}
	valueDialog->show();
}
