/* $Id: SelectPointTool.cc,v 1.11 2004/03/29 16:06:50 pklaus Exp $
 *
 * SelectPointTool.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <math.h>
#include "../pixmaps/point_select.xpm"
#include "../cursors/select_point.xbm"
#include "../cursors/select_point_mask.xbm"
#include "../cursors/select_path.xbm"
#include "../cursors/select_path_mask.xbm"
#include "../cursors/change_handle.xbm"
#include "../cursors/change_handle_mask.xbm"
#include "../cursors/move.xbm"
#include "../cursors/move_mask.xbm"
#include "../cursors/move_magnet.xbm"
#include "../cursors/move_magnet_mask.xbm"
#include "../cursors/move_copy.xbm"
#include "../cursors/move_copy_mask.xbm"
#include "../cursors/move_copy_magnet.xbm"
#include "../cursors/move_copy_magnet_mask.xbm"
#include "config.h"
#include "SelectPointTool.hh"
#include "GDDocument.hh"
#include "CursorFactory.hh"
#include "DocWindow.hh"
#include "BezierNode.hh"
#include "SelectionItemList.hh"
#include "BezierPath.hh"
#include "SelectableItem.hh"
#include "GDApplication.hh"
#include "TransformAction.hh"
#include "MoveHandleAction.hh"

#define doubleMAX(a,b)	(fabs(a) > fabs(b)) ? fabs(a) : fabs(b)


TransformationMatrix m;

SelectPointTool::SelectPointTool() {
	downButton = -1;
	pixmapData = point_select_xpm;
	toolName = "SelectPointTool";
	moving = false;
	nodeCaught = false;
	
	//cursors
	CursorFactory::createCursorFromData(selectPointCursor,
					select_point_bits, select_point_mask_bits,
					select_point_width, select_point_height,
					select_point_x_hot, select_point_y_hot);
	CursorFactory::createCursorFromData(selectPathCursor,
					select_path_bits, select_path_mask_bits,
					select_path_width, select_path_height,
					select_path_x_hot, select_path_y_hot);
	CursorFactory::createCursorFromData(moveCursor,
					move_bits, move_mask_bits,
					move_width, move_height,
					move_x_hot, move_y_hot);
	CursorFactory::createCursorFromData(moveMagnetCursor,
					move_magnet_bits, move_magnet_mask_bits,
					move_magnet_width, move_magnet_height,
					move_magnet_x_hot, move_magnet_y_hot);
	CursorFactory::createCursorFromData(moveCopyCursor,
					move_copy_bits, move_copy_mask_bits,
					move_copy_width, move_copy_height,
					move_copy_x_hot, move_copy_y_hot);
	CursorFactory::createCursorFromData(moveCopyMagnetCursor,
					move_copy_magnet_bits, move_copy_magnet_mask_bits,
					move_copy_magnet_width, move_copy_magnet_height,
					move_copy_magnet_x_hot, move_copy_magnet_y_hot);
	CursorFactory::createCursorFromData(changeHandleCursor,
					change_handle_bits, change_handle_mask_bits,
					change_handle_width, change_handle_height,
					change_handle_x_hot, change_handle_y_hot);
}

void SelectPointTool::activate() {
	Tool::activate();
}

void SelectPointTool::deactivate() {
	Tool::deactivate();
}

gint SelectPointTool::buttonDown(GdkEventButton *ev, DocWindow *docwin) {

	GDDocument	*doc;
	
	downButton = ev->button;
	movingNode = NULL;
	selectNode = NULL;
	handleReflectMode = false;
	
	if (downButton == 1) {
		doc = docwin->doc;
		shiftSelect = false;
		sx = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
		sy = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
		gdouble tolerance = doc->application->preferences.catchingRange / docwin->osBuffer.scaleFactor;
		SelectableItem *sitem = doc->checkPoints(sx, sy, tolerance, false);
		if (ev->state & GDK_SHIFT_MASK) {
			shiftSelect = true;
		}
		if (sitem) {
			if (! (shiftSelect || sitem->selected())) {
				doc->deSelectAll();
			}
			// node directly hit
			sitem->getCoordinates(sx, sy);
			// select point
			if (ev->state & GDK_MOD1_MASK) {
				// select path
				sitem->selectPath(shiftSelect);
			} else {
				sitem->select(shiftSelect);
			}
		} else {
			// no point hit, check curves
			sitem = doc->checkCurves(
							sx, sy, 
							doc->application->preferences.catchingRange / docwin->osBuffer.scaleFactor);
			if (sitem) {
				if (! (shiftSelect || sitem->selected())) {
					doc->deSelectAll();
				}
				if (ev->state & GDK_MOD1_MASK) {
					//g_printerr("SelectPointTool::buttonUp curveSelect\n");
					sitem->selectPath(shiftSelect);
				} else {
					if (! strcmp(sitem->identify(), "BezierNode")) {
						((BezierNode *) sitem)->selectCurve();
					} else {
						sitem->select(shiftSelect);
					}
				}
			} else {
				// no point and no curve hit, check areas
				sitem = doc->checkArea(sx, sy);
				if (sitem) {
					if (! (shiftSelect || sitem->selected())) {
						doc->deSelectAll();
					}
					if (shiftSelect && sitem->selected()) {
						((GObject *) sitem)->deSelectAll();
					} else {
						((GObject *) sitem)->selectAll();
					}
				}
			}
		}
		if (sitem != NULL) {
			if (! strcmp(sitem->identify(), "BezierNode")) {
				BezierNode *bnode = (BezierNode *) sitem;
				if (ev->state & GDK_CONTROL_MASK) {
					// special handle behaviour
					/*g_printerr("SelectPointTool::buttonDown: Ctrl: oc: %f,%f  ph: %f,%f  nh: %f,%f  tol: %f\n",
								bnode->onCurve.x, bnode->onCurve.y,
								bnode->nextHandle.x, bnode->nextHandle.y,
								bnode->prevHandle.x, bnode->prevHandle.y,
								tolerance);*/
					if (bnode->selectedItem == 0) {
						// got the onCurve point
						if (fabs(bnode->nextHandle.x - bnode->onCurve.x) < tolerance &&
							fabs(bnode->nextHandle.y - bnode->onCurve.y) < tolerance) {
							// nextHandle is more or less equal to onCurve => new target
							//g_printerr("move next handle\n");
							bnode->setNextHandle(bnode->onCurve.x, bnode->onCurve.y);
							bnode->selectedItem = 1;
							setHandleTarget(bnode);
							if (fabs(bnode->prevHandle.x - bnode->onCurve.x) < tolerance &&
								fabs(bnode->prevHandle.y - bnode->onCurve.y) < tolerance) {
								// both handles are more or less equal to onCurve => set handleReflectMode
								handleReflectMode = true;
							}
						} else if (fabs(bnode->prevHandle.x - bnode->onCurve.x) < tolerance &&
								fabs(bnode->prevHandle.y - bnode->onCurve.y) < tolerance) {
							// prevHandle is more or less equal to onCurve => new target
							//g_printerr("move prev handle\n");
							bnode->setPreviousHandle(bnode->onCurve.x, bnode->onCurve.y);
							bnode->selectedItem = -1;
							setHandleTarget(bnode);
						} else {
							// click on the curve point => change to corner point
							//g_printerr("change corner\n");
							bnode->setNextHandle(bnode->onCurve.x, bnode->onCurve.y);
							bnode->setPreviousHandle(bnode->onCurve.x, bnode->onCurve.y);
							bnode->straightHandles = true;
						}
					} else {
						// handles should be idependent now
						bnode->straightHandles = false;
						setHandleTarget(bnode);
					}
				} else {
					// normal behaviour
					if (bnode->selected() && bnode->selectedItem == 0) {	// node is selected
						// a selected point should be moved
						movingNode = bnode;
						movingItem = 0;
						selection = new SelectionItemList();
						doc->getSelection(*selection);
						doc->context.set_function(GDK_INVERT);
						selection->startTransformation(docwin, doc->context);
						bnode->getCoordinates(sx, sy);
					} else if ((bnode->testPreviousHandleVisible() && bnode->selectedItem == -1) // prev handle is to be moved
								|| (bnode->testNextHandleVisible() && bnode->selectedItem == 1)) { // next handle is to be moved
						// move a handle
						setHandleTarget(bnode);
					} else {
						if (! (ev->state & GDK_SHIFT_MASK)) {
							// deselect everything
							doc->deSelectAll();
						}
					}
				}
			} else {
				// not a BezierNode
				movingNode = sitem;
				movingItem = 0;
				selection = new SelectionItemList();
				doc->getSelection(*selection);
				doc->context.set_function(GDK_INVERT);
				selection->startTransformation(docwin, doc->context);
			}
		} else {
			if (! (ev->state & GDK_SHIFT_MASK)) {
				// deselect everything
				doc->deSelectAll();
			}
		}
		startx = (gint) ev->x;
		starty = (gint) ev->y;
		oldx = startx;
		oldy = starty;
		moving = true;
	} else {
		Tool::buttonDown(ev, docwin);
	}
	return 0;
}

void SelectPointTool::setHandleTarget(BezierNode *bnode) {
	movingNode = bnode;
	movingItem = bnode->selectedItem;
	selection = new SelectionItemList();
	SelectionItem *mnode = new SelectionItem(movingNode, SELECTIONITEM_MODE_COPY_OBJECT);
	selection->insert(selection->end(), mnode);
	if (bnode->selectedItem == -1) {
		sx = bnode->prevHandle.x;
		sy = bnode->prevHandle.y;
	} else {
		sx = bnode->nextHandle.x;
		sy = bnode->nextHandle.y;
	}
}

gint SelectPointTool::motion(GdkEventMotion *ev, DocWindow *docwin) {
	GdkDrawable	*drawable;
		
	if (ev->state & GDK_BUTTON1_MASK) {
		GDDocument *doc = docwin->doc;
		doc->context.set_function(GDK_INVERT);
		if (movingNode) {
			gdouble dx = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
			gdouble dy = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
			nodeCaught = magnet(dx, dy, docwin);
			setCursor(ev->state, docwin);
			if (ev->state & GDK_SHIFT_MASK) {
				// single axis/ equal axis movement
				gdouble deltax = fabs(dx - sx);
				gdouble deltay = fabs(dy - sy);
				if (deltax > 0 && deltay > 0 && deltax / deltay > 0.5 && deltax / deltay < 2) {
					gdouble delta = deltax > deltay ? deltax : deltay;
					dx = sx + copysign(delta, dx - sx);
					dy = sy + copysign(delta, dy - sy);
				} else if (deltax > deltay) {
					dy = sy;
				} else {
					dx = sx;
				}
			}
			if (movingItem == 0) {
				// move the selected node
				dx -= sx;
				dy -= sy;
				m.reset();
				m.setTranslation(dx, dy);
				selection->transformAndDraw(docwin, doc->context, m);
			} else if (movingItem == -1) {
				// move prev handle
				BezierNode *mnode = (BezierNode *) ((SelectionItem *)*selection->begin())->moving;
				mnode->drawSegments(docwin, doc->context);
				dx -= mnode->prevHandle.x;
				dy -= mnode->prevHandle.y;
				if (mnode->straightHandles) {
					mnode->drawNextHandle(docwin, doc->context);
					if (handleReflectMode) {
						mnode->nextHandle.x += -dx;
						mnode->nextHandle.y += -dy;
					} else {
						TransformationMatrix tm;
						gdouble rotangle = tm.getAngle(
								mnode->prevHandle.x + dx - mnode->onCurve.x,
								mnode->prevHandle.y + dy - mnode->onCurve.y) -
											tm.getAngle(
								mnode->prevHandle.x - mnode->onCurve.x, 
								mnode->prevHandle.y - mnode->onCurve.y);
						tm.setRotation(rotangle);
						tm.setCenter(mnode->onCurve.x, mnode->onCurve.y);
						tm.transform(mnode->nextHandle);
					}
					mnode->drawNextHandle(docwin, doc->context);
				}
				mnode->drawPreviousHandle(docwin, doc->context);
				mnode->prevHandle.x += dx;
				mnode->prevHandle.y += dy;
				mnode->drawSegments(docwin, doc->context);
				mnode->drawPreviousHandle(docwin, doc->context);
			} else if (movingItem == 1) {
				// move next handle
				BezierNode *mnode = (BezierNode *) ((SelectionItem *)*selection->begin())->moving;
				mnode->drawSegments(docwin, doc->context);
				dx -= mnode->nextHandle.x;
				dy -= mnode->nextHandle.y;
				if (mnode->straightHandles) {
					mnode->drawPreviousHandle(docwin, doc->context);
					if (handleReflectMode) {
						mnode->prevHandle.x += -dx;
						mnode->prevHandle.y += -dy;
					} else {
						TransformationMatrix tm;
						gdouble rotangle = tm.getAngle(
								mnode->nextHandle.x + dx - mnode->onCurve.x,
								mnode->nextHandle.y + dy - mnode->onCurve.y) -
											tm.getAngle(
								mnode->nextHandle.x - mnode->onCurve.x, 
								mnode->nextHandle.y - mnode->onCurve.y);
						tm.setRotation(rotangle);
						tm.setCenter(mnode->onCurve.x, mnode->onCurve.y);
						tm.transform(mnode->prevHandle);
					}
					mnode->drawPreviousHandle(docwin, doc->context);
				}
				mnode->drawNextHandle(docwin, doc->context);
				mnode->nextHandle.x += dx;
				mnode->nextHandle.y += dy;
				mnode->drawSegments(docwin, doc->context);
				mnode->drawNextHandle(docwin, doc->context);
			}
		} else {
			nodeCaught = false;
			setCursor(ev->state, docwin);
			// draw the elastic rectangle
			Gdk_Window drawable = docwin->DocWinDrawingArea->get_window();
			drawRectangle(drawable, doc->context, startx, starty, oldx, oldy);
			drawRectangle(drawable, doc->context, startx, starty, (int) ev->x, (int) ev->y);
		}
		oldx = (gint) ev->x;
		oldy = (gint) ev->y;
	} else if (ev->state & (GDK_BUTTON2_MASK | GDK_BUTTON3_MASK)) {
		Tool::motion(ev, docwin);
	} else {
		setCursor(ev->state, docwin);
	}
	
  return 0;
}

gint SelectPointTool::buttonUp(GdkEventButton *ev, DocWindow *docwin) {
	if (downButton == 1) {
		GDDocument *doc = docwin->doc;
		if (movingNode && selection) {
			if (ev->state & GDK_MOD1_MASK) {
				TransformationMatrix lastTransform = selection->getLastTransformation();
				if (movingItem) {
					// one of the two handles was moved
					MoveHandleAction *newAction = new MoveHandleAction((BezierNode *) ((SelectionItem *)*selection->begin())->moving,
																	movingItem, true);
					doc->setAction(newAction);
				} else {
					// one or more nodes were moved
					if (! lastTransform.isIdentityMatrix()) {
						TransformAction *newAction = new TransformAction(lastTransform, true);
						doc->setAction(newAction);
						// insert a copy of the selection
						//g_printerr("SelectPointTool::buttonUp insert a copy\n");
					}
				}
				docwin->doc->deSelectAll();
				selection->insertMoving(docwin->doc);
			} else {
				TransformationMatrix lastTransform = selection->getLastTransformation();
				if (movingItem) {
					// one of the two handles was moved
					MoveHandleAction *newAction = new MoveHandleAction((BezierNode *) ((SelectionItem *)*selection->begin())->moving,
																	movingItem, false);
					doc->setAction(newAction);
				} else {				
					if (! lastTransform.isIdentityMatrix()) {
						TransformAction *newAction = new TransformAction(lastTransform);
						doc->setAction(newAction);
						// apply the movement to the original
						//g_printerr("SelectPointTool::buttonUp apply to original\n");
					}
				}
				selection->applyToOriginal();
			}
			selection->endTransformation(docwin, doc->context);
			delete selection;
			selection = NULL;
		}
		if (! movingNode) {
			//g_printerr("SelectPointTool::buttonUp selectRectangle\n");
			gdouble l,t,r,b;
			if (startx > oldx) {
				l = oldx / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
				r = startx / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
			} else {
				l = startx / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
				r = oldx / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
			}
			if (starty > oldy) {
				t = oldy / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
				b = starty / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
			} else {
				t = starty / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
				b = oldy / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
			}
			int sS = 0;
			if (shiftSelect) {
				sS = 128;
			}
			doc->selectRectangle(l, t, r, b, (ev->state & GDK_MOD1_MASK ? 1 : 0) | sS);
		}
		//doc->context.set_function(GDK_SET);
		doc->redraw();
	}
	moving = false;
	handleReflectMode = false;
	downButton = 0;
	return 0;
}

void SelectPointTool::setCursor(guint state, DocWindow *docwin) {
	if (moving) {
		if (state & GDK_MOD1_MASK) {
			if (nodeCaught) {
				docwin->DocWinDrawingArea->get_window().set_cursor(moveCopyMagnetCursor);
			} else {
				docwin->DocWinDrawingArea->get_window().set_cursor(moveCopyCursor);
			}
		} else {
			if (nodeCaught) {
				docwin->DocWinDrawingArea->get_window().set_cursor(moveMagnetCursor);
			} else {
				docwin->DocWinDrawingArea->get_window().set_cursor(moveCursor);
			}
		}
	} else {
		if (state & GDK_CONTROL_MASK) {
			docwin->DocWinDrawingArea->get_window().set_cursor(changeHandleCursor);
		} else if (state & GDK_MOD1_MASK) {
			docwin->DocWinDrawingArea->get_window().set_cursor(selectPathCursor);
		} else {
			docwin->DocWinDrawingArea->get_window().set_cursor(selectPointCursor);
		}
	}
}
