/* $Id: SelectableItem.hh,v 1.10 2004/07/21 06:31:20 pklaus Exp $
 *
 * SelectableItem.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef _SELECTABLEITEM_HH
#  define _SELECTABLEITEM_HH
#include "gdk--.h"
#include "GDResource.hh"
#include "BoundingBox.hh"
#include <vector>

class BezierNode;
class TransformationMatrix;
class DocWindow;
class GDDocument;

class Intersection {
public:
	BezierNode *nodeA;
	gdouble splitRatioA;
	position intersectPos;
	BezierNode *nodeB;
	gdouble splitRatioB;
};
class CurveSegment {
public:
	BezierNode *origNode;
	gdouble segmentSize;
	gdouble splitbase;
	position oc1;
	position ha1;
	position ha2;
	position oc2;
	gboolean isLine;

	CurveSegment() { isLine = false; };
};

typedef vector<Intersection> IntersectionVector;


class SelectableItem {
private:
	SelectableItem *transformSource;
	TransformationMatrix nextTransformation;
	gboolean nextIncFill;

public:
	virtual void startSelectionMovement(DocWindow *docwin, Gdk_GC& context) {};
	virtual void endSelectionMovement(DocWindow *docwin, Gdk_GC& context) {};
	virtual void drawSelection(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea) = 0;
	virtual void redrawSelection(DocWindow *docwin, Gdk_GC& context, RectangleArea& drawArea);
	virtual void transformByMatrix(TransformationMatrix& m, gboolean incFillPattern = TRUE) = 0;
	virtual void setFillMatrix(TransformationMatrix& m) = 0;
	virtual SelectableItem *clone() = 0;
	virtual void setCoords(SelectableItem *from) = 0;
	virtual BezierNode *prevNode() { return NULL; }; // only makes sense in case of a BezierNode
	virtual BezierNode *nextNode() { return NULL; }; // only makes sense in case of a BezierNode
	virtual GObject *getGObject() { return (GObject *) this; }; // only BezierNode will override this
	virtual gchar* identify() = 0;
	virtual void clear() = 0;
	virtual gboolean getCoordinates(gdouble& x, gdouble&y) { return 0; };
	virtual gboolean selected() = 0;
	virtual void select(gboolean shiftSelect) = 0;
	virtual void deSelect() = 0;
	virtual void selectPath(gboolean shiftSelect) { this->select(shiftSelect); };
	virtual void deSelectPath() { deSelect(); };
	virtual void selectObject(gboolean shiftSelect) { selectPath(shiftSelect); };
	virtual void deSelectObject() { deSelectPath(); };
	virtual BoundingBox getBoundingBox() = 0;
	virtual void setDocument(GDDocument *document) {};
	virtual void gatherResource(GDResourceList& collection, gint type) {};
	virtual ~SelectableItem();
};

#endif
