/* $Id: TMatrixTool.cc,v 1.10 2004/06/19 16:27:26 pklaus Exp $
 *
 * TMatrixTool.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <math.h>
#include "../pixmaps/rectangle.xpm"
#include "../cursors/crosshair.xbm"
#include "../cursors/crosshair_mask.xbm"
#include "../cursors/move.xbm"
#include "../cursors/move_mask.xbm"
#include "../cursors/move_magnet.xbm"
#include "../cursors/move_magnet_mask.xbm"
#include "TMatrixTool.hh"
#include "SelectionItemList.hh"
#include "CursorFactory.hh"
#include "GDDocument.hh"
#include "DocWindow.hh"
#include "TransformAction.hh"

TMatrixTool::TMatrixTool() {
	pixmapData = rectangle_xpm;
	toolName = "TMatrixTool";
	nodeCaught = false;
	CursorFactory::createCursorFromData(crosshairCursor,
					crosshair_bits, crosshair_mask_bits,
					crosshair_width, crosshair_height,
					crosshair_x_hot, crosshair_y_hot);
	CursorFactory::createCursorFromData(moveCursor,
					move_bits, move_mask_bits,
					move_width, move_height,
					move_x_hot, move_y_hot);
	CursorFactory::createCursorFromData(moveMagnetCursor,
					move_magnet_bits, move_magnet_mask_bits,
					move_magnet_width, move_magnet_height,
					move_magnet_x_hot, move_magnet_y_hot);
}

void TMatrixTool::activate() {
	Tool::activate();
	mode = 0;
}

void TMatrixTool::deactivate() {
	Tool::deactivate();
}

gint TMatrixTool::buttonDown(GdkEventButton *ev, DocWindow *docwin) {
	GDDocument	*doc = docwin->doc;
	//g_printerr("TMatrixTool::buttonDown: doc=%lX, button=%d, state=%d\n", doc, ev->button, ev->state);
	downButton = ev->button;
	if (downButton == 1) {
		if (ev->state & GDK_MOD1_MASK) {
			//g_printerr("TMatrixTool::buttonDown showing Dialog \n");
			mode = TMATRIX_MODE_DIALOG;
			cx = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
			cy = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
			magnet(cx, cy, docwin);
			selection = new SelectionItemList();
			doc->getSelection(*selection);
			dialogDoc = doc;
			showValueDialog();
		} else {			
			if (mode == TMATRIX_MODE_CENTER_SET) {
				sx = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
				sy = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
				magnet(sx, sy, docwin);
				TransformationMatrix tm = calcMatrix(sx, sy, cx, cy, sx, sy, ev->state);
				selection->transformAndDraw(docwin, doc->context, tm);
			} else {
				cx = ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x;
				cy = ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y;
				magnet(cx, cy, docwin);
				selection = new SelectionItemList();
				doc->getSelection(*selection);
				doc->context.set_function(GDK_INVERT);
				selection->startTransformation(docwin, doc->context);
			}
		}
	} else {
		Tool::buttonDown(ev, docwin);
	}
	return 0;
}

gint TMatrixTool::motion(GdkEventMotion *ev, DocWindow *docwin) {
	if (ev->state & GDK_BUTTON1_MASK) {
		if (mode == TMATRIX_MODE_CENTER_SET) {
			GDDocument	*doc = docwin->doc;
			doc->context.set_function(GDK_INVERT);
			TransformationMatrix tm = calcMatrix(sx, sy, cx, cy,
					ev->x / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.x,
					ev->y / docwin->osBuffer.scaleFactor + docwin->osBuffer.origin.y,
					ev->state);
			//tm.printMatrix();
			selection->transformAndDraw(docwin, doc->context, tm);
		}
	} else if (ev->state & (GDK_BUTTON2_MASK | GDK_BUTTON3_MASK)) {
		Tool::motion(ev, docwin);
	} else {
		nodeCaught = false;
		setCursor(ev->state, docwin);
	}
	
  return 0;
}

gint TMatrixTool::buttonUp(GdkEventButton *ev, DocWindow *docwin) {
	if (downButton == 1) {
		GDDocument *doc = docwin->doc;
		if (mode == TMATRIX_MODE_CENTER_SET) {
			if (ev->state & GDK_MOD1_MASK) {
				TransformationMatrix lastTransform = selection->getLastTransformation();
				TransformAction *newAction = new TransformAction(lastTransform, true);
				doc->setAction(newAction);
				doc->deSelectAll();
				selection->insertMoving(doc);
			} else {
				TransformationMatrix lastTransform = selection->getLastTransformation();
				TransformAction *newAction = new TransformAction(lastTransform);
				doc->setAction(newAction);
				selection->applyToOriginal();
			}
			selection->endTransformation(docwin, doc->context);
			delete selection;
			selection = NULL;
			doc->redraw();
			mode = 0;
		} else if (mode == 0) {
			mode = TMATRIX_MODE_CENTER_SET;
		} else {
			mode = 0;
		}
	} else {
		mode = 0;
		Tool::buttonUp(ev, docwin);
	}
	return 0;
}

void TMatrixTool::doTransform(TransformationMatrix &tm, gboolean copy, gboolean positionOnly) {
	//g_printerr("TMatrixTool::doTransform: copy: %s\n", copy ? "true" : "false");
	//tm.printMatrix();
	selection->transformByMatrix(tm, !positionOnly);
	if (copy) {
		TransformationMatrix lastTransform = selection->getLastTransformation();
		TransformAction *newAction = new TransformAction(lastTransform, true);
		dialogDoc->setAction(newAction);
		dialogDoc->deSelectAll();
		selection->insertMoving(dialogDoc);
	} else {
		TransformationMatrix lastTransform = selection->getLastTransformation();
		TransformAction *newAction = new TransformAction(lastTransform);
		dialogDoc->setAction(newAction);
		selection->applyToOriginal();
	}
	delete selection;
	selection = NULL;
	dialogDoc->redraw();
	dialogDoc = NULL;
	mode = 0;
}

gint TMatrixTool::showOptions() {
	return 0;
}

void TMatrixTool::setCursor(guint state, DocWindow *docwin) {
	if (moving) {
		if (nodeCaught) {
			// set the cursor
			docwin->DocWinDrawingArea->get_window().set_cursor(moveMagnetCursor);
		} else {
			// set the cursor
			docwin->DocWinDrawingArea->get_window().set_cursor(moveCursor);
		}
	} else {
		if (state & GDK_MOD1_MASK) {
			docwin->DocWinDrawingArea->get_window().set_cursor(crosshairCursor);
		} else {
			docwin->DocWinDrawingArea->get_window().set_cursor(crosshairCursor);
		}
	}
}
