/* $Id: TextProperties.cc,v 1.14 2003/11/16 18:26:53 pklaus Exp $
 *
 * TextProperties.cc
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "gestalter.h"
#include "TextProperties.hh"
#include "FontPool.hh"
#include "GDFont.hh"
#include "GDDocument.hh"
#include "SelectionList.hh"
#include "GDApplication.hh"
#include "TextStyle.hh"
#include "TextAction.hh"
#include <gtk--/menu.h>
#include <gtk--/base.h>

#include <libgnome/libgnome.h>
#include <gdk/gdkkeysyms.h>
#include <gtk--/accelgroup.h>
#include <gtk--/label.h>
#include <gtk--/box.h>
#include <gtk--/adjustment.h>

TextProperties::TextProperties(GDApplication *gda) : Gtk::Window(GTK_WINDOW_TOPLEVEL) {
	application = gda;

// from TextProperties_glade
	Gtk::AccelGroup *TextProperties_accgrp = Gtk::AccelGroup::create();
	_data = new GlademmData(TextProperties_accgrp);

	Gtk::Label *FamilyLabel = manage(new class Gtk::Label(_("Family")));
	FamilyCombo = manage(new class Gtk::Combo());

	Gtk::HBox *hbox9 = manage(new class Gtk::HBox(false, 0));
	Gtk::Label *WeightLabel = manage(new class Gtk::Label(_("Weight/Slant")));
	FaceCombo = manage(new class Gtk::Combo());

	Gtk::HBox *hbox10 = manage(new class Gtk::HBox(false, 0));
	Gtk::HBox *hbox8 = manage(new class Gtk::HBox(false, 0));
	Gtk::Label *SizeLabel = manage(new class Gtk::Label(_("Text Size")));
	Gtk::Adjustment *TextSizeSpinButton_adj = manage(new class Gtk::Adjustment(12, 0.01, 1000, 1, 10, 10));
	TextSizeSpinButton = manage(new class UnitsSpinButton(*TextSizeSpinButton_adj, 1, 2));

	Gtk::HBox *hbox12 = manage(new class Gtk::HBox(false, 0));
	Gtk::Label *LineSpaceLabel = manage(new class Gtk::Label(_("Line Space")));
	Gtk::Adjustment *LineSpaceSpinButton_adj = manage(new class Gtk::Adjustment(16, 0.03, 1100, 1, 10, 10));
	LineSpaceSpinButton = manage(new class UnitsSpinButton(*LineSpaceSpinButton_adj, 1, 2));

	Gtk::HBox *hbox13 = manage(new class Gtk::HBox(false, 0));
	AutoLineSpaceCheckButton = manage(new class Gtk::CheckButton(_("Auto Line Space"), 0, 0.5));

	Gtk::HBox *hbox11 = manage(new class Gtk::HBox(false, 0));
	Gtk::Label *label17 = manage(new class Gtk::Label(_("Base Line Shift")));
	Gtk::Adjustment *BaseLineShiftSpinButton_adj = manage(new class Gtk::Adjustment(0, -10000, 10000, 1, 10, 10));
	BaseLineShiftSpinButton = manage(new class UnitsSpinButton(*BaseLineShiftSpinButton_adj, 1, 2));

	Gtk::HBox *hbox15 = manage(new class Gtk::HBox(false, 0));
	Gtk::Label *TrackingLabel = manage(new class Gtk::Label(_("Tracking (1/1000 EM)")));
	Gtk::Adjustment *TrackingSpinButton_adj = manage(new class Gtk::Adjustment(0, -10000, 1000, 1, 10, 10));
	TrackingSpinButton = manage(new class PropertiesSpinButton(*TrackingSpinButton_adj, 1, 0));

	Gtk::HBox *hbox16 = manage(new class Gtk::HBox(false, 0));
	Gtk::Label *HorizontalScaleLabel = manage(new class Gtk::Label(_("Scale Horizontal")));
	Gtk::Adjustment *HorizontalScaleSpinButton_adj = manage(new class Gtk::Adjustment(100, 0.1, 5000, 1, 10, 10));
	HorizontalScaleSpinButton = manage(new class PropertiesSpinButton(*HorizontalScaleSpinButton_adj, 1, 1));

	Gtk::Label *PercenLabel = manage(new class Gtk::Label(_("%")));
	Gtk::HBox *hbox17 = manage(new class Gtk::HBox(false, 0));
	Gtk::HBox *hbox14 = manage(new class Gtk::HBox(false, 0));
	Gtk::VBox *vbox5 = manage(new class Gtk::VBox(false, 0));
	FamilyLabel->set_name("FamilyLabel");
	FamilyLabel->set_alignment(0.5,0.5);
	FamilyLabel->set_padding(5,0);
	FamilyLabel->set_justify(GTK_JUSTIFY_CENTER);
	FamilyLabel->set_line_wrap(false);
	FamilyCombo->get_entry()->set_name("FamilyEntry");
	FamilyCombo->get_entry()->set_flags(GTK_CAN_FOCUS);
	FamilyCombo->get_entry()->set_visibility(true);
	FamilyCombo->get_entry()->set_editable(false);
	FamilyCombo->get_entry()->set_text(_(""));
	FamilyCombo->set_name("FamilyCombo");
	FamilyCombo->set_case_sensitive(false);
	FamilyCombo->set_use_arrows(true);
	FamilyCombo->set_use_arrows_always(false);
	{  

		const char * const items[] = { N_(""), 0 };
		FamilyCombo->set_popdown_strings(items);
	}
	FamilyCombo->get_entry()->set_text(_(""));
	hbox9->set_name("hbox9");
	hbox9->set_border_width(10);
	hbox9->pack_start(*FamilyLabel, false, false, 0);
	hbox9->pack_start(*FamilyCombo, false, false, 0);
	WeightLabel->set_name("WeightLabel");
	WeightLabel->set_alignment(0.5,0.5);
	WeightLabel->set_padding(5,0);
	WeightLabel->set_justify(GTK_JUSTIFY_CENTER);
	WeightLabel->set_line_wrap(false);
	FaceCombo->get_entry()->set_name("FaceEntry");
	FaceCombo->get_entry()->set_flags(GTK_CAN_FOCUS);
	FaceCombo->get_entry()->set_visibility(true);
	FaceCombo->get_entry()->set_editable(false);
	FaceCombo->get_entry()->set_text(_(""));
	FaceCombo->set_name("FaceCombo");
	FaceCombo->set_case_sensitive(false);
	FaceCombo->set_use_arrows(true);
	FaceCombo->set_use_arrows_always(false);
	{  

		const char * const items[] = { N_(""), 0 };
		FaceCombo->set_popdown_strings(items);
	}
	FaceCombo->get_entry()->set_text(_(""));
	hbox10->set_name("hbox10");
	hbox10->set_border_width(10);
	hbox10->pack_start(*WeightLabel, false, false, 0);
	hbox10->pack_start(*FaceCombo, false, false, 0);
	hbox8->set_name("hbox8");
	hbox8->pack_start(*hbox9);
	hbox8->pack_start(*hbox10);
	SizeLabel->set_name("SizeLabel");
	SizeLabel->set_alignment(0.5,0.5);
	SizeLabel->set_padding(5,0);
	SizeLabel->set_justify(GTK_JUSTIFY_CENTER);
	SizeLabel->set_line_wrap(false);
	TextSizeSpinButton->set_name("TextSizeSpinButton");
	TextSizeSpinButton->set_flags(GTK_CAN_FOCUS);
	TextSizeSpinButton->set_update_policy(GTK_UPDATE_ALWAYS);
	TextSizeSpinButton->set_numeric(false);
	TextSizeSpinButton->set_digits(2);
	TextSizeSpinButton->set_wrap(false);
	TextSizeSpinButton->set_snap_to_ticks(false);
	hbox12->set_name("hbox12");
	hbox12->set_border_width(10);
	hbox12->pack_start(*SizeLabel, false, false, 0);
	hbox12->pack_start(*TextSizeSpinButton);
	LineSpaceLabel->set_name("LineSpaceLabel");
	LineSpaceLabel->set_sensitive(false);
	LineSpaceLabel->set_alignment(0.5,0.5);
	LineSpaceLabel->set_padding(5,0);
	LineSpaceLabel->set_justify(GTK_JUSTIFY_CENTER);
	LineSpaceLabel->set_line_wrap(false);
	LineSpaceSpinButton->set_name("LineSpaceSpinButton");
	LineSpaceSpinButton->set_sensitive(false);
	LineSpaceSpinButton->set_flags(GTK_CAN_FOCUS);
	LineSpaceSpinButton->set_update_policy(GTK_UPDATE_ALWAYS);
	LineSpaceSpinButton->set_numeric(false);
	LineSpaceSpinButton->set_digits(2);
	LineSpaceSpinButton->set_wrap(false);
	LineSpaceSpinButton->set_snap_to_ticks(false);
	hbox13->set_name("hbox13");
	hbox13->set_border_width(10);
	hbox13->pack_start(*LineSpaceLabel, false, false, 0);
	hbox13->pack_start(*LineSpaceSpinButton);
	AutoLineSpaceCheckButton->set_name("AutoLineSpaceCheckButton");
	AutoLineSpaceCheckButton->set_flags(GTK_CAN_FOCUS);
	AutoLineSpaceCheckButton->set_border_width(10);
	AutoLineSpaceCheckButton->set_mode(true);
	AutoLineSpaceCheckButton->set_active(true);
	hbox11->set_name("hbox11");
	hbox11->pack_start(*hbox12);
	hbox11->pack_start(*hbox13);
	hbox11->pack_start(*AutoLineSpaceCheckButton, false, false, 0);
	label17->set_name("label17");
	label17->set_alignment(0,0.5);
	label17->set_padding(5,0);
	label17->set_justify(GTK_JUSTIFY_LEFT);
	label17->set_line_wrap(false);
	BaseLineShiftSpinButton->set_usize(55,-1);
	BaseLineShiftSpinButton->set_name("BaseLineShiftSpinButton");
	BaseLineShiftSpinButton->set_flags(GTK_CAN_FOCUS);
	BaseLineShiftSpinButton->set_update_policy(GTK_UPDATE_ALWAYS);
	BaseLineShiftSpinButton->set_numeric(false);
	BaseLineShiftSpinButton->set_digits(2);
	BaseLineShiftSpinButton->set_wrap(false);
	BaseLineShiftSpinButton->set_snap_to_ticks(false);
	hbox15->set_name("hbox15");
	hbox15->set_border_width(8);
	hbox15->pack_start(*label17, false, false, 0);
	hbox15->pack_start(*BaseLineShiftSpinButton);
	TrackingLabel->set_name("TrackingLabel");
	TrackingLabel->set_alignment(0.5,0.5);
	TrackingLabel->set_padding(5,0);
	TrackingLabel->set_justify(GTK_JUSTIFY_CENTER);
	TrackingLabel->set_line_wrap(false);
	TrackingSpinButton->set_name("TrackingSpinButton");
	TrackingSpinButton->set_flags(GTK_CAN_FOCUS);
	TrackingSpinButton->set_update_policy(GTK_UPDATE_ALWAYS);
	TrackingSpinButton->set_numeric(false);
	TrackingSpinButton->set_digits(0);
	TrackingSpinButton->set_wrap(false);
	TrackingSpinButton->set_snap_to_ticks(false);
	hbox16->set_name("hbox16");
	hbox16->set_border_width(10);
	hbox16->pack_start(*TrackingLabel, false, false, 0);
	hbox16->pack_start(*TrackingSpinButton);
	HorizontalScaleLabel->set_name("HorizontalScaleLabel");
	HorizontalScaleLabel->set_alignment(0.5,0.5);
	HorizontalScaleLabel->set_padding(5,0);
	HorizontalScaleLabel->set_justify(GTK_JUSTIFY_CENTER);
	HorizontalScaleLabel->set_line_wrap(false);
	HorizontalScaleSpinButton->set_usize(55,-1);
	HorizontalScaleSpinButton->set_name("HorizontalScaleSpinButton");
	HorizontalScaleSpinButton->set_flags(GTK_CAN_FOCUS);
	HorizontalScaleSpinButton->set_update_policy(GTK_UPDATE_ALWAYS);
	HorizontalScaleSpinButton->set_numeric(false);
	HorizontalScaleSpinButton->set_digits(1);
	HorizontalScaleSpinButton->set_wrap(false);
	HorizontalScaleSpinButton->set_snap_to_ticks(false);
	PercenLabel->set_name("PercenLabel");
	PercenLabel->set_alignment(0.5,0.5);
	PercenLabel->set_padding(0,0);
	PercenLabel->set_justify(GTK_JUSTIFY_CENTER);
	PercenLabel->set_line_wrap(false);
	hbox17->set_name("hbox17");
	hbox17->set_border_width(10);
	hbox17->pack_start(*HorizontalScaleLabel, false, false, 0);
	hbox17->pack_start(*HorizontalScaleSpinButton);
	hbox17->pack_start(*PercenLabel, false, false, 0);
	hbox14->set_name("hbox14");
	hbox14->pack_start(*hbox15);
	hbox14->pack_start(*hbox16);
	hbox14->pack_start(*hbox17);
	vbox5->set_name("vbox5");
	vbox5->pack_start(*hbox8, false, false, 0);
	vbox5->pack_start(*hbox11);
	vbox5->pack_start(*hbox14);
	this->set_name("TextProperties");
	this->set_border_width(5);
	this->set_title(_("Text Properties"));
	this->set_modal(false);
	this->set_position(GTK_WIN_POS_NONE);
	this->add_accel_group(*(_data->getAccelGroup()));
	this->add(*vbox5);
	FamilyLabel->show();
	FamilyCombo->show();
	hbox9->show();
	WeightLabel->show();
	FaceCombo->show();
	hbox10->show();
	hbox8->show();
	SizeLabel->show();
	TextSizeSpinButton->show();
	hbox12->show();
	LineSpaceLabel->show();
	LineSpaceSpinButton->show();
	hbox13->show();
	AutoLineSpaceCheckButton->show();
	hbox11->show();
	label17->show();
	BaseLineShiftSpinButton->show();
	hbox15->show();
	TrackingLabel->show();
	TrackingSpinButton->show();
	hbox16->show();
	HorizontalScaleLabel->show();
	HorizontalScaleSpinButton->show();
	PercenLabel->show();
	hbox17->show();
	hbox14->show();
	vbox5->show();
	FamilyCombo->get_entry()->delete_text.connect(SigC::slot(this, &TextProperties::checkFamilyDelete));
	FamilyCombo->get_entry()->insert_text.connect(SigC::slot(this, &TextProperties::checkFamilyInsert));
	FamilyCombo->get_entry()->changed.connect(SigC::slot(this, &TextProperties::setFamily));
	FamilyCombo->get_entry()->key_press_event.connect(SigC::slot(this, &TextProperties::keyPressFamily));
	FaceCombo->get_entry()->delete_text.connect(SigC::slot(this, &TextProperties::checkFaceDelete));
	FaceCombo->get_entry()->insert_text.connect(SigC::slot(this, &TextProperties::checkFaceInsert));
	FaceCombo->get_entry()->changed.connect(SigC::slot(this, &TextProperties::setFace));
	TextSizeSpinButton->changed.connect(SigC::slot(this, &TextProperties::changeTextSize));
	LineSpaceSpinButton->changed.connect(SigC::slot(this, &TextProperties::changeLineSpace));
	AutoLineSpaceCheckButton->toggled.connect(SigC::slot(this, &TextProperties::on_AutoLineSpaceCheckButton_toggled));
	BaseLineShiftSpinButton->changed.connect(SigC::slot(this, &TextProperties::changeBaseLineShift));
	TrackingSpinButton->changed.connect(SigC::slot(this, &TextProperties::changeTracking));
	HorizontalScaleSpinButton->changed.connect(SigC::slot(this, &TextProperties::changeHorizontalScale));
	delete_event.connect(SigC::slot(this, &TextProperties::hideTextProperties));
// end from TextProperties_glade

	createFamilyMenu();
	familyDisabled = false;
	faceDisabled = false;
	currentDoc = NULL;
	updateAllowed = true;
	
	gda->prefsChanged.connect(SigC::slot(this, &TextProperties::onPrefsChanged));

}

void TextProperties::onPrefsChanged(Preferences *newPrefs) {
	createFamilyMenu();
}

gint TextProperties::keyPressFamily(GdkEventKey *ev) {
	//g_printerr("TextProperties::keyPressFamily keyval: %d, length: %d, str: %s\n", ev->keyval, ev->length, ev->string);
	return 0;
}

void TextProperties::checkFamilyDelete(gint start_pos, gint end_pos) {
	//g_printerr("TextProperties::checkFamilyDelete: start: %d, end: %d\n", start_pos, end_pos);
}

void TextProperties::checkFamilyInsert(const gchar *text, gint length, gint *position) {
	//g_printerr("TextProperties::checkFamilyInsert: text: %s, len: %d, pos: %d\n", text, length, *position);
}

void TextProperties::checkFaceDelete(gint start_pos, gint end_pos) {
	//g_printerr("TextProperties::checkFaceDelete: start: %d, end: %d\n", start_pos, end_pos);
}

void TextProperties::checkFaceInsert(const gchar *text, gint length, gint *position) {
	//g_printerr("TextProperties::checkFaceInsert: text: %s, len: %d, pos: %d\n", text, length, *position);
}

void TextProperties::changeTextSize() {
	gdouble newSize = TextSizeSpinButton->get_value_as_float();
	//g_printerr("TextProperties::changeTextSize: %f\n", newSize);
// 	if (newSize < 1) {
// 		string test = TextSizeSpinButton->get_text();
// 		string lstext = LineSpaceSpinButton->get_text();
// 		string blstext = BaseLineShiftSpinButton->get_text();
// 		string trtext = TrackingSpinButton->get_text();
// 		string hstext = HorizontalScaleSpinButton->get_text();
// 		string font = FamilyCombo->get_entry()->get_text();
// 		g_printerr("Something wrong s: %s ls: %s bls: %s tr: %s hs: %s font: %s\n", test.c_str(), 
// 				lstext.c_str(), blstext.c_str(), trtext.c_str(), hstext.c_str(),
// 				font.c_str());
// 	}
	if (updateAllowed && currentDoc && currentDoc->selection) {
		//g_printerr("TextProperties::changeTextSize updating current selection\n");
		TextStyle style;
		style.setPointSize(newSize);
		style.setValueMask(TEXT_STYLE_POINT_SIZE_MASK);
		TextAction *newAction = new TextAction(style);
		currentDoc->setAction(newAction);
		currentDoc->currentTextStyle |= style;
		currentDoc->selection->setTextStyle(style);
		currentDoc->redraw();
		if (AutoLineSpaceCheckButton->get_active()) {
			fillLineSpace(1.2 * newSize, true);
		}
	}
}

void TextProperties::changeLineSpace() {
	//g_printerr("TextProperties::changeLineSpace\n");
	gdouble newLineSpace = LineSpaceSpinButton->get_value_as_float();
	if (updateAllowed && currentDoc && currentDoc->selection) {
		TextStyle style;
		style.setLineSpace(newLineSpace);
		style.setValueMask(TEXT_STYLE_LINE_SPACE_MASK);
		TextAction *newAction = new TextAction(style);
		currentDoc->setAction(newAction);
		currentDoc->currentTextStyle |= style;
		currentDoc->selection->setTextStyle(style);
		currentDoc->redraw();
	}
}

void TextProperties::on_AutoLineSpaceCheckButton_toggled() {
	//g_printerr("TextProperties::on_AutoLineSpaceCheckButton_toggled\n");
	gboolean autoLineSpace = AutoLineSpaceCheckButton->get_active();
	LineSpaceSpinButton->set_sensitive(!autoLineSpace);
	if (updateAllowed && currentDoc && currentDoc->selection) {
		TextStyle style;
		style.setAutoLineSpace(autoLineSpace);
		style.setValueMask(TEXT_STYLE_AUTO_LINE_SPACE_MASK);
		TextAction *newAction = new TextAction(style);
		currentDoc->setAction(newAction);
		currentDoc->currentTextStyle |= style;
		currentDoc->selection->setTextStyle(style);
		currentDoc->redraw();
	}
}

void TextProperties::changeBaseLineShift() {
	//g_printerr("TextProperties::changeBaseLineShift\n");
	gdouble newShift = BaseLineShiftSpinButton->get_value_as_float();
	if (updateAllowed && currentDoc && currentDoc->selection) {
		TextStyle style;
		style.setBaseLineShift(newShift);
		style.setValueMask(TEXT_STYLE_BASE_LINE_SHIFT_MASK);
		TextAction *newAction = new TextAction(style);
		currentDoc->setAction(newAction);
		currentDoc->currentTextStyle |= style;
		currentDoc->selection->setTextStyle(style);
		currentDoc->redraw();
	}
}

void TextProperties::changeTracking() {
	//g_printerr("TextProperties::changeTracking\n");
	gdouble newTracking = TrackingSpinButton->get_value_as_float();
	if (updateAllowed && currentDoc && currentDoc->selection) {
		TextStyle style;
		style.setTracking(newTracking);
		style.setValueMask(TEXT_STYLE_TRACKING_MASK);
		TextAction *newAction = new TextAction(style);
		currentDoc->setAction(newAction);
		currentDoc->currentTextStyle |= style;
		currentDoc->selection->setTextStyle(style);
		currentDoc->redraw();
	}
}

void TextProperties::changeHorizontalScale() {
	//g_printerr("TextProperties::changeHorizontalScale\n");
	gdouble newScale = HorizontalScaleSpinButton->get_value_as_float();
	newScale /= 100; // the dialog specifies the value in percent
	if (updateAllowed && currentDoc && currentDoc->selection) {
		TextStyle style;
		style.setHorizontalScale(newScale);
		style.setValueMask(TEXT_STYLE_HORIZONTAL_SCALE_MASK);
		TextAction *newAction = new TextAction(style);
		currentDoc->setAction(newAction);
		currentDoc->currentTextStyle |= style;
		currentDoc->selection->setTextStyle(style);
		currentDoc->redraw();
	}
}

void TextProperties::setDocument(GDDocument *document) {
	//g_printerr("TextProperties::setDocument %lX\n", document);
	if (currentDoc != document) {
		currentDoc = document;
		if (selConnection.connected()) {
			selConnection.disconnect();
		}
		if (document) {
			document->selection->fillTextProperties(this);
			selConnection = document->selection->changed.connect(SigC::slot(this, &TextProperties::updateValues));
		}
	}
}

void TextProperties::createFamilyMenu() {
	FamilyMap families = application->fontPool->getFamilies();
	vector<string> vec;
	for (FamilyIterator fmit = families.begin(); fmit != families.end(); fmit++) {
		vec.insert(vec.end(), fmit->first);
	}
	Gtk::SArray sarray(vec);
	FamilyCombo->set_popdown_strings(sarray);
}

void TextProperties::setFamily() {
	//g_printerr("TextProperties::setFamily\n");
	if (familyDisabled) {
		return;
	}
	string family = FamilyCombo->get_entry()->get_text();
	//g_printerr("TextProperties::setFamily family: %s\n", family.c_str());
	FamilyMap families = application->fontPool->getFamilies();
	FamilyIterator fmit = families.find(family.c_str());
	int cursor = FamilyCombo->get_entry()->get_position();
	if (family.size()) {
		cursor++;
	}
	//g_printerr("TextProperties::setFamily cursor: %d\n", cursor);
	familyDisabled = true;
	if (fmit == families.end()) {
		int res = -1;
		for (fmit = families.begin(); fmit != families.end() && res < 0; fmit++) {
			res = strncasecmp(fmit->first.c_str(), family.c_str(), cursor);
			if (res >= 0) {
				fmit--;
			}
		}
		if (fmit != families.end()) {
			family = string(fmit->first);
			/* familyDisabled = true;
			FamilyCombo->get_entry()->set_text(family);
			if (cursor < family.size()) {
				FamilyCombo->get_entry()->set_position(cursor - 1);
			}
			//familyDisabled = false; */
			currentFamily = fmit->second;
			createFaceMenu();
		}
	} else {
		currentFamily = fmit->second;
		createFaceMenu();
	}
	familyDisabled = false;
	//g_printerr("TextProperties::setFamily end\n");
}

void TextProperties::createFaceMenu() {
	//g_printerr("TextProperties::createFaceMenu: \n");
	if (currentFamily.begin() == currentFamily.end()) {
		g_printerr("currentFamily has no members\n");
	}
	vector<string> vec;
	for (FontIterator fit = currentFamily.begin(); fit != currentFamily.end(); fit++) {
		vec.insert(vec.end(), fit->first);
		//g_printerr("TextProperties::createFaceMenu: insert %s\n", fit->first);
	}
	Gtk::SArray sarray(vec);
	FaceCombo->set_popdown_strings(sarray);
}

void TextProperties::setFace() {
	//g_printerr("TextProperties::setFace\n");
	if (faceDisabled) return;
	string face = FaceCombo->get_entry()->get_text();
	//g_printerr("TextProperties::setFace face: %s\n", face.c_str());
	FontIterator fit = currentFamily.find(face.c_str());
	int cursor = FaceCombo->get_entry()->get_position();
	if (face.size()) {
		cursor++;
	}
	if (fit == currentFamily.end()) {
		int res = -1;
		for (fit = currentFamily.begin(); fit != currentFamily.end() && res < 0; fit++) {
			res = strncasecmp(fit->first.c_str(), face.c_str(), cursor);
			if (res >= 0) {
				fit--;
			}
		}
		if (fit != currentFamily.end()) {
			face = string(fit->first);
			faceDisabled = true;
			/*FaceCombo->get_entry()->set_text(face);
			if (cursor < face.size()) {
				FaceCombo->get_entry()->set_position(cursor - 1);
			}*/
			faceDisabled = false;
			currentFace = fit->second;
			setTextFont();
		}
	} else {
		currentFace = fit->second;
		setTextFont();
	}
}

void TextProperties::setTextFont() {
	if (updateAllowed && currentDoc && currentDoc->selection) {
		TextStyle style;
		style.setFont(currentFace);
		style.setValueMask(TEXT_STYLE_FONT_MASK);
		TextAction *newAction = new TextAction(style);
		currentDoc->setAction(newAction);
		currentDoc->currentTextStyle |= style;
		currentDoc->selection->setTextStyle(style);
		currentDoc->redraw();
	}
}

void TextProperties::fillFont(GDFont *font, gboolean valid) {
	updateAllowed = false;
	familyDisabled = true;
	faceDisabled = true;
	if (valid && font) {
		FamilyCombo->get_entry()->set_text(font->getFamily());
		FaceCombo->get_entry()->set_text(font->getFaceName());
	} else {
		FamilyCombo->get_entry()->set_text("");
		FaceCombo->get_entry()->set_text("");
	}
	faceDisabled = false;
	familyDisabled = false;
	updateAllowed = true;
}

void TextProperties::fillPointSize(gdouble size, gboolean valid) {
	//g_printerr("TextProperties::fillPointSize size: %f (%c)\n", size, valid ? 't' : 'f');
	updateAllowed = false;
	if (valid) {
		TextSizeSpinButton->set_value(size);
	} else {
		TextSizeSpinButton->set_text("");
	}
	updateAllowed = true;
	//g_printerr("TextProperties::fillPointSize END size: %f (%c)\n", size, valid ? 't' : 'f');
}

void TextProperties::fillLineSpace(gdouble space, gboolean valid) {
	updateAllowed = false;
	if (valid) {
		LineSpaceSpinButton->set_value(space);
	} else {
		LineSpaceSpinButton->set_text("");
	}
	updateAllowed = true;
}

void TextProperties::fillAutoLineSpace(gboolean autoLS, gboolean valid) {
	updateAllowed = false;
	if (valid) {
		AutoLineSpaceCheckButton->set_active(autoLS);
	} else {
		AutoLineSpaceCheckButton->set_active(false);
	}
	updateAllowed = true;
}

void TextProperties::fillBaseLineShift(gdouble shift, gboolean valid) {
	updateAllowed = false;
	if (valid) {
		BaseLineShiftSpinButton->set_value(shift);
	} else {
		BaseLineShiftSpinButton->set_text("");
	}
	updateAllowed = true;
}

void TextProperties::fillTracking(gdouble tracking, gboolean valid) {
	updateAllowed = false;
	if (valid) {
		TrackingSpinButton->set_value(tracking);
	} else {
		TrackingSpinButton->set_text("");
	}
	updateAllowed = true;
}

void TextProperties::fillHorizontalScale(gdouble scale, gboolean valid) {
	updateAllowed = false;
	if (valid) {
		HorizontalScaleSpinButton->set_value(scale * 100);
	} else {
		HorizontalScaleSpinButton->set_text("");
	}
	updateAllowed = true;
}

void TextProperties::updateValues(GDDocument *document) {
	//g_printerr("TextProperties::updateValues doc=%lX\n", document);
	document->selection->fillTextProperties(this);
}

gint TextProperties::sbFocusOut(GdkEventFocus *evt) {
	//g_printerr("TextProperties::sbFocusOut\n");
// 	string test = TextSizeSpinButton->get_text();
// 	string lstext = LineSpaceSpinButton->get_text();
// 	string blstext = BaseLineShiftSpinButton->get_text();
// 	string trtext = TrackingSpinButton->get_text();
// 	string hstext = HorizontalScaleSpinButton->get_text();
// 	string font = FamilyCombo->get_entry()->get_text();
// 	g_printerr("Something wrong s: %s ls: %s bls: %s tr: %s hs: %s font: %s\n", test.c_str(), 
// 			lstext.c_str(), blstext.c_str(), trtext.c_str(), hstext.c_str(),
// 			font.c_str());
	return 0;
}

gint TextProperties::hideTextProperties(GdkEventAny *ev) {
	application->hideTextProperties();
	return 1;
}

TextProperties::~TextProperties() {
	delete _data;
}
