/* $Id: TextRow.hh,v 1.15 2004/07/21 06:31:20 pklaus Exp $
 *
 * TextRow.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef _TEXTROW_HH
#  define _TEXTROW_HH

#include "gestalter.h"
#include <vector>
#include <glib--.h>
#include "GDStyle.hh"
#include "TextStyle.hh"
#include "TransformationMatrix.hh"
#include <iostream>

class GDCharacter;
class GDDocument;

class TextRow {

protected:
	vector<GDCharacter *>	chars;
	double length; // length of base line in current text size
	double descender; // height of text row in current text size
	double ascender; // max ascender of text row in current text size
	double linespace; // line spacing in points based on current text size

	void showText(ostream& psout, GDStyle& fs, GDStyle& ss, GDDocument *doc);
	
public:
	GDStyle drawStyle; // styles are used for initial style of every character
	GDStyle fillStyle;
	TextStyle defaultStyle;

	TextRow();
	TextRow(const TextRow& from); // copy constructor
	
	void insertChar(gint after, GDCharacter* ch);
	GDCharacter *removeChar(gint pos);
	void removeChars(gint startpos, gint endpos);
	GDCharacter *getChar(guint pos);
	guint size();
	
	void calcLength();
	double getWidth() { return length; };
	double getHeight() { return ascender+descender; };
	double getAscender() { return ascender; };
	double getDescender() { return descender; };
	double getLineSpace() { return linespace; };
	
	void writeXML(ostream& xmlout, gdouble x, gdouble y, GDDocument *doc);
	TextStyle *writePS(ostream& psout, gdouble x, gdouble y, TextStyle *last, GDDocument *doc);
	
	void transformByMatrix(TransformationMatrix& m, gboolean incFillPattern = TRUE);
	void setFillMatrix(TransformationMatrix& m);

	void changeDocument(GDDocument *document);
	
	~TextRow();

};


#endif
