/* $Id: TextStyle.hh,v 1.8 2003/12/06 21:42:11 pklaus Exp $
 *
 * TextStyle.hh
 *
 * Gestalter - The free vector drawing application
 * Copyright (c) 2002 Philipp Klaus <gestalter@linotux.ch>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#ifndef _TEXTSTYLE_HH
#  define _TEXTSTYLE_HH

#include "gestalter.h"
#include <glib--.h>
#include <iostream>

class GDFont;

#define		TEXT_STYLE_POINT_SIZE_MASK			(1 << 0)	/*   1 */
#define		TEXT_STYLE_LINE_SPACE_MASK			(1 << 1)	/*   2 */
#define		TEXT_STYLE_BASE_LINE_SHIFT_MASK		(1 << 2)	/*   4 */
#define		TEXT_STYLE_TRACKING_MASK			(1 << 3)	/*   8 */
#define		TEXT_STYLE_HORIZONTAL_SCALE_MASK	(1 << 4)	/*  16 */
#define		TEXT_STYLE_FONT_MASK				(1 << 5)	/*  32 */
#define		TEXT_STYLE_AUTO_LINE_SPACE_MASK		(1 << 6)	/*  64 */
#define		TEXT_STYLE_JUSTIFICATION_MASK		(1 << 7)	/* 128 */

#define		TEXT_STYLE_JUSTIFICATION_LEFT		0
#define		TEXT_STYLE_JUSTIFICATION_RIGHT		1
#define		TEXT_STYLE_JUSTIFICATION_CENTER		2
#define		TEXT_STYLE_JUSTIFICATION_BLOCK		3

class TextStyle {

protected:

	double	pointSize;
	double	lineSpace;
	double	baseLineShift;
	double	tracking;
	double	horizontalScale;
	gboolean autoLineSpace;
	GDFont	*font;
	guint	valueMask;
	guint	justification;

public:

	TextStyle();
	TextStyle(const TextStyle& from);
	TextStyle operator=(const TextStyle& from);
	int operator==(const TextStyle& from);
	TextStyle operator&(const TextStyle& b);
	void operator&=(const TextStyle& ts);
	void operator|=(const TextStyle& ts);
	
	void setFont(GDFont *font);
	GDFont *getFont() { return font; };
	
	void setPointSize(double value);
	double getPointSize() { return pointSize; };
	
	void setLineSpace(double value);
	double getLineSpace() { return lineSpace; };
	
	void setAutoLineSpace(gboolean value);
	gboolean getAutoLineSpace() { return autoLineSpace; };
	
	void setBaseLineShift(double value);
	double getBaseLineShift() { return baseLineShift; };
	
	void setTracking(double value);
	double getTracking() { return tracking; };
	
	void setHorizontalScale(double value);
	double getHorizontalScale() { return horizontalScale; };
	
	void setJustification(guint value);
	guint getJustification() { return justification; };

	guint getValueMask() { return valueMask; };
	void setValueMask(guint mask) { valueMask = mask; };

	void writeXML(ostream& xmlout);
	void writePS(ostream& psout);
	void writeDiffPS(TextStyle& old, ostream& psout);
	
	~TextStyle();
};


#endif
